/**********************************************************

 * Filename					: processgroupinfo_handler.js

 * Purpose					: Exposes Process Info API from signage

 * Author(s)				: chamoknath.choudhury@lge.com, amber.pandit@lge.com

 * Creation date			: May 02, 2016

 **********************************************************/



(function(){



	module.exports = processGroupInfo;



	var spawn = require('child_process');



	function processGroupInfo(msg, socket) {

		//Expected msg format : {"subscribe" : true/false, "interval": milliseconds}

		var processGroupInfoProtocol = socket._protocols['processgroupinfo'];

		if(processGroupInfoProtocol){

			clearInterval(processGroupInfoProtocol);

		}

		if(msg.subscribe === true){

			//Interval because subscribe is true, it will send data at each interval

			if(msg.intervalValue && !isNaN(msg.intervalValue)){

				socket._protocols['processgroupinfo'] = setInterval(function(){

					//Code for Command Execution and emit the data back to client

					getProcessGroupInfo(socket);

				}, msg.intervalValue);

			}else{

				response = {"returnValue": false, "result" : "Invalid Interval Value"};

				socket.emit('processgroupinfo', response);

			}

		} else{

			//No Interval because subscribe is false, it will send data only once

			getProcessGroupInfo(socket);

		}

	}

	function getProcessGroupInfo(socket) {

		var response = {};

		try{

			var cmd = 'date +"%s"; ps -A -m -o psr,pcpu,pmem,rss,cmd';

			var getProcessInfo = spawn.exec(cmd, function(error, stdout, stdin){

				if(error){

					response = {"returnValue": false, "result" : error};

					socket.emit('processgroupinfo', response);

				}else{

					formatProcessGrpInfo(stdout, function(result){

						response = {"returnValue": true, "result" : result};

						socket.emit('processgroupinfo', response);

					});

				}

			});

		}catch(e){

			response = {"returnValue": false, "result" : e};

			socket.emit('processgroupinfo', response);

		}

	}

	function getProcessGrp(pname) {

		var group;

		switch (pname) {

		case '/usr/bin/WebAppMgr':

		case '/usr/lib/qt5/libexec/QtWebEngineProcess':

		case '/usr/bin/QtWebProcess':

			group = 'Web App'; break;

		case 'com.lg.app.signage.signagemanagement.service':

			group = 'JS Service'; break;

		case 'com.webos.service.commercial.signage.localfileserver':

		case 'com.webos.service.commercial.signage.storageservice':

			group = 'SCAP Service'; break;

		default:

			group = 'System';

		}

		return group;

	}

	function formatProcessGrpInfo(stdout, callback) {

		var date, lines, processCmd, processCpu = 0, processMemorySize = 0, processMemoryPercent = 0,

		processCores, qtProcCount = 0, returnObj = {}, columns, processGrp, ignoreProcessLines = false,

		processGroups = ["Web App", "JS Service", "SCAP Service", "System"], processinfo = {};



		lines = stdout.split("\n");

		date = lines[0] * 1000;

		for (var i = 0, length = processGroups.length; i < length; i++) { // creating skeleton object at the beginning

			var tempObj;

			processinfo[processGroups[i]] = {};

			tempObj = processinfo[processGroups[i]];

			tempObj.pid = 4194305; // Maximum process id  + 1, currently to match the UI response format

			tempObj.cpu = 0;

			tempObj.memory = {"size": 0,"percent": 0};

			tempObj.cores = {};

			tempObj = undefined;

		}

		//Initial two lines not required, so index starts with 2

		for (var index = 2, dataLength = lines.length; index < dataLength; index++) {

			columns = lines[index].trim().split(/\s+/);

			if (columns[0].indexOf('-') == 0) {

				processCmd = columns[4];

				if (processCmd.indexOf("[") > -1 || processCmd.indexOf("ps") == 0 || processCmd.indexOf("-sh") == 0) { // ignore unwanted processes

					ignoreProcessLines = true;

					continue;

				}

				ignoreProcessLines = false;

				processGrp = getProcessGrp(processCmd);

				processCpu = parseFloat(columns[1]);

				processMemoryPercent = parseFloat(columns[2]);

				processMemorySize = parseFloat(columns[3]);

				processinfo[processGrp].cpu += processCpu;

				processinfo[processGrp].memory.size +=  processMemorySize;

				processinfo[processGrp].memory.percent +=  processMemoryPercent;

			} else {

				if (!ignoreProcessLines) {

					processCores = processinfo[processGrp].cores;

					if (processCores[columns[0]] === undefined) {

						processCores[columns[0]] = 0;

					}

					processCores[columns[0]] += parseFloat(columns[1]);

				}

			}

		}

		returnObj = {

				"date": date,

				"processinfo": processinfo,

				"qtProcCount": qtProcCount, // currently to match the UI response format

				"processList": processGroups

		};

		callback(returnObj);

	}

}());