/**********************************************************

* Filename					: processinfo_handler.js

* Purpose					: Exposes Process Info API from signage

* Author(s)					: chamoknath.choudhury@lge.com, amber.pandit@lge.com

* Creation date				: April 18, 2016

**********************************************************/



(function(){



	module.exports = processInfo;



	var spawn = require('child_process');



	function processInfo(msg, socket) {



		//Expected msg format : {"subscribe" : true/false, "interval": milliseconds}

		var processInfoProtocol = socket._protocols['processinfo'];

		if(processInfoProtocol){

			clearInterval(processInfoProtocol);

		}

		if(msg.subscribe === true){

			//Interval because subscribe is true, it will send data at each interval

			if(msg.intervalValue && !isNaN(msg.intervalValue)){

				socket._protocols['processinfo'] = setInterval(function(){

					//Code for Command Execution and emit the data back to client

					getProcessInfo(msg.processList, socket);

				}, msg.intervalValue);

			}else{

				response = {"returnValue": false, "result" : "Invalid Interval Value"};

				socket.emit('processinfo', response);

			}

		} else{

			//No Interval because subscribe is false, it will send data only once

			getProcessInfo(msg.processList, socket);

		}

	}



	function getProcessInfo(processList, socket) {

		var response = {};

		try{

			// Start

			cmd = 'ps aux --sort=-rss,pcpu| awk \'{if ($3>0 && $4>0 && $11!~/\\[/) print $2,$11}\';ps aux|grep \[Q\]tWebProcess|awk \'{print $2,$11,$12}\';';

			cmd += 'date +"%s";ps -C ' + "\""+processList+"\"" + ' -m -o psr,pcpu,pmem,rss,pid,cmd';

			// end

			//var cmd = 'date +"%s"; ps -C ' + "\""+processList+"\"" + ' -m -o psr,pcpu,pmem,rss,pid,cmd';

			var getProcessInfo = spawn.exec(cmd, function(error, stdout, stdin){

				if(error){

					response = {"returnValue": false, "result" : error};

					socket.emit('processinfo', response);

				}else{

					formatProcessInfo(stdout, function(result){

						response = {"returnValue": true, "result" : result};

						socket.emit('processinfo', response);

					});

				}

			});

		}catch(e){

			response = {"returnValue": false, "result" : e};

			socket.emit('processinfo', response);

		}

	}



	function formatProcessInfo(stdout, callback) {



		var sysinfo = {};



		var cpuinfo = {};

		var meminfo = {};

		var processinfo = {};



		var allvalues = stdout.split("\n");



		// Start

		var processList = [], pListEndIndex = 0;

		var qtProc = '/usr/bin/QtWebProcess';



		for(var i = 0; ;i++) {

			var values = allvalues[i].split(' ');

			var pid = parseFloat(values[0]); // pid



			//4194304 -> this value is the max pid possible for linux and this check confirms the entry of date in miliseconds in this line

			if(values[0] > 4194304) {

				pListEndIndex = i;

				break;

			}

			var name = values[1];// command or processname



			if (values.length == 3) {

				if (name == qtProc) {

					name = name + " " + values[2];

				}



				if (name == 'grep' || name == '/bin/sh') {

					continue;

				}

			} else if (name == qtProc) {

				continue;

			}



			if (pid == 'undefined' || pid == null || name == '-sh' || name == 'ps') {

				continue;

			}



			var process = name.split('/'), processName = process[process.length - 1];

			processList.push(processName);

		}

		// end



		// setup date

		//var date = allvalues[0];



		// Start

		var date = allvalues[pListEndIndex];

		//end



		var index, columns, processDataStartIndex;

		var processCmd, processName, processCpu, processMemorySize, processMemoryPercent, processCores, qtProcCount=0, processId;

		var firstProcess = true;



		//for (index = 1; index < (allvalues.length - 1); index++ ) {

		for (index = pListEndIndex + 1; index < (allvalues.length - 1); index++ ) {

			columns = allvalues[index].split(/\s+/);



			if ((columns[0].indexOf('PSR') == 0) && (columns[1].indexOf('%CPU') == 0)) {

				processDataStartIndex = index;

			}

			// setup processinfo

			if ((processDataStartIndex !== undefined) && (index > processDataStartIndex)) {

				if (columns[1].indexOf('-') == 0) {

					if (!firstProcess) {

						processinfo[processName] = {

								"pid": processId,

								"cpu": processCpu,

								"memory": {

									"size": processMemorySize,

									"percent": processMemoryPercent

								},

								"cores": processCores

						};

					}

					processCmd = columns[6].split('/');

					if (processCmd.indexOf("QtWebProcess") > -1) {

						processName = processCmd[processCmd.length - 1] + " " + columns[7];

						qtProcCount++;

					} else {

						processName = processCmd[processCmd.length - 1];

					}

					processCpu = parseFloat(columns[2]);

					processMemoryPercent = parseFloat(columns[3]);

					processMemorySize = parseFloat(columns[4]);

					processId = columns[5];

					processCores = {};

					firstProcess = false;

				} else {

					if (processCores[columns[1]] === undefined) {

						processCores[columns[1]] = 0;

					}



					processCores[columns[1]] += parseFloat(columns[2]);

				}

			}

		}



		processinfo[processName] = {

				"pid": processId,

				"cpu": processCpu,

				"memory": {

					"size": processMemorySize,

					"percent": processMemoryPercent

				},

				"cores": processCores

		};



		sysinfo = {

				"date": date * 1000,

				"processinfo": processinfo,

				"qtProcCount": qtProcCount,

				"processList": processList

		};



		callback(sysinfo);

	}

}());