/**********************************************************

* Filename					: processstatus_handler.js

* Purpose					: Exposes Process Status API from signage

* Author(s)					: chamoknath.choudhury@lge.com, amber.pandit@lge.com

* Creation date				: April 19, 2016

**********************************************************/



(function(){



	module.exports = processStatus;



	var spawn = require('child_process');



	function processStatus(msg, socket) {

		//Expected msg format : {"subscribe" : true/false, "interval": milliseconds}

		var processStatusProtocol = socket._protocols['processstatus'];

		if(processStatusProtocol){

			clearInterval(processStatusProtocol);

		}

		if(msg.subscribe === true){

			//Interval because subscribe is true, it will send data at each interval

			if(msg.intervalValue && !isNaN(msg.intervalValue)){

				socket._protocols['processstatus'] = setInterval(function(){

					//Code for Command Execution and emit the data back to client

					getProcessStatus(msg.pid, socket);

				}, msg.intervalValue);

			}else{

				response = {"returnValue": false, "result" : "Invalid Interval Value"};

				socket.emit('processstatus', response);

			}

		} else{

			//No Interval because subscribe is false, it will send data only once

			getProcessStatus(msg.pid, socket);

		}

	}



	function getProcessStatus(pid, socket) {

		var response = {};

		try{

			var cmd = 'cat /proc/' + pid + '/status';

			var getProcessStatus = spawn.exec(cmd, function(error, stdout, stdin){

				if(error){

					response = {"returnValue": false, "result" : error};

					socket.emit('processstatus', response);

				} else {

					response = {"returnValue": true, "result" : stdout};

					socket.emit('processstatus', response);

				}

			});

		}catch(e){

			response = {"returnValue": false, "result" : e};

			socket.emit('processstatus', response);

		}

	}



})();