/**********************************************************

* Filename					: sysinfo_handler.js

* Purpose					: Exposes System Info API from signage

* Author(s)					: chamoknath.choudhury@lge.com, amber.pandit@lge.com

* Creation date				: April 18, 2016

**********************************************************/



(function(){



	module.exports = systemInfo;



	var spawn = require('child_process');



	function systemInfo(msg, socket) {

		//Expected msg format : {"subscribe" : true/false, "interval": milliseconds}



		var systemInfoProtocol = socket._protocols['sysinfo'];

		if(systemInfoProtocol){

			clearInterval(systemInfoProtocol);

		}



		if(msg.subscribe === true){

			//Interval because subscribe is true, it will send data at each interval

			if(msg.intervalValue && !isNaN(msg.intervalValue)){

				socket._protocols['sysinfo'] = setInterval(function(){

					//Code for Command Execution and emit the data back to client

					getSystemInfo(socket);

				}, msg.intervalValue);

			}else{

				response = {"returnValue": false, "result" : "Invalid Interval Value"};

				socket.emit('sysinfo', response);

			}

		} else{

			//No Interval because subscribe is false, it will send data only once

			getSystemInfo(socket);

		}

	}



	function getSystemInfo(socket) {

		var response = {};

		try{

			var cmd = 'date +"%s"; grep "cpu *" /proc/stat; free –k';

			var getMemory = spawn.exec(cmd, function(error, stdout, stdin){

				if(error){

					response = {"returnValue": false, "result" : error};

					socket.emit('sysinfo', response);

				} else {

					getCpuInfo(stdout, function(result){

						response = {"returnValue": true, "result" : result};

						socket.emit('sysinfo', response);

					});

				}

			});

		}catch(e){

			response = {"returnValue": false, "result" : e};

			socket.emit('sysinfo', response);

		}

	}



	function getCpuInfo(stdout, callback) {



		var sysinfo = {};



		var cpuinfo = {};

		var meminfo = {};



		var allvalues = stdout.split("\n");



		// setup date

		var date = allvalues[0];



		var index, columns;



		for (index = 1; index < (allvalues.length - 1); index++ ) {

			columns = allvalues[index].split(/\s+/);



			// setup cpuinfo

			if (columns[0].indexOf('cpu') == 0) {

				var subTotal = parseInt(columns[1]) + parseInt(columns[2]) + parseInt(columns[3]) + parseInt(columns[4]);

				var userModeCpuOccupation = (parseInt(columns[1]) * 100) / subTotal;

				var kernelModeCpuOccupation = (parseInt(columns[3]) * 100) / subTotal;

				var overallCpuOccupation = userModeCpuOccupation + kernelModeCpuOccupation;



				cpuinfo[columns[0]] = {

						"overall": +overallCpuOccupation.toFixed(2),

						"usermode": +userModeCpuOccupation.toFixed(2),

						"kernelmode": +kernelModeCpuOccupation.toFixed(2)

				};

			}



			// setup meminfo

			if (columns[0].indexOf('Mem:') == 0) {

				meminfo['memory'] = {

						"total": +columns[1],

						"used": +columns[2],

						"free": +columns[3],

						"shared": +columns[4],

						"buffers": +columns[5],

						"cached": +columns[6]

				}

			}



			if (columns[0].indexOf('-/+') == 0) {

				meminfo['buffers'] = {

						"used": +columns[2],

						"free": +columns[3]

				}

			}



			if (columns[0].indexOf('Swap:') == 0) {

				meminfo['swap'] = {

						"total": +columns[1],

						"used": +columns[2],

						"free": +columns[3]

				}

			}

		}



		sysinfo = {

				"date": date * 1000,

				"cpuinfo": cpuinfo,

				"meminfo": meminfo

		};



		callback(sysinfo);

	}



})();