/**********************************************************
* Filename		: websocket_service.js
* Purpose		: Exposes service for Beanviser
* Author(s)		: chamoknath.choudhury@lge.com, amber.pandit@lge.com
* Creation date	: April 18, 2016
**********************************************************/

(function(){

	var Service = require('webos-service');
	//Register com.yourdomain.@DIR@.service, on both buses
	var serviceId = "com.lg.app.signage.signagemanagement.service";

	var service = new Service(serviceId);
	//a method that always returns the same value
	service.register("init", function(message) {
		//try catch for exception if it does not find socket.io or if port is already busy
		try {
			var sysinfo = require('./handlers/sysinfo_handler');
			var processinfo = require('./handlers/processinfo_handler');
			var procinfo = require('./handlers/procinfo_handler');
			var processstatus = require('./handlers/processstatus_handler');
			var pmmodule = require('./handlers/pmmodule_handler');
			var taillog = require('./handlers/taillogs_handler');
			var killtailprocess = require('./handlers/killtailprocess_handler');
			var inodeinfo = require('./handlers/inodeinfo_handler');
			var cpucore = require('./handlers/cpucore_handler');
			var procgroupinfo = require('./handlers/processgroupinfo_handler');
			var macinfo = require('./handlers/macinfo_handler');
			var systeminfo = require('./handlers/systeminfo_handler');

			var io = require('socket.io')();

			io.listen(9762);
			io.set('heartbeat interval', 2000);
			io.set('heartbeat timeout', 10000);

			io.on('connection', function(socket) {
				//It will listen for get-api call
				socket._protocols = {};
				socket.on('sysinfo', function(msg){sysinfo(msg, socket);});
				socket.on('processinfo', function(msg){processinfo(msg, socket);});
				socket.on('procinfo', function(msg){procinfo(msg, socket);});
				socket.on('processstatus', function(msg){processstatus(msg, socket);});
				socket.on('pmmodules', function(msg){pmmodule(msg, socket);});
				socket.on('taillogs', function(msg){taillog(msg, socket);});
				socket.on('killtailprocess', function(msg){killtailprocess(msg, socket);});
				socket.on('inodeinfo', function(msg){inodeinfo(msg, socket);});
				socket.on('cpucore', function(msg){cpucore(msg, socket);});
				socket.on('processgroupinfo', function(msg){procgroupinfo(msg, socket, message);});
				socket.on('macinfo', function(msg){macinfo(msg, socket, service);});
				socket.on('systeminfo', function(msg){systeminfo(msg, socket, service);});
			});
		}catch(e){
			message.respond({
				returnValue: true,
				message: "Socket Connection Error. Please Try Again."
			});
		}

	});//service register end

})();