// Include gulp
var gulp = require('gulp');

// Include Our Plugins
var jshint		= require('gulp-jshint');
var concat		= require('gulp-concat');
var minifyCss	= require('gulp-minify-css');
var uglify		= require('gulp-uglify');
var rename		= require('gulp-rename');
var htmlreplace	= require('gulp-html-replace');
var sourcemaps	= require('gulp-sourcemaps');
var clean 		= require('gulp-clean');
var del 		= require('del');
var runSequence = require('run-sequence');

var folders = {};
	folders.root 	= './dist/';
	folders.css 	= folders.root +'css/';
	folders.js 		= folders.root +'scripts/';
	folders.fonts 	= folders.root +'fonts/';
	folders.images 	= folders.root +'images/';


// Concatenate & Minify JS window.html
gulp.task('window:head-scripts', function() {
    return gulp.src([
    	'./scripts/vendor/jquery-1.11.2.min.js',
    	'./scripts/vendor/sweet-alert.min.js'
    	])
        .pipe(concat('window-head-all.min.js'))
        .pipe(uglify())
        .pipe(gulp.dest(folders.js));
});

gulp.task('window:footer-scripts', function() {
    return gulp.src(['./scripts/main.js'])
        .pipe(concat('window-footer-all.min.js'))
        .pipe(uglify())
        .pipe(gulp.dest(folders.js));
});

// replace HTML scripts
gulp.task('window:replace', function() {
    return gulp.src('screens/window.html')
	    .pipe(htmlreplace({
	        'window-head-js'  : '../../scripts/window-head-all.min.js',
	        'window-footer-js': '../../scripts/window-footer-all.min.js',
	        // 'window-head-css' : 'css/window-footer-all.min.css'
	    }))
	    .pipe(gulp.dest(folders.root + 'screens/'));
});


// Concatenate & Minify JS startup-screen.html
gulp.task('startupscreen:head-scripts', function() {
    return gulp.src([
    	'./scripts/vendor/pace.min.js',
    	'./scripts/vendor/angular/angular.min.js',
    	'./scripts/chrome-i18n.js',
        './scripts/controllers/startup-screen.js',
        './scripts/vendor/underscore-min.js'
    	])
        .pipe(concat('startupscreen-head-all.min.js'))
        .pipe(uglify())
        .pipe(gulp.dest(folders.js));
});

gulp.task('startupscreen:footer-scripts', function() {
    return gulp.src([
    	'./scripts/vendor/jquery-2.1.1.min.js',
    	'./scripts/vendor/bootstrap.min.js',
    	'./scripts/vendor/plugins.min.js',
    	'./scripts/vendor/nifty.min.js',
    	'./scripts/vendor/demo/layouts.js',
    	'./scripts/vendor/sweet-alert.min.js',
    	'./scripts/vendor/URI.min.js',
    	'./scripts/vendor/zip.js/zip.js',
    	'./scripts/vendor/zip.js/zip-ext.js'
		])
        .pipe(concat('startupscreen-footer-all.min.js'))
        .pipe(uglify())
        .pipe(gulp.dest(folders.js));
});

// replace HTML scripts
gulp.task('startupscreen:replace', function() {
    return gulp.src('./screens/start-config/startup-screen.html')
	    .pipe(htmlreplace({
	        'startupscreen-head-js'  : '../../scripts/startupscreen-head-all.min.js',
	        'startupscreen-footer-js': '../../scripts/startupscreen-footer-all.min.js',
	        // 'startupscreen-head-css' : 'css/startupscreen-footer-all.min.css'
	    }))
	    .pipe(gulp.dest(folders.root + 'screens/start-config/'));
});

gulp.task('minify-css', function() {
    return gulp.src('./css/*.css')
        .pipe(sourcemaps.init())
        .pipe(minifyCss())
        .pipe(sourcemaps.write())
        .pipe(gulp.dest(folders.css));
});

// copy all files to dist folder
gulp.task('copy-files', function() {
    return gulp.src('**')
        .pipe(sourcemaps.init())
        .pipe(sourcemaps.write())
        .pipe(gulp.dest(folders.root));
});

gulp.task('uglify-js',function(){
    return gulp.src([
                'scripts/utils.js',
                'scripts/ClassRss.js',
                'scripts/FileSync.js',
                'scripts/FileSync.Http.js',
                'scripts/FileSync.Drive.js',
                'scripts/PlayListManager.js',
                'scripts/Player.js',
                'scripts/ManagerConnect.js',
                'scripts/HttpRequest.js',
                'scripts/background.js',
                ])
        .pipe(uglify())
        .pipe(gulp.dest(folders.js));
});

// remove non useds files in dist folder
gulp.task('clean', function() {
    return gulp.src([
    	folders.root +'dist',
    	folders.root +'templates',
    	folders.root +'node_modules',
    	folders.root +'images/openimages',
    	folders.root +'images/screencapture',
    	folders.root +'*.log',
    	folders.root +'gulpfile.js',
    	folders.root +'package.json',
    	])
        .pipe(clean());
});

gulp.task('windowHtml', ['window:head-scripts','window:footer-scripts', 'window:replace']);
gulp.task('startupscreenHtml', ['startupscreen:head-scripts','startupscreen:footer-scripts', 'startupscreen:replace']);

gulp.task('test', function(callback){
    runSequence('copy-files','clean',['startupscreenHtml', 'windowHtml', 'minify-css'], callback);
});

// Default Task
gulp.task('default', function (callback){
	runSequence('copy-files', 'clean', ['startupscreenHtml',  'windowHtml', 'minify-css'], 'uglify-js', callback);
});
