'use strict';

(function(window, jQuery) {

    var that;

    /**
     * Classe para controlar a lista de áudio
     *
     * @class ClassAudio
     * @param {Element} objElementHtml Elemento html de áudio
     * @param {Object} playerData     Objeto de com os dados do player
     * @param {JSON} jsonPlaylist   Lista de áudios a serem tocados
     */
    function ClassAudio (objElementHtml, playerData, jsonPlaylist) {

        // that
        that = this;

        // element audio
        this.elementAudio = objElementHtml;

        // objeto player
        if (!_.isUndefined(playerData)) {
            this.setPlayer(playerData);
        }

        // playlist
        if (!_.isUndefined(jsonPlaylist)) {
            // caso seja informado a playlist
            this.setjsonPlaylist(jsonPlaylist);
        }

        // para saber se o player está tocando ou não
        this.player = false;

        // item que está sendo tocado
        this.currentPlayer = undefined;

        // total de itens
        this.countPlayer = 0;

        // buscando as configuraçõess
        player4uc.storage.get(['settings.projectName', 'settings.appDir'], function(storageObj) {

            // pasta do projeto
            that.appDir = storageObj['settings.appDir'];

            // pasta padrão dos dados
            that.folderFiles = FileSync.PROJECT_PREFIX + storageObj['settings.projectName'];

            // configura o arquivo de media
            that.config = {
                pathFilePlayList: that.folderFiles + '/' + FileSync.PLAYER_DIR_JSON + '/' + 'media.' + that.objPlayer.id + '.a0.json',
                filePlaylist: 'media.' + that.objPlayer.id + '.a0.json',
                folderFilesAudio: FileSync.PLAYER_DIR_AUDIO
            }

            if (_.isUndefined(that.playlist)) {
                // caso não seja informada a playlist
                filePlaylistAudio(that.config.pathFilePlayList, function doneFilePlayListAudio(mediaData) {
                    that.setjsonPlaylist(mediaData);
                });
            }
        });
    }

    /**
     * Método para iniciar todo o processo para que o áudio seja tocado
     *
     * @method init
     * @return {void}
     */
    ClassAudio.prototype.init = function (pararMusica) {
        // verifica se a playlist está preenchida
        if (_.isUndefined(this.playlist)) {
            return console.error('Playlist not found');
        }

        // that
        var that = this;

        // this.validateConstruction();

        // gerar a lista de source
        openFile(this.elementAudio, this.playlist, function() {
            // controla o carrocel
            setCarrocel(that.elementAudio);

            // evento para quando o áudio acabar
            that.eventPlay();

            // dá player no primeiro item
            setTimeout(function PlayerFirstAudio(){that.getItemNext(pararMusica)}, 3000);
        });
    };

    /**
     * Método para da o player
     *
     * @method play
     * @return {void}
     */
    ClassAudio.prototype.play = function () {
        this.elementAudio.play();
    };

    /**
     * Método para da stop no áudio
     * @method stop
     * @return {void}
     */
    ClassAudio.prototype.stop = function () {
        this.elementAudio.pause();
    };

    /**
     * Método para colocar o player no modo mudo
     * @method mute
     * @return {void}
     */
    ClassAudio.prototype.mute = function () {
        // this.elementAudio.setAttribute('muted', true);
        this.volume(0);
    }

    /**
     * Método para controlar o volume do áudio
     *
     * @method volume
     * @param integer volume Valor entre 0 e 100
     * @return {void}
     */
    ClassAudio.prototype.volume = function (volume) {
        this.elementAudio.setAttribute('volume', volume/100);
    }

    /**
     * Método para resetar a play list de áudio
     * @method resetList
     * @return {void}
     */
    ClassAudio.prototype.resetList = function () {
        // limpa lista de arquivos
        jQuery(this.elementAudio).find("source").remove();

        // lista playlist
        this.playlist = undefined;

        // remove o evento
        jQuery(this.elementAudio).unbind('ended');
    }

    /**
     * Método para gerar o evento que tornará o player sempre com áudio
     * @method eventPlay
     * @param  {integer} next
     * @return {void}
     */
    ClassAudio.prototype.eventPlay = function() {
        // evento para quando o áudio acabar
        jQuery(this.elementAudio)
            .bind('ended', function() {
                that.getItemNext(true);
            });
    }

    /**
     * Método para ir para o próximo item
     *
     * @method getItemNext
     * @return {void}
     */
    ClassAudio.prototype.getItemNext = function(pararMusica) {

        // não parar e trocar a musica caso ela esteje tocando
        if (_.isUndefined(pararMusica) && !_.isUndefined(that.currentPlayer)) {
            return;
        }

        var next = 0;

        // caso não exista nenhum item na lista
        if (that.countPlayer === 0) {
            return;
        }

        if (!_.isUndefined(that.currentPlayer)) {
            next = that.currentPlayer + 1;

            // próximo
            if (that.countPlayer < next) {
                next = 0;
            }
        } else {
            // seta um valor
            that.currentPlayer = 0;
        }

        // o último item tocado
        var elementPrev = (getSourcePlayer(that.elementAudio, (that.currentPlayer? that.currentPlayer : 0)));

        // seta o valor de current
        that.currentPlayer = next;


        // elemento
        var element = getSourcePlayer(that.elementAudio, next);

        // caso o áudio seja inválido
        if (_.isUndefined(element) || jQuery(element).attr('tocar') === "false") {
            // procura outro áudio
            this.getItemNext(pararMusica);

            // console
            return console.error('Id do áudio informado não será tocado.');
        }

        // seta o próximo item
        that.setAudioPlayer(element);

        // inicia o áudio
        that.play();

        // caso o último item tocado tenha sido carrocel
        if (jQuery(elementPrev).attr('carrocel')) {
            // troca o item do carrocel
            setCarrocel(this.elementAudio);
        }
    };

    /**
     * Método para setar o o próximo item da lista
     * @method setAudioPlayer
     * @param {Element} element Elemento source
     */
    ClassAudio.prototype.setAudioPlayer = function (element) {
        // atribue o caminho do áudio
        this.elementAudio.setAttribute('src', element.src);
    }

    /**
     * Método para setar o json da playlist
     * @method setjsonPlaylist
     * @param  {json} jsonPlaylist
     * @return {void}
     */
    ClassAudio.prototype.setjsonPlaylist = function(jsonPlaylist) {
        if (_.isUndefined(jsonPlaylist)) {
            return console.error('Playlist inválid.');
        }

        this.playlist = jsonPlaylist;
    };

    /**
     * Método para setar o objeto do player
     * @method setPlayer
     * @param {Object} objPlayer {id: integer}
     */
    ClassAudio.prototype.setPlayer = function (objPlayer) {
        if (!_.isObject(objPlayer) || !('id' in objPlayer)) {
            return contole.error('O Objeto não é válido.');
        }

        this.objPlayer = objPlayer;
    }

    ClassAudio.prototype.validateConstruction = function (exception, endCallback) {
        var projPath = this.folderFiles;
        var that = this;

        // iniciando o processo (Callback Hell)
        // verificar se o arquivo media.<idPlayer>.a0.json
        filePlaylistAudio (that.config.pathFilePlayList, function (mediaJson) {

            // em caso de erro
            if (_.isUndefined(mediaJson)) {
                throw new Error('Arquivo media.<idPlayer>.a0.json não existe');
            }

            // verificar se a pasta mp3 existe
            folderExist (function (folderEntity) {

                if (_.isUndefined(folderEntity)) {
                    throw new Error ('Pasta MP3 não foi criada.');
                }

                // verificar se os mp3's estão na pasta mp3
                filesAudioExist (mediaJson, folderEntity, endCallback);
            });
        });

        /**
         * Método para verificar se a pasta mp3 existe
         * @param  {Function} callback(folderEntity|false)
         * @return {void}
         */
        function folderExist (callback) {
            var folderAudios = projPath + '/' + FileSync.PLAYER_DIR_AUDIO;

            // abre a pasta
            openFolderChrome (function (appDir) {

                // verifica se a pasta existe
                appDir.getDirectory(folderAudios, {create: true}, function doneGetDirectoryFoldeExist (folderEntity) {
                    console.log("doneGetDirectoryFoldeExist", folderEntity);

                    // callback
                    callback(folderEntity);
                },
                function errorGetDirectoryFoldeExist (e) {
                    console.error(e);

                    // callback
                    callback();
                });
            });
        }

        /**
         * Método para verificar se os arquivos mp3 da playlist existe
         * @param  {Function} callback(boolean)
         * @return {void}
         */
        function filesAudioExist (fileList, folderEntity, callback) {

            // percorre a lista
            async.eachSeries(fileList.list, function processItensAudio (item, next) {

                // verifica se o arquivo existe
                folderEntity.getFile(item.arquivo, {create: false}, function doneGetFile(fileEntity) {

                    // caso não exista nenhum arquivo
                    if (!(_.isUndefined(fileEntity))) {
                        console.log("Arquivo " + item.arquivo + " não encontrado.");
                    }

                    next();
                });
            }, function doneListFilesAudio () {
                // callback
                callback();
            });
        }

    };

    /**
     * Método para verificar se o arquivo de playlist de música existe
     *
     * @method filePlaylistAudio
     * @param  {Function} callback(object|undefined)
     * @return {void}
     */
    function filePlaylistAudio (pathFile, callback) {
        // abre a pasta
        openFolderChrome (function (appDir) {
            // busca a playlist de áudio
            FileSync.readJSONFile(appDir, pathFile, function(jsonItem, error) {
                // caso não exista nenhum arquivo
                if (!(_.isNull(error))) {
                    return callback();

                }

                // sucesso
                callback(jsonItem);
            });
        });
    }

    /**
     * Método para abrir uma pasta no chrome
     *
     * @method openFolderChrome
     * @param  {Function} callback
     * @return {void}
     */
    function openFolderChrome (callback) {
        player4uc.storage.get(['settings.appDir', 'settings.projectName'], function(storageObj) {

            // busca os arquivos na pasta de dados
            player4uc.fileSystem.restoreEntry(storageObj['settings.appDir'], function appDirRestored(appDir) {
                // em caso de erro
                if (_.isNull(appDir)) {
                    callback();
                }

                //sucesso
                callback(appDir);
            });
        })
    }

    /**
     * Método para retornar o elemento source
     *
     * @method getSourcePlayer
     * @param  {Element} ElementAudio
     * @param  {integer} idSource     Id do áudio
     * @return {jQuery#Element}
     */
    function getSourcePlayer (ElementAudio, idSource) {
        var element = jQuery(ElementAudio).find('source');

        return (_.isUndefined(element)? undefined : element[idSource]);
    }

    /**
     * Método para ordenar a playlist
     *
     * @method orderPlayList
     * @param  {json} playlist
     * @return {array}
     */
    function orderPlayList (playlist) {
        if (!(_.isObject(playlist)) || !('seq' in playlist)) {
            return [];
        }

        return playlist.seq.split(",");
    }

    /**
     * Método para ler o arquivo de carrocel de áudios
     *
     * @method openFile
     * @param  {Element} ElementAudio
     * @param  {JSON} playlist
     * @return {void}
     */
    function openFile(ElementAudio, playlist, endCallback) {
        var i, item, itemJson;

        // ordem da playlist
        var order = orderPlayList(playlist);

        // loop
        async.eachSeries(order,
            function orderList(orderI, next) {
                var itemPlaylist = playlist.itens[orderI];

                // caso seja do tipo carrocel
                if (itemPlaylist.tipo == "car") {

                    // arquivo para download
                    itemJson = 'categoria.' + itemPlaylist.arquivo + '.json';

                    // player4uc busca o arquivo
                    player4uc.fileSystem.restoreEntry(that.appDir, function appDirRestored(rootDirEntry) {
                        // lendo json do carrocel
                        FileSync.readJSONFile(rootDirEntry, that.folderFiles + '/json/' + itemJson, function (jsonFile, error) {
                            addSources(jsonFile, error, next);
                        });
                    });
                }

                // case seja do tipo i
                if (itemPlaylist.tipo == "i") {
                    addOneSource(itemPlaylist, next);
                }
            },
            function doneOpenfile() {
                endCallback();
            }
        );

        /**
         * Método para adicionar source ao áudio utilizando uma lista
         * @param {JSON} jsonFile
         */
        function addSources(jsonFile, error, callback) {

            // console.log('TAG_MAYCON addSources: jsonFile, error', jsonFile, error);

            // caso aconteça algum erro
            if (!(_.isNull(error))) {
                return console.error(error);
            }

            // caso o objeto não seja um json
            if (!(_.isObject(jsonFile))) {
                jsonFile = JSON.parse(jsonFile);
            }

            async.eachSeries(jsonFile.itens,
                function(item, next) {
                // itens de carrocel
                item.carrocel = true;

                addOneSource(item, next);
            },
            // callback
            callback);

        }

        /**
         * Método para adicionar o elemento source de áudio
         * @param {JSON} item
         */
        function addOneSource (item, callback) {
            // somente se o tipo for i
            if (item.tipo == 'i') {
                // gerar o source do arquivo
                var element = generateSourceFile(item);

                // adiciona o source no elemento áudio
                addSourceAudio(ElementAudio, element);

                // volta
                callback();
            }
        }
    }

    /**
     * Método para adicionar elementos sources no elemento áudio
     *
     * @method addSourceAudio
     * @param {Element} ElementAudio
     * @param {Element} ElementSource
     */
    function addSourceAudio(ElementAudio, ElementSource) {
        ElementAudio.appendChild(ElementSource);

        that.countPlayer++;
    }

    /**
     * Método para gerar a lista com os arquivos
     *
     * @method generateSourceFile
     * @param  {json} itemList
     * @return {ElementSource}
     */
    function generateSourceFile(itemList) {

        // validando json
        if ((!('arquivo' in itemList))) {
            return console.error('JSON not suported', itemList);
        }

        var elementSource = document.createElement('source');
        elementSource.id = itemList.arquivo;
        elementSource.type = getTypeAudio(itemList.arquivo);
            elementSource.setAttribute('tocar', true);

        // caso seja carrocel
        if ('carrocel' in itemList) {
            elementSource.setAttribute('carrocel', true);
            elementSource.setAttribute('tocar', false);
        }

        player4uc.storage.get('settings.appDir', function(storageObj) {

            player4uc.fileSystem.restoreEntry(storageObj['settings.appDir'], function appDirRestored(appDir) {

                var filePath = that.folderFiles + '/' + FileSync.PLAYER_DIR_AUDIO + '/' + itemList.arquivo;

                // Método para buscar o arquivo
                FileSync.getFile(
                    appDir,
                    filePath,
                    /**
                     * Método de callback
                     * @param  {EntityFile} fileAudioEntity Arquivo a ser buscado
                     * @param  {string|null} error
                     * @return {void}
                     */
                    function(fileAudioEntity, error) {

                        if (!_.isUndefined(error)){
                            return console.error('File "' + filePath + '" not found.', error);
                        }

                        // carrega o arquivo
                        fileAudioEntity.file(function(audioFile) {
                            elementSource.src = URL.createObjectURL(audioFile);
                        });
                    }
                );
            });
        });


        return elementSource;
    };

    /**
     * Método para retornar o tipo de áudio
     * @method getTypeAudio
     * @param  {Element} item
     * @return {string}
     */
    function getTypeAudio(item) {
        var ext = item.split('.').pop();

        switch (ext) {
            case 'mp3':
                return 'audio/mp3';
                break;
            case 'mp4':
                return 'video/mp4';
                break;
            case 'ogg':
                return 'audio/ogg';
                break;
        }

        console.error('Extension not found');
        return '';
    };

    /**
     * Método para controlar o carrocel
     *
     * @method setCarrocel
     */
    function setCarrocel(elementAudio) {
        var arrayCarrocel = jQuery(elementAudio).find('source[carrocel=true]');

        var countCarrocel = arrayCarrocel.length;

        // caso a lista esteja vazia
        if (!countCarrocel) {
            return;
        }

        var cnt = 0,
            item,
            next;

        // verifica se já existe alguém marcado para tocar
        next = 0;
        for(cnt=0; cnt< countCarrocel; cnt++) {
            item = $(arrayCarrocel[cnt]);

            //quem toca
            if (item.attr('tocar') == "true") {
                // seta todos como false
                arrayCarrocel.attr('tocar', false);

                // verificar se esse cara é último
                if (cnt < (countCarrocel-1)) {
                    next = cnt + 1;
                }

                // seleciona o próximo
                arrayCarrocel[next].setAttribute('tocar', true);

                break;
            }
         };

        // caso ninguém ainda tenha sido selecionado
        if (!next) {
            arrayCarrocel[next].setAttribute('tocar', true);
        }
    }


    window.ClassAudio = ClassAudio;

})(window, jQuery);
