(function(_global) {

    /**
     * Classe para controlar a exibição do carrossel
     *
     * @class Carrossel
     * @constructor
     * @param {string} projPath Caminho do projeto
     */
	function Carrossel(projPath) {
		this.projPath = projPath || '';
        this.current = [];
        this.contentsValidade = [];
	}

    /**
     * Método para setar o JSON com o conteúdo
     *
     * @method setItens
     * @param {JSON} itens
     */
	Carrossel.prototype.setItens = function(itens) {
		this.itens = itens;
	}

    /**
     * Método para setar o código do carrossel
     *
     * @method setCodeCarrossel
     * @param {string} code
     */
    Carrossel.prototype.setCodeCarrossel = function (code) {
        this.code = code;
    }

    /**
     * Set itens with validade especific
     * @param {Array} content
     */
    Carrossel.prototype.setValidadeContent = function __doneSetValidadeContent (content) {
        this.contentsValidade = content;
    }

    /**
     * Método para validar os itens da lista
     *
     * @method getNext
     * @param  {function} callBack
     * @param  {object} loop     Json com o conteudo
     * @return {void}
     */
    Carrossel.prototype.getNext = function(callBack, loop) {
        // console.log('getNext loop', loop);

        if(loop == undefined)
            var loop = this.itens.length;
        else if(loop == 0)
            return callBack('No Carrossel feed valid')

        if((typeof(this.itens) != 'object') || !this.itens.length)
            return callBack('Carrossel list is empty');

        if(this.itens.length == 0)
            return callBack('Carrossel list is empty');

        //Restart the rss feed pointer
        if(typeof(this.current[this.code]) == 'undefined' || this.current[this.code] >= this.itens.length - 1)
            this.current[this.code] = 0;
        else
            this.current[this.code]++;

        var item = this.itens[this.current[this.code]];
        var that = this;

        // verify the content has validade especific
        var itemEspecific = _.findWhere(this.contentsValidade, {arquivo: item.arquivo});
        if (!_.isEmpty(itemEspecific)) {
            item.schedule = itemEspecific.schedule;
            item.startDate = itemEspecific.startDate;
            item.endDate = itemEspecific.endDate;
        }

        // retorna o conteudo
        this.validateRss(item, function(isAvailable){

            if(isAvailable)
                return callBack(item);
            else
                that.getNext(callBack, loop-1);
        });
    }

    /**
     * Método para avançar um 1 item
     *
     * @method next
     * @return {Function}
     */
	Carrossel.prototype.next = function() {
		if(typeof(this.current[this.code]) == 'undefined')
			return this.current[this.code] = 0;

		if(this.current[this.code] >= this.itens.length -1)
			return this.current[this.code] = 0;

		this.current[this.code] = this.current[this.code] + 1;
		return this.current[this.code];
	}


    /**
     * Método para buscar um item em especifico
     *
     * @method getItem
     * @param  {integer} current
     * @return {object}
     */
	Carrossel.prototype.getItem = function(current) {
		var item;
		if(!current)
			var current = this.current[this.code];
		item = this.itens[current];

		return item;
	}

    /**
     * Método com funções para validar o rss
     *
     * @method validateRss
     * @param  {json} item
     * @param  {function} callBack
     * @return {void}
     */
	Carrossel.prototype.validateRss = function(item, callBack){
		this.validateDate(item, callBack);
	}

	/**
	 * Method to check new validate date
     *
     * @method validateDate
	 * @param Json item
	 * @return Boolean
	 */
	Carrossel.prototype.validateDate = function(item, callBack){
		if(!item)
			return callBack(false);

		return callBack(Player.prototype.dateValidate(item));
	}

    /**
     * Método para resetar o ponteiro da playlist
     *
     * @method resetPlaylistCurrent
     * TODO: o legal é que o método para realizar o player saiba que eu informei 0
     * @return {void}
     */
    Carrossel.prototype.resetPlaylistCurrent = function __functionResetPlaylistCurrent() {
        var i;
        for (i in this.current) {
            this.current[i] = -1;
        }
    }

	_global.Carrossel = Carrossel;
})(window);
