'use strict';

/**
 * Classe para gerenciar a exibição do conteudo dos players
 * @return {void}
 */
(function (window) {

    // constantes
    var TYPE_WEBVIEW = 'iframe';
    var TYPE_VIDEO = 'video';
    var idElements = 0;

    /**
     * Método para gerar o layout na janela do player
     *
     * @class ClassLayout
     * @constructor
     * @param {Element} Container     Elemento html da área do conteúdo
     * @param {Object} JsonAllLayout Json com a descriçaõ da área a ser carregada
     */
    function ClassLayout (Container, JsonAllLayout) {
        this.jsonAllLayout = undefined;
        this.jsonLayout = undefined;
        this.listElements = [];

        // Element Container
        this.ObjContainer = Container;;

        // seta json layout
        if (!_.isUndefined(JsonAllLayout)) {
            this.setJsonAllLayout(JsonAllLayout);
        }
    }

    /**
     * Método para criar todos os campos de um layout
     *
     * @method createLayout
     * @param  {boolean} activeLayout Se as áreas do layout devem ser exibidas
     * @return {void}
     */
    ClassLayout.prototype.createLayout = function (activeLayout, NotCreateAreaMainFirstPlayer) {
        // criando as áreas
        var jsonLayout = this.getLayout();
        var i, element;

        // verifica se existe a chave
        if (_.isUndefined(jsonLayout) || !('area' in jsonLayout)) {
            return _logError('Json inválido.');;
        }

        // monta layout
        for(i in jsonLayout.area) {

            // caso não seja para criara área principal
            if (NotCreateAreaMainFirstPlayer && jsonLayout.area[i].main == "true") {

                // caso entre nessa condição a lista de classes deve ter somente a área principal
                this.listElements[0].setJsonArea(jsonLayout.area[i]);

                // próximo conteúdo
                continue;
            }

            // gerar elemento
            element = new LayoutArea(jsonLayout.area[i]);
            // element = this.createElement(jsonLayout.area[i]);

            // caso o elemento esteja vazio
            if (!_.isUndefined(element)) {
                this.listElements.push(element);
            }
        }

        // console.log('listElements', listElements);
        //ativa os layout no documento
        if (_.isUndefined(activeLayout)) {
            this.loadLayout(this.listElements, undefined, NotCreateAreaMainFirstPlayer);
        }
    }

    /**
     * Método para ativar os elementos no container
     *
     * @method loadLayout
     * @param  {Array} listElementsForce lista de elements
     * @param  {Element} ObjContainerForce Objeto Elemento onde os itens serão adicionados
     * @return {void}
     */
    ClassLayout.prototype.loadLayout = function (listElementsForce, ObjContainerForce, NotCreateAreaMainFirstPlayer) {
        // caso não seja passado nenhum elemento
        if (_.isUndefined(listElementsForce)) {
            listElementsForce = this.listElements;
        }

        // caso seja passado outro container
        if (_.isUndefined(ObjContainerForce)) {
            ObjContainerForce = this.ObjContainer;
        }

        // caso não exista o objeto container
        if (_.isUndefined(ObjContainerForce) || _.isNull(ObjContainerForce)) {
            _logError('Nenhum container foi reportado.');
            return;
        }

        // console.log('ObjContainerForce', ObjContainerForce);

        // adiciona elementos ao container
        var i, notCreate;
        for (i in listElementsForce) {
            notCreate = false;
            // caso não seja para criara área principal
            if (NotCreateAreaMainFirstPlayer && listElementsForce[i].getMain()) {
                notCreate = true;
            }

            ObjContainerForce.appendChild(listElementsForce[i].createElement(notCreate));
        }
    }

    /**
     * Método para resetart as variáveis do layout
     *
     * @method resetLayout
     * @return {void}
     */
    ClassLayout.prototype.resetLayout = function (removeAreaMain) {
        this.jsonAllLayout = undefined;
        this.jsonLayout = undefined;
        var i;

        // inativa as classes
        for (i=0; i<this.listElements.length; i++) {
            // this.listElements[i].stop();
        }

        // apaga todos os elementos
        var countArea = this.ObjContainer.children.length;
        for (i = 0 ; i < countArea; i++) {
            // remover o conteúdo da div container
            var area = this.ObjContainer.children[1];

            // caso não seja para excluir a área principal
            if (area != undefined) {
                if (removeAreaMain && area.getAttribute('data-principal') == "true") {
                    continue;
                }

                area.remove();
            }
        }

        this.listElements = _.filter(this.listElements, function _filter(classLayoutArea) {
            return classLayoutArea.getMain();
        });

        // this.idElements = undefined;
    }

    /**
     * Método para retornar a área principal
     *
     * @method getAreaMain
     * @return {Element} Elemento html da área
     */
    ClassLayout.prototype.getAreaMain = function () {
        var element = _.find(this.listElements, function(element) { return element.getMain(); });

        if (_.isUndefined(element)) {
            return;
        }

        return element.getElementHtml();
    }

    /**
     * Método para retornar as áreas auxiliares
     *
     * @method getAreasAux
     * @return {array} Elemento html da área
     */
    ClassLayout.prototype.getAreasAux = function () {
        var elements = _.filter(this.listElements, function(element) { return !element.getMain(); });

        if (_.isEmpty(elements)) {
            return;
        }

        return elements;
    }

    /**
     * Método para retornar um elemento
     *
     * @method getElement
     * @param  {string} id Id do elemento
     * @return {Element}
     */
    ClassLayout.prototype.getElement = function (id) {
        element = _.find(this.listElements, function(element) {element.id === id});

        if (!_.isUndefined(element)) {
            return element;
        }

        return;
    }

    /**
     * Método para setar o objeto com todos os layouts
     *
     * @method setJsonAllLayout
     * @param {json} jsonLayout
     */
    ClassLayout.prototype.setJsonAllLayout = function (jsonLayout) {

        if (!_.isObject(jsonLayout)) {
            return _logError('Conteudo especificado é inválido.');
        }

        if (!('layouts' in jsonLayout) && !('default' in jsonLayout.layouts)) {
            return _logError('Json não está no formato correto.', jsonLayout);
        }

        // reseta dados do layout
        // this.resetLayout();

        // seta json
        this.jsonAllLayout = jsonLayout;

        // seleciona o layout padrão
        // this.setLayout(this.jsonAllLayout.layouts.default);
    }

    /**
     * Método para setar o id do layout a ser aplicado
     *
     * @method setLayout
     * @param {json} id
     */
    ClassLayout.prototype.setLayout = function (id) {

        // caos não seja informado um id
        if (!id) {
            return;
        }

        var jsonItem = _.findWhere(this.jsonAllLayout.layouts.layout, {"id": id+""});

        if (_.isUndefined(jsonItem)) {
            return _logError('Não foi possível encontrar o layout especificado.');
        }

        this.jsonLayout = jsonItem;
    }

    /**
     * Método para retornar o objeto do layout
     *
     * @method getLayout
     * @return {json}
     */
    ClassLayout.prototype.getLayout = function() {

        if (_.isUndefined(this.jsonLayout)) {
            _logInfo('Nenhum layout definido.');
            return undefined;
        }

        return this.jsonLayout;
    }

    /**
     * Método para retornar o id do layout selecionado
     *
     * @method getIdLayout
     * @return {integer} Id do layout
     */
    ClassLayout.prototype.getIdLayout = function () {
        var json = this.getLayout();
        return (!_.isUndefined(json)? json.id : 0);
    }

    window.ClassLayout = ClassLayout;
}) (window);
