(function() {

	var currentDownloads = [];
    var pendingDownloadsMedia = [];
    var pendingDownloadsMediaAudio = [];
    var pendingDownloadsLayoutXml = [];
	var pendingDownloadsLayoutTemplate = [];
	var pendingDownloadsGeneric = [];

    var genericsDownload = [];
    var layoutXmlDownload;
    var layoutTemplateDownload;
    var mediasDownload;
    var mediasAudioDownload;

    var timeOutGetDownload;

    /**
     * Classe para gerenciar todos os downloads de conteúdo
     *
     * @class DownloadCenter
     * @param {[type]} env4YouSee     [description]
     * @param {string} projPath       Caminho da pasta do projeto
     * @param {Directory} rootDirEntry   Classe com o diretório raiz do projeto
     * @param {ManagerConnect} managerConnect Classe para gerenciar as conexões com o socket
     */
    function DownloadCenter(env4YouSee, projPath, rootDirEntry, managerConnect){
        // console.log('init DownloadCenter');

        DownloadCenter.COMPLETED = 'COMPLETED';
        DownloadCenter.NOT_FOUND = 'NOT_FOUND';
        DownloadCenter.FAILED = 'FAILED';
        DownloadCenter.FOUND = 'FOUND';
        DownloadCenter.managerConnect = managerConnect;

        DownloadCenter.projPath = projPath;
        DownloadCenter.rootDirEntry = rootDirEntry;

        for (index = 0; index<4; index++) {
            genericsDownload[index]          = new GenericDownload(index, rootDirEntry, projPath, env4YouSee);
        }

        layoutXmlDownload         = new XmlDownload(index, rootDirEntry, projPath, env4YouSee);
        layoutTemplateDownload    = new LayoutTemplateDownload(index, rootDirEntry, projPath, env4YouSee);
        mediasDownload            = new MediaDownload(index, rootDirEntry, projPath, env4YouSee);
        mediasAudioDownload       = new MediaAudioDownload(index, rootDirEntry, projPath, env4YouSee);

        // adiciona uma instancia para download
        DownloadCenter.getDownload();

        // quantidade máxima de download simultâneos
        DownloadCenter.downloadsCurrent = 4;

        // download do conteúdo
        // A cada 5 segundos ele será executado
        // Está sendo utilizado o throttle, para que o player não fique esperando que todos os arquivos sejam processados
        // para que somente depois o download inicie
        // NÃO PODE SER MENOR QUE 5 SEGUNDO, SENÃO O MESMO ARQUIVO SERÁ FEITO O DOWNLOAD MAIS DE 1 VEZ
        DownloadCenter.throttleDownload = _.throttle(DownloadCenter.getDownload, 5000);
	}

	DownloadCenter.addDownload = function(downloadItem){
		if(downloadItem != undefined){
			switch(downloadItem.type){
				case FileSync.FILE_TYPE_MEDIA:

					if(
                        (__.findWhere(pendingDownloadsMedia, {"file": downloadItem.file}) == undefined) &&
						(__.findWhere(currentDownloads, {"file": downloadItem.file}) == undefined)
                    ) {
                    	pendingDownloadsMedia[pendingDownloadsMedia.length] = downloadItem;

                        // semaforo para download das medias
                        Download.media = true;

						_log('Add to download: ', downloadItem.file);
					} else {
						// console.log('Item já está na lista de downloads');
                    }

					break;

                case FileSync.FILE_TYPE_AUDIO:

                    if(
                        (__.findWhere(pendingDownloadsMediaAudio, downloadItem) == undefined) &&
                        (__.findWhere(currentDownloads, downloadItem) == undefined)
                    ){
                        pendingDownloadsMediaAudio[pendingDownloadsMediaAudio.length] = downloadItem;

                        // semafaro para download da playlist de música
                        Download.audio = true;

                        _log('Add to download: ', downloadItem.file);

                    }else {
                        // console.log('Item já está na lista de downloads');
                    }

                    break;

                // Download dos arquivo xml do layout
                case FileSync.FILE_TYPE_XML:

                    if(
                        (__.findWhere(pendingDownloadsLayoutXml, downloadItem) == undefined) &&
                        (__.findWhere(currentDownloads, downloadItem) == undefined)
                    ){
                        pendingDownloadsLayoutXml[pendingDownloadsLayoutXml.length] = downloadItem;

                        // ativa o dowload
                        Download.xml = true;

                        _log('Add to download: ', downloadItem.file);
                    }else {
                        // console.log('Item já está na lista de downloads');
                    }

                    break;

                // Downlaod dos templates do layout
                case FileSync.FILE_TYPE_LAYOUT_TEMPLATE:

                    if(
                        (__.findWhere(pendingDownloadsLayoutTemplate, downloadItem) == undefined) &&
                        (__.findWhere(currentDownloads, downloadItem) == undefined)
                    ){
                        pendingDownloadsLayoutTemplate[pendingDownloadsLayoutTemplate.length] = downloadItem;

                        // ativa download
                        Download.layout = true;

                        // logs
                        _log('Add to download: ', downloadItem.file);
                    }else {
                        // console.log('Item já está na lista de downloads');
                    }

                    break;

				default:
					if( (__.findWhere(pendingDownloadsGeneric, {"file": downloadItem.file}) == undefined) &&
						(__.findWhere(currentDownloads, {"file": downloadItem.file}) == undefined) ){

						pendingDownloadsGeneric[pendingDownloadsGeneric.length] = downloadItem;
						_log('Add to download: ', downloadItem.file);
					}else {
						// console.log('Item já está na lista de downloads');
                    }

					break;
			}

            // _log('Add to download: ', downloadItem.file);

            // download do conteúdo
            DownloadCenter.throttleDownload();
		}
	};

	/**
	 * Metodo para remover um item da lista de downloads em andamentos
     *
     * @method remove
	 * @param downloadItem Item a ser removido da lista
	 * @param downloadCompleted Indica se o download foi concluído
	 */
	DownloadCenter.remove = function(downloadItem, downloadStatus){
		// logs
        console.log('remove downloadItem ' , downloadItem.file);

		currentDownloads = __.without(currentDownloads, downloadItem);

        // caso o donwload tenha sido completado com sucesso
        if(downloadStatus == DownloadCenter.COMPLETED){

            // verifica se é do tipo institucional
			if(downloadItem.type == FileSync.FILE_TYPE_MEDIA){
                // Enviar playlog do tipo DownEnd apenas para conteúdos
				DownloadCenter.managerConnect.log.save('playerlog',downloadItem.file, 'DOWN_END');
			}

            // caso seja para notificar via php
			if(downloadItem.notify){
				// Enviar notificação via on_message_complete
				DownloadCenter.managerConnect.onMessageComplete(downloadItem.file);
			}

        } else if(downloadStatus == DownloadCenter.NOT_FOUND){
            if(downloadItem.notify){
                // Enviar notificação via on_message_completE
                DownloadCenter.managerConnect.onMessageComplete(downloadItem.file);
            }
		} else if(downloadStatus == DownloadCenter.FOUND){
            // caso o arquivo já esteje em disco


        } else if(downloadItem.trying < 10 ){
            // aguarda 10 segundos para colocar na lista de novo
            setTimeout(function() {
                downloadItem.trying = downloadItem.trying + 1;
                DownloadCenter.addDownload(downloadItem);
            }, 5000);
        }
	};

    /**
     * Semaforos para download do conteúdo
     * @type {Object}
     */
    var Download = {
        audio: false,
        media: false,
        layout: false,
        xml: false,
        generic: false
    };

    /**
     * Método para realizar o download das depedências
     *
     * @method getDownload
     * @return {void}
     */
    DownloadCenter.getDownload = function () {
        var that = this;

        // GAMB MUITO LOUCA PARA RESOLVER PROBLEMA DE DOWNLOAD DA SAMSUNG
        // if (!player4uc.download.DOWNLOADER_PLUGIN.length === player4uc.download.countPluginDownloader) {
        //     currentDownloads = [];
        // }

        // lista de arquivos para download
        // async.parallel([
        async.series([
            ///////////////////////
            // playlist de áudio //
            ///////////////////////
            // that.getDownloadAudio,

            //////////////////////////
            // arquivos de template //
            //////////////////////////
            that.getDownloadLayoutTemplate,

            ////////////////////////
            // playlist de videos //
            ////////////////////////
            this.getDownloadMedia,

            ////////////////////////////
            // arquivos xml do layout //
            ////////////////////////////
            this.getDownloadLayoutXML
        ], function doneGetDownloadAll (error) {

            // eventos para avisar que todos os arquivos foram feitod download
            EventDownload.trigger('finishDownloadFiles');
        });
    };

    /**
     * Método para realizado o download de todos os arquivos de media para playlist de áudio
     *
     * @method getDownloadAudio
     * @param  {function} endCallback
     * @return {void}
     */
    DownloadCenter.getDownloadAudio = function (endCallback) {
        // caso não seja para executar
        if (!Download.audio) {
            return endCallback();
        }

        var that = this;

        // verifica array de medias
        async.each(pendingDownloadsMediaAudio,
            function trackAudioDownload (item, callback) {
                // arquivo para donwload
                var itemDownload = DownloadCenter.getMediaAudioDownloadItem();

                // semafaro
                Download.audio = false;

                // download
                mediasAudioDownload.download(itemDownload, function doneDownload(file, status) {
                    // remove o arquivo da list de download
                    DownloadCenter.remove(file, status);

                    // callback
                    callback();
                });

                // logs
                // console.log('TAG_MAYCON DownloadCenter getDownload: item', item.file);
            },
            function doneAudioDownload (error) {
                // quando não existe mais arquivos para download dispara o evento
                EventDownload.trigger('resetPlayList');

                // semafaro
                Download.audio = true;

                // callback
                if (!_.isUndefined(endCallback)) endCallback();

                // logs
                // console.log('TAG_MAYCON DownloadCenter getDownload doneAudioDownload:');
            }
        );
    }

    /**
     * Método para realizado o download dos arquivos de media da playlist principal
     *
     * @method getDownloadMedia
     * @param  {function} endCallback
     * @return {void}
     */
    DownloadCenter.getDownloadMedia = function (endCallback) {
        // caso esteja bloqueado
        if (!Download.media || !pendingDownloadsMedia.length) {
            return endCallback();
        }

        var that = this;

        // executa lista de download
        async.eachLimit(pendingDownloadsMedia,
            DownloadCenter.downloadsCurrent,
            function trackMediaDownload (item, callback) {
                // arquivo para donwload
                var itemDownload = DownloadCenter.getMediaDownloadItem();

                // semafaro
                Download.media = false;

                // download
                mediasDownload.download(itemDownload, function doneDownload(file, status) {
                    // remove o arquivo da list de download
                    DownloadCenter.remove(file, status);

                    // callback
                    callback();
                });

                // logs
                // console.log('TAG_MAYCON DownloadCenter getDownload: item', item.file);
            },
            function doneMediaDownload (error) {
                // quando não existe mais arquivos para download dispara o evento
                EventDownload.trigger('resetPlayListMedia');

                // semafaro
                Download.media = true;

                // callback
                if (!_.isUndefined(endCallback)) endCallback();

                // logs
                // console.log('TAG_MAYCON DownloadCenter getDownload doneMediaDownload:');
            }
        );
    }

    /**
     * Método para realizado o download dos arquivos dos templates
     *
     * @method getDownloadLayoutTemplate
     * @param  {function} endCallback
     * @return {void}
     */
    DownloadCenter.getDownloadLayoutTemplate = function (endCallback) {

        // caso esteja bloqueado
        if (!pendingDownloadsLayoutTemplate.length) {
            return endCallback();
        }

        var that = this;

        // executa lista de download
        async.eachLimit(pendingDownloadsLayoutTemplate,
            DownloadCenter.downloadsCurrent,
            function LayoutTemplateDownload (item, callback) {
                // arquivo para donwload
                var itemDownload = DownloadCenter.getLayoutTemplateDownloadItem();

                // semafaro
                Download.layout = false;

                // download
                layoutTemplateDownload.download(itemDownload, function doneDownload(file, status) {
                    // remove o arquivo da list de download
                    DownloadCenter.remove(file, status);

                    // callback
                    callback();
                });

                // logs
                // console.log('TAG_MAYCON DownloadCenter getDownloadLayoutTemplate: item', item.file);
            },
            function doneGetDownloadLayoutTemplate (error) {
                // quando não existe mais arquivos para download dispara o evento
                EventDownload.trigger('finishDownloadFilesLayoutTemplate');

                // semafaro
                Download.layout = true;

                // callback
                endCallback();

                // logs
                // console.log('TAG_MAYCON DownloadCenter getDownloadLayoutTemplate doneGetDownloadLayoutTemplate:');
            }
        );
    }

    /**
     * Método para realizado o download das fontes do layout
     *
     * @method getDownloadLayoutXML
     * @param  {function} endCallback
     * @return {void}
     */
    DownloadCenter.getDownloadLayoutXML = function (endCallback) {
        // caso esteja bloqueado
        if (!Download.xml || !pendingDownloadsLayoutXml.length) {
            return endCallback();
        }

        var that = this;

        // executa lista de download
        async.each(pendingDownloadsLayoutXml,
            function LayoutXMLDownload (item, callback) {
                // arquivo para donwload
                var itemDownload = DownloadCenter.getXmlDownloadItem();

                // semafaro
                Download.xml = false;

                // download
                layoutXmlDownload.download(itemDownload, function doneDownload(file, status) {
                    // remove o arquivo da list de download
                    DownloadCenter.remove(file, status);

                    // callback
                    callback();
                });

                // logs
                // console.log('TAG_MAYCON DownloadCenter LayoutXMLDownload: item', item.file);
            },
            function doneGetDownloadLayoutXML (error) {
                // quando não existe mais arquivos para download dispara o evento
                // EventDownload.trigger('getDownloadLayoutXML');

                // Atualiza as notícias dos templates
                EventLayout.trigger('updatePlayListNewsLayout');

                // semafaro
                Download.xml = true;

                // callback
                endCallback();

                // logs
                // console.log('TAG_MAYCON DownloadCenter LayoutXMLDownload doneGetDownloadLayoutXML:');
            }
        );
    }

    /**
     * Método para controlar o timeout do contúedo
     *
     * @method timeOutDownlaod
     * @return {void}
     */
    function timeOutDownlaod() {
        clearTimeout(timeOutGetDownload);
        timeOutGetDownload = setTimeout(DownloadCenter.getDownload, 100000);
    }

    /**
     * Método para retornar o próximo download dos xmls
     *
     * @method getXmlDownloadItem
     * @return {XmlDownload}
     */
    DownloadCenter.getXmlDownloadItem = function(){
        if(pendingDownloadsLayoutXml.length > 0){
            var downloadItem = __.first(pendingDownloadsLayoutXml);
            pendingDownloadsLayoutXml = __.rest(pendingDownloadsLayoutXml);
            currentDownloads[currentDownloads.length] = downloadItem;

            return downloadItem;
        }else{
            return null;
        }
    };

    /**
     * Método para retornar o próximo download dos templates do layout
     *
     * @method getLayoutTemplateDownloadItem
     * @return {LayoutTemplateDownload}
     */
    DownloadCenter.getLayoutTemplateDownloadItem = function(){
        if(pendingDownloadsLayoutTemplate.length > 0){
            var downloadItem = __.first(pendingDownloadsLayoutTemplate);
            pendingDownloadsLayoutTemplate = __.rest(pendingDownloadsLayoutTemplate);
            currentDownloads[currentDownloads.length] = downloadItem;

            return downloadItem;
        }else{
            return null;
        }
    };

    /**
     * Método para retornar o próximo download da playlist de música
     *
     * @method getMediaAudioDownloadItem
     * @return {MediaAudioDownload}
     */
    DownloadCenter.getMediaAudioDownloadItem = function(){
        if(pendingDownloadsMediaAudio.length > 0){
            var downloadItem = __.first(pendingDownloadsMediaAudio);
            pendingDownloadsMediaAudio = __.rest(pendingDownloadsMediaAudio);
            currentDownloads[currentDownloads.length] = downloadItem;

            return downloadItem;
        }else{
            return null;
        }
    };


    /**
     * Método para retornar o próximo item de media
     *
     * @method getMediaDownloadItem
     * @return {MediaDownload}
     */
	DownloadCenter.getMediaDownloadItem = function(){
		if(pendingDownloadsMedia.length > 0){
			var downloadItem = __.first(pendingDownloadsMedia);
			pendingDownloadsMedia = __.rest(pendingDownloadsMedia);
			currentDownloads[currentDownloads.length] = downloadItem;

			return downloadItem;
		}else{
			return null;
		}
	};

    /**
     * Método para retornar o próximo item generico
     * @method getGenericDownloadItem
     * @return {[GenericDownload]}
     */
	DownloadCenter.getGenericDownloadItem = function(){
		if(pendingDownloadsGeneric.length > 0){
			var downloadItem = __.first(pendingDownloadsGeneric);
			pendingDownloadsGeneric = __.rest(pendingDownloadsGeneric);
			currentDownloads[currentDownloads.length] = downloadItem;

			return downloadItem;
		}else{
			return null;
		}
	};

	window.DownloadCenter = DownloadCenter;
})();
