// scripts/Event/EventLayout.js

(function() {
'use strict';

    /**
     * Objeto para gerenciar os eventos de layout
     *
     * @class EventLayout
     * @type {Object}
     */
    var EventLayout = {};

    // aplica o controle de evento ao objeto
    Evento.applyTo(EventLayout);

    /////////////
    // eventos //
    /////////////

    function updatePlayListNewsLayout(resetLayout) {
        var listWindow = player4uc.window.getAll()['mainWindow-1'];
        var i, i2, classLayoutTmp, arrayAreasAuxTmp, mediaFileTmp;


        // caso seja para resetar o layout
        if (resetLayout) {
            EventLayout.trigger('loadLayout', [listWindow, undefined, undefined, 1]);
        }

        classLayoutTmp = listWindow.contentWindow.ClassLayout;

        // caso a classe esteja indefinida
        if (classLayoutTmp === undefined) {
            // logs
            var msgErro = 'EventLayout.updatePlayListNewsLayout: Classe de template não foi definida.';
            _log(msgErro);

            return;
        }

        arrayAreasAuxTmp = classLayoutTmp.getAreasAux();

        // cria conteúdo para nova área
        for (i2 in arrayAreasAuxTmp) {

            // seta o novo container
            // função da variável classRssPlayerLayout, registrada no arquivo background
            classRssPlayerLayout[i2].setContainer(arrayAreasAuxTmp[i2].getElementHtml());

            // caminho do arquivo
            mediaFileTmp = 'rss.' + arrayAreasAuxTmp[i2].getFonte() + '.xml'

            // seta o arquivo de playlist
            classRssPlayerLayout[i2].setPlayList(mediaFileTmp);
        }
    }

    var updatePlayListNewsLayoutDebounce = _.debounce(updatePlayListNewsLayout, 1000);

    /**
     * Método para atualizar a playlist de notícias dos layouts
     *
     * @event updatePlayListNewsLayout
     * @param  {boolean} resetLayout  caso seja para resetar o layout
     * @return {void}
     */
    EventLayout.bind('updatePlayListNewsLayout', function EventUpdatePlayListNewsLayout(resetLayout) {

        // chama a função
        updatePlayListNewsLayoutDebounce(resetLayout);

        // console.info('EventLayout updatePlayListNewsLayout: resetLayout', resetLayout);
    });



    /**
     * Evento para atualizar o layout na janela
     *
     * @event loadLayout
     * @param {AppWindown} appWindow
     * @param {integer} layoutId Id do layout
     */
    EventLayout.bind('loadLayout', function EventloadLayout(AllArguments) {
        // caso seja enviado um array
        var appWindow = AllArguments;
        if (_.isArray(AllArguments)) {
            appWindow = AllArguments[0];
            var layoutId = AllArguments[1];
            var callback = AllArguments[2];
            var force = AllArguments[3];
        }

        // player
        var playerData = appWindow.contentWindow.playerData;

        // atualiza o template
        // método está no arquivo background.js
        applyLayoutWindow (appWindow, playerData, layoutId, function doneChangeLayoutCallback() {
            // callback
            if (typeof callback == 'function') {
                callback(appWindow);
            }

            _log('Layout já aplicado.', layoutId);
        }, force);

        // console.info('EventLayout loadLayout: AllArguments', AllArguments);
    });


    /**
     * Método para recarregar um template do layout
     *
     * @event loadTemplate
     * @param {[type]} template
     */
    EventLayout.bind('loadTemplate', function EventloadTemplate(keyArea) {
        // reseta o somente á area
        // variável está no arquivo background
        classRssPlayerLayout[keyArea].resetContainer(keyArea);
    });


    function throttleProcessFile(idPlayer) {
        var filePlayer = FileSyncLayoutTemplate.getNameFile(idPlayer);
        sync.processFileTemplate([filePlayer], function doneProcessFileTemplate() {
            console.log('Arquivo de layout processado.');
        });

        // console.info('EventLayout processFileLayout');
    }

    // aguarda por 10 segundos
    // É preciso que se utilize Throttle, para que o player possa baixar
    // todos os arquivos. Caso utilize debounce, quando o player entender que o template
    // deu problema ele irá reiniciar, e ai o debounce não vai ser executado
    var execThrottleProcessFile = _.throttle(throttleProcessFile, 5000);

    /**
     * Evento para processar o arquivo de layout
     *
     * @event processFileLayout
     * @param {intever} idPlayer id do player
     */
    EventLayout.bind('processFileLayout', function _EventloadTemplate(idPlayer) {

        // chama a função
        execThrottleProcessFile(idPlayer);
    });

    /**
     * Método para excluir arquivos desnecessários do layout
     *
     * @event verifyFilesLayoutForDelete
     * @return {void}
     */
    EventLayout.bind('verifyFilesLayoutForDelete', function _EventVerifyFilesLayoutForDelete() {

        // ler todos os arquivos de layout.<>.json da pasta json
        // unir todos os templates e fontes de todos os layouts de todos os players
        // verificar quais arquivos são válidos

        FileSyncLayoutTemplate.verifyFilesLayoutForDelete();

        // console.info('EventLayout verifyFilesLayoutForDelete');
    });

    /**
     * Evento para verificar o arquivo list local do projeto se o player está projeto para iniciar
     * @param  {integer} AllArguments Se deve ou não baixar arquivo list novamente
     * @return {void}
     */
    EventLayout.bind('checkProject', function _EventChangeFileMedia(AllArguments) {

        var isUpdate = AllArguments;

        // verifica todo o projeto
        sync.init(isUpdate);
    });

    EventLayout.bind('changeFileMedia', function _EventChangeFileMedia(AllArguments) {
        // troca o arquivo media ser tocado
        var day = AllArguments;
        var player = undefined;
        if (_.isArray(AllArguments)) {
            day = AllArguments[0];
            player = AllArguments[1];
        }

        // Busca todos os layouts
        var windowPlayer = player4uc.window.getAll()['mainWindow-1'];

        // troca o arquivo media
        windowPlayer.playlistManager.loadFiles(day, function _doneLoadFiles() {
            // verifica se existe arquivos novos para o layout
            EventLayout.trigger('checkProject', true);

            // reseta a lista de itens a serem tocados
            playlistManager.resetPlaylistCurrent();
        });
    });

   window.EventLayout = EventLayout;
}) (window);
