'use strict';

/*
TODO:

FileSync:
	Sincroniza os arquivos locais com os arquivos do GoogleDrive - OK
	- Precisa receber o nome do projeto. - OK
	- Verifica se o projeto existe no Google Drive e se não existir dispara erro - OK
	- Se existir o projeto, busca os arquivos da pasta json e da pasta videos (futuramente outras) - OK
	- Enquanto percorre os arquivos, monta na memória a estrutura atual de arquivos no Drive - OK
	- Para cada pasta encontrada, percorre todos os arquivos e coloca para baixar para a pasta local - OK
Falta
		- Antes de baixar, verifica a última atualização do arquivo e só baixa se estiver desatualizado
	- Após preparar a estrutura de arquivos do Drive percorre os arquivos locais
	- Monta a estrutura atual dos arquivos locais
	- Verifica se há arquivos locais que não são necessários e apaga
*/


if (typeof FileSync !== "undefined") {
	// console.log(FileSync);
	// throw new Error('FileSync API already defined.');
}

(function() {

	function fileErrorHandler(e, context) {
	  console.error('Error: ', e.name, e.message);
	  console.error('Context: ', context);
	  // throw new Error(e.message);
	}

	/**
	 * Class to handle the synchronization with Google Drive
	 */
	function DriveFileSync(projectName, player) {
		console.log('DriveFileSync', projectName);

		this.parent.constructor.call(this, projectName);

		var that = this;

		/**
		 * @var GoogleDriveFileObject projectDir
		 */
		this.projectDir = null;

		this.checkProject(function(projectDir) {
			that.getProjectFileTree.call(that, projectDir);
		});
	}
	// extends
	// DriveFileSync.inheritsFrom(FileSync);


	DriveFileSync.prototype.checkProject = function(callback) {
		var that = this;
		console.log('searching drive for ' + this.projectName + " folder");
		gapi.client.drive.files.list({
				q: "title = '" + this.projectName + "'",
				maxResults: 2
			}, function rootDirReturn(json, text) {
				if (!json || !('items' in json) || !json.items.length) {
					throw new Error(chrome.i18n.getMessage("cantFindProjectFolder"));
				}
				that.projectDir = json.items[0];
				if (callback)
					callback(that.projectDir);
			}
		);
	};

	/**
	 * Walks through the project dir to get the project file tree
	 * @param GoogleDriveFileObject projectDir
	 */
	DriveFileSync.prototype.getProjectFileTree = function(projectDir) {
		var titleQuery = "title = 'json' or title = 'videos' or title = 'project.json'";
		var that = this;
		var totalItens = 0;


		function finished(itemIdx) {
			console.log("========================", itemIdx, totalItens);
			if (itemIdx == totalItens) {
				// console.log(JSON.stringify(that.projectTree));
				console.log('finished loading Drive Tree');
				console.log('Syncing the project tree');
				that.syncProjectFiles();
			}
		}

		console.log('projectDir', projectDir);
		console.log('searching videos and json folders');
		gapi.client.drive.children.list(projectDir.id, {q: titleQuery}, function(json, text) {
			if (!json || !json.items.length) {
				throw new Error(chrome.i18n.getMessage("cantFindDriveFileOrFolder"));
			}

			totalItens = json.items.length;
			var finishing = 0;

			for (var i = 0; i < json.items.length; i++) {
				that.getMetadata.call(that, json.items[i], function(json) {
					that.setProjectFile.call(that, json, function() {
						finished(++finishing);
					});

					// finished(++finishing);
				});
			}
		});
	};

	DriveFileSync.prototype.setProjectFile = function(metadata, finishCallback) {
		var isDir = metadata.mimeType == DRIVE_DIR;
		this.setProjectTreeLeaf(metadata.title, this.projectName, isDir);
		if (isDir) {
			this.mapSubdirTree(metadata, this.projectName + "/" + metadata.title, finishCallback);
		} else {
			this.setFileToSync(metadata, this.projectName);
			finishCallback();
		}
	};

	/**
	 * A recursive method that maps the subdirs tree
	 */
	DriveFileSync.prototype.mapSubdirTree = function(subdirObj, subdirPath, finishCallback) {
		var that = this;
		gapi.client.drive.children.list(subdirObj.id, {}, function(json, text) {
			if (!json || !json.items.length) {
				console.log("+++++++", json, subdirObj);
				throw new Error(chrome.i18n.getMessage("cantFindDriveFileOrFolder") + " " + subdirObj.title);
			}

			var totalItems = json.items.length;
			var currentItem = 0;
			// for each file/dir, gets its type and fills in the project tree array
			for (var i = 0; i < json.items.length; i++) {
				that.getMetadata.call(that, json.items[i], function(json) {
					currentItem++;
					console.log('currentItem', currentItem, 'totalItems', totalItems);
					that.mapSubdirFile.call(that, json, subdirPath, totalItems == currentItem ? finishCallback : null);
				});
			}
		});
	};

	DriveFileSync.prototype.mapSubdirFile = function(metadata, subdirPath, finishCallback) {
		if (metadata.mimeType == DRIVE_DIR) {
			this.setProjectTreeLeaf(metadata.title, subdirPath, true);

			this.mapSubdirTree(metadata, subdirPath + "/" + metadata.title);
		} else {
			this.setProjectTreeLeaf(metadata.title, subdirPath, false);
			this.setFileToSync(metadata, subdirPath);

			if (finishCallback) {
				finishCallback();
			}
		}
	};

	/**
	 * gets file metadata on Google Drive
	 */
	DriveFileSync.prototype.getMetadata = function(driveObj, callback) {
		var that = this;
		gapi.client.drive.files.get(driveObj.id, {}, function(json, text) {
			if (!json) {
				throw new Error(chrome.i18n.getMessage("cantFindDriveFileOrFolder"));
			}
			callback.call(that, json);
		});
	};

	/**
	 * Creates the project structure locally, with directories and files
	 * Download the files from google drive
	 */
	DriveFileSync.prototype.syncProjectFiles = function() {
		var that = this;
		this.createProjectDirs(createdProjectDirs);

		function AfterDownload(fileToSync, rootDirEntry) {
			this.fileToSync = fileToSync;
			this.rootDirEntry = rootDirEntry;

			var that = this;

			this.afterDownloadCB = function(response, rawResp) {
				if(typeof rawResp == 'string')
					rawResp = JSON.parse(rawResp);

				writeDownloadedFile(that.rootDirEntry, that.fileToSync.path, that.fileToSync.file.title, response, rawResp.gapiRequest.data.contentType);
			};
		}

		function createdProjectDirs(rootDirEntry) {
			// must to search project files and try to download and write the file
			for (var i = 0; i < that.filesToSync.length; i++) {
				var ad = new AfterDownload(that.filesToSync[i], rootDirEntry);
				gapi.client.drive.files.download(that.filesToSync[i].file, ad.afterDownloadCB);

			}
		}

		function writeDownloadedFile(rootDirEntry, path, fileName, fileContent, contentType) {
			console.log('------- ', rootDirEntry, path, fileName, fileContent, contentType);
			console.log('falhou aqui <--></-->')
			var pathParts = path.split('/');
			if (path !== "") {
				rootDirEntry.getDirectory(pathParts[0], {create: true}, function(dirEntry) {
					writeDownloadedFile(dirEntry, pathParts.slice(1).join("/"), fileName, fileContent, contentType);
				});
			} else {
				rootDirEntry.getFile(fileName, {create: true}, function(fileEntry) {
					fileEntry.createWriter(function(fileWriter) {
						var blob = new Blob([fileContent], {type: contentType});
						fileWriter.onwriteend = function(e) {
							console.log('finished to write the file ' + fileName + ' with type ' + contentType);

							// broadcast to other parts of the system that a new file was downloaded
							chrome.runtime.sendMessage({"onfiledownload": fileName});
						};
						fileWriter.onwriteerror = function(e) {
							console.log('Write failed: ' + e.toString());
						};
						fileWriter.write(blob);
					});
				});
			}
		}
	};

	DriveFileSync.prototype.createProjectDirs = function(callback) {
		console.log('createProjectDirs -------------------- ')
		var that = this;
		var appDir;
		chrome.storage.local.get(['settings.appDir'], gotAppDir);

		// callback called when storage.local.get() returns
		function gotAppDir(storageObj) {
			if (typeof storageObj['settings.appDir'] !== "undefined") {
				appDir = storageObj['settings.appDir'];
				chrome.fileSystem.isRestorable(appDir, appDirIsRestorable);
			}
		}

		// callback called when isRestorable is called
		function appDirIsRestorable(isRestorable) {
			if (isRestorable) {
				// when the appDir is restorable, we can open and start writing files
				chrome.fileSystem.restoreEntry(appDir, appDirRestored);
			} else {
				// get the user back to the configuration screen
				window.startupScreen();
				chrome.app.window.current().close();
			}
		}

		// callback called when restoreEntry returns
		function appDirRestored(appDirEntry) {
			createDirectoryStructure(appDirEntry, that.projectTree, finishedCreatingDirStructure);
		}

		// creates all the directory tree structure
		function createDirectoryStructure(rootDirEntry, folders, finished) {
			if (typeof folders === "undefined") {
				folders = that.projectTree;
			}
			if (!folders['dir']) {
				// throw new Error("foldersRootNoDir");
				console.log("foldersRootNoDir" + " ", folders);
				return;
			}
			rootDirEntry.getDirectory(folders['dir'], {create: true}, function(dirEntry) {
				if (folders.files.length) {
					for (var i = 0; i < folders.files.length; i++) {
						if (typeof folders.files[i].dir !== "undefined") {
							createDirectoryStructure(dirEntry, folders.files[i]);
						}
					}
				}
				if (typeof finished !== "undefined") {
					finished(rootDirEntry);
				}
			});
		}

		function finishedCreatingDirStructure(rootDirEntry) {
			console.log("wrote all the needed directories ");
			callback(rootDirEntry);
		}
	};


	window.DriveFileSync = DriveFileSync;

})();
