/*
TODO:

FileSync:
    Sincroniza os arquivos locais com os arquivos do GoogleDrive - OK
    - Precisa receber o nome do projeto. - OK
    - Verifica se o projeto existe no Google Drive e se não existir dispara erro - OK
    - Se existir o projeto, busca os arquivos da pasta json e da pasta videos (futuramente outras) - OK
    - Enquanto percorre os arquivos, monta na memória a estrutura atual de arquivos no Drive - OK
    - Para cada pasta encontrada, percorre todos os arquivos e coloca para baixar para a pasta local - OK
Falta
        - Antes de baixar, verifica a última atualização do arquivo e só baixa se estiver desatualizado
    - Após preparar a estrutura de arquivos do Drive percorre os arquivos locais
    - Monta a estrutura atual dos arquivos locais
    - Verifica se há arquivos locais que não são necessários e apaga
    */
if (typeof FileSync !== "undefined") {
    // console.log(FileSync);
    // throw new Error('FileSync API already defined.');
}

(function() {
    function fileErrorHandler(e, context) {
        _logError('Error: ', e.name, e.message);
        _logError('Context: ', context);
        // throw new Error(e.message);
    }

    var thatHttpFileSync;

    /**
     * ------------------------------------------------------------------
     * Class to handle the synchronization with 4YouSee
     * @class HttpFileSync
     */
    function HttpFileSync(projectName, player, playlistManager, managerConnect, athenticate, appWindow, aux)
    {

        zip.workerScriptsPath = './scripts/vendor/zip.js/';
        this.parent.constructor.call(this, projectName);
        /**
         * @var GoogleDriveFileObject projectDir
         */
         this.projectDir = null;

        this.status = 'stopped' // stopped | started
        // set objeto do player
        this.setObjectPlayer(player);

        this.playlistManager = playlistManager;
        this.managerConnect = managerConnect;
        this.appWindow = appWindow;

        // seta arquivos
        this.setNameFiles(this.player);

        // objeto instanciado
        thatHttpFileSync = this;

        // caso a classe seja somente uma auxiliar
        this.aux = aux || false;

        this.init();
    }

    // extends
    HttpFileSync.inheritsFrom(FileSync);
    HttpFileSync.prototype.init = function(isUpdate)
    {
        // objeto instanciado
        thatHttpFileSync = this;
        var that = this;
        this.isUpdate = isUpdate;


        _log("TAG_MAYCON HttpFileSync.prototype.init: isUpdate, this.status", isUpdate, this.status);

        if (this.status == 'stopped') {
            this.status = 'started';

            // agrupando os logs
            console.group("INICIANDO O CICLO");

            // verifica o arquivo list
            this.getProjectFileTree(function(playlists) {
                // logs
                // _log("TAG_MAYCON FIM getProjectFileTree");

                that.checkProject(isUpdate, function _finishCheckProject() {
                    // logs
                    // _log("TAG_MAYCON FIM checkProject");
                });
            });
        }
    };

    /**
     * Finaliza a atualização dos arquivos
     *
     * @method initFinishCallback
     * @return {void}
     */
    HttpFileSync.prototype.initFinishCallback = function()
    {

        // altera o valor da tag
        this.status = 'stopped';

        if(this.isUpdate && !this.aux){
            this.playlistManager.loadFiles();
        }

        // logs
        _log('TAG_MAYCON--- initFinishCallback');



        // fecha o agrupamento dos logs
        console.groupEnd();
    };

    /**
     * Método para seta o objeto de player
     * @param {Object} player
     */
    HttpFileSync.prototype.setObjectPlayer = function(player)
    {
        this.player = player;
    };

    /**
     * Método para setar os caminhos do arquivos a serem processados
     * @param {Object} player
     */
    HttpFileSync.prototype.setNameFiles = function(player)
    {
        this.fileList = 'list.' + player.id + '.json';
        this.urlDownloadJson =  player.url + '/common/json/';
        this.urlDownloadXml =  player.url + '/common/xml/';
    };

    /**
     * Walks through the project dir to get the project file tree
     *
     * @method getProjectFileTree
     * @param GoogleDriveFileObject projectDir
     */
    HttpFileSync.prototype.getProjectFileTree = function(callback)
    {

        var that = this;
        this.createProjectDirs(function(rootDirEntry) {
            // callback
            callback();
        });
    };

    /**
     * Método para criar todas as pastas do projeto
     *
     * @method createProjectDirs
     * @param  {Function} callback
     * @return {void}
     */
    HttpFileSync.prototype.createProjectDirs = function(callback)
    {
        var that = this;
        var appDir = window.appDir;

        // verifica a pasta de arquivos do player
        appDirRestored(appDir);

        // creates all the directory tree structure
        function createDirectoryStructure(rootDirEntry, folders, finished) {
            // console.log("TAG_MAYCON createDirectoryStructure", folders);

            if (typeof folders === "undefined") {
                folders = that.projectTree;
            }

            if (!folders['dir']) {
                // throw new Error("foldersRootNoDir");
                // console.log("foldersRootNoDir" + " ", folders);

                return finished(false);
            }

            // cria a pasta
            player4uc.filesystem.mkdir(folders['dir'], {}, finished)
        }

        // callback called when restoreEntry returns
        function appDirRestored(appDirEntry) {

            // cria a pasta principal
            createDirectoryStructure(appDirEntry, that.projectTree, function _createFolderDefault(){

                var listFolders = [
                    FileSync.PLAYER_DIR_RSS_TEMPLATE,
                    FileSync.PLAYER_DIR_RSS_IMG,
                    FileSync.PLAYER_DIR_JSON,
                    FileSync.PLAYER_DIR_MEDIA,
                    ////////////
                    // layout //
                    ////////////
                    FileSync.PLAYER_DIR_LAYOUT,
                    FileSync.PLAYER_DIR_LAYOUT + '/' + FileSync.PLAYER_DIR_XML,
                    FileSync.PLAYER_DIR_LAYOUT + '/' + FileSync.PLAYER_DIR_RSS_TEMPLATE,
                    FileSync.PLAYER_DIR_LAYOUT_RSS_IMG,

                    /////////
                    // MP3 //
                    /////////
                    FileSync.PLAYER_DIR_AUDIO
                ];

                async.eachSeries(listFolders, function _eachSeriesFolders(nameFolder, nextEach) {
                    var ObjectFolder = _.clone(that.projectTree);
                    ObjectFolder.dir = ObjectFolder.dir + '/' + nameFolder;

                    // cria a pasta
                    createDirectoryStructure(appDirEntry, ObjectFolder, nextEach);
                }, function _doneCreateFolders() {
                    // callback
                    callback(appDirEntry);
                });
            });
        }
    };

    /**
     * Método para verificar no disco se o arquivo list existe
     *
     * @method getListFromDisk
     * @param  {Function} callback
     * @return {void}
     */
    function getListFromDisk (callback)
    {
        var pathFileList = window.projectName + '/json/' + thatHttpFileSync.fileList;

        // lê o arquivo JSON
        FileSync.readJSONFile(window.rootDirEntry, pathFileList, callback);

        // console.info("finish getListFromDisk");
    }

    /**
     * Método para realizar o download do arquivo list
     *
     * @method getListWeb
     * @param  {Function} callback
     * @return {void}
     */
    function getListWeb (player, callback)
    {
        var UrlFileList = player.url + '/common/json/' + thatHttpFileSync.fileList;
        var dirPathFile = window.projectName + '/json/';

        // download do arquivo
        thatHttpFileSync.downloadFileJson(UrlFileList, dirPathFile, callback);
    }

    /**
     * Método para verificar o projeto assim que ele inicia
     *
     * @method checkProject
     * @param  {Boolean} isUpdate
     * @param  {Function}  finishCallback
     * @return {void}
     */
    HttpFileSync.prototype.checkProject = function(isUpdate, finishCallback)
    {
        var that = this;

        // processa a lista de funções
        async.waterfall (
        [
            /**
             * Método para buscar o arquivo list, em disco ou na web
             * @param  {Function} next
             * @return {void}
             */
            function _InitProcessFileList(next) {
                getListFromDisk(function _doneGetListFromDisk(listData) {
                    // logs
                    // console.info("checkProject InitProcessFileList getListFromDisk");

                    // caso exista erros
                    if (!listData || isUpdate) {
                        // será necessário realizar o download do arquivo
                        getListWeb (that.player, function _doneGetListWeb(dataJson) {
                            var error = null;

                            // em caso de erros
                            if (!dataJson) {
                                error = 500;
                            }

                            // callback
                            next(error, dataJson);
                        });

                        return;
                    }

                    // callback
                    next(null, listData);

                });
            },

            /**
             * Processa todo o arquivo list, e verifica quais precisão ser processados
             * @param  {Object}   dataJson JSON com a lista de arquivos
             * @param  {Function} next
             * @return {void}
             */
            function _processFileListImportant (dataJson, next) {

                // console.info("checkProject processFileListImportant");

                // verifica se o arquivo é um JSON
                FileSync.JSONParse(dataJson, function _doneJSONParse(dataJson) {

                    // processa arquivo list
                    thatHttpFileSync.processFilesList(dataJson, next);
                });
            },

            /**
             * Método para realizar o download dos arquivos do list
             * @param  {Array}   listArray
             * @param  {Function} next
             * @return {void}
             */
            that.downloadFilesList,

            /**
             * Método para processar todos os arquivos media.<idPlayer>.json
             * @param  {Object}   listArray JSON com a lista de arquivos válidos
             * @param  {Function} next
             * @return {void}
             */
            that.processFilesMedia,

            /**
             * Método para processar os arquivos de notícia da playlist principal
             * @param  {Object} listData
             * @param  {Function} finishCallback
             * @return {void}
             */
            that.processFileNewsPlayList,

            /**
             * Método para processar todos os arquivos media.<idPlayer>.json e categoria.<idPlayer>.json
             * @param  {Object}   listArray JSON com a lista de arquivos válidos
             * @param  {Function} next
             * @return {void}
             */
            that.processFileCarrossel,

            /**
             * Método para realizar o download dos templates e fontes dos layout
             * @param  {Array} listData
             * @param  {Function} finishCallback
             * @return {void}
             */
            that.processFileTemplate,

            /**
             * Método para processar os arquivos de notícia dos templates do layout
             * @param  {Object} listData
             * @param  {Function} finishCallback
             * @return {void}
             */
            that.processFileNewsTemplate,

            /**
             * Método para verificar quais arquivos de conteúdo não são mais necessários para o player
             * @param  {Object} listData
             * @param  {Function} finishCallback
             * @return {void}
             */
            // Foi desativada
            // Quando se tem 2 players 1 player fica apagando conteúdo do outro
            // ATUALIZADO EM 12/01: Na sprint atual o APP terá somente 1 player rodando
            that.processFileSelecao,
        ],
        function _doneCheckProject(error, result) {

            switch (error) {
                //caso o arquivo list não tenha sido processado
                case 500:
                    // para evitar um loop
                    // e trava o javascript, foi adicionado um delay de 1s
                    setTimeout(function() { that.checkProject(isUpdate, finishCallback); }, 1000);
                    break;

                // caso tudo tenha ocorrido bem
                default:
                    // semafaro para atualizações
                    thatHttpFileSync.initFinishCallback();

                    // notifica o socket do arquivo list
                    thatHttpFileSync.managerConnect.sendNotification(thatHttpFileSync.fileList, function(data) {}, true);
            }
        }
        );
    };

    /**
     * Método para realizar o download dos arquivos do list
     *
     * @method downloadFilesList
     * @param  {Array}   listArray
     * @param  {Function} next
     * @return {void}
     */
    HttpFileSync.prototype.downloadFilesList = function _HttpFileSyncDownloadFilesList (listArray, finishCallback)
    {
        var fileName = [],
            media;
        var that = thatHttpFileSync;

        // faz o download de cada arquivo listado
        async.each(listArray, function _processDownloadFilesList (file, nextEach) {
            // nome do arquivo
            fileName.push(file);

            // url do arquivo para download
            media = thatHttpFileSync.urlDownloadJson + file;

            if(file.match('selecao\.[0-9]+\.json')){
                return FileSyncSelecao.downloadFileMedia(media, nextEach);
            }

            if(file.match('rss\.player\.[0-9]+\.json')) {
                return FileSyncRss.downloadFileMedia(media, nextEach);
            }

            if(file.match('media\.[0-9]+\.[0-6]\.json')){
                return FileSyncMedia.downloadFileMedia(media, nextEach);
            }

            if(file.match('layout\.[0-9]+\.json')){
                return FileSyncLayoutTemplate.downloadFileMedia(media, nextEach);
            }

            if(file.match('categoria\.[0-9]+\.json')){
                return FileSyncCarrossel.downloadFileMedia(media, nextEach);
            }

            if(file.match('rss\.[0-9]+\.xml')){
                // caso seja notícia do template o caminho é outro
                media = file;

                return FileSyncRssNews.downloadFileMedia(media, nextEach);
            }

            // callback caso não seja nehuma opção
            nextEach();
        }, function _doneDownloadFilesList (error) {
            var i;

            // O SOCKET DEVE SER NOTIFICADO A CADA ARQUIVO QUE FOR FEITO O DOWLOAD
            // notifica o socket que baixou o arquivo
            window.managerConnect.sendNotification(fileName, function(data) {
                // console.log("TAG_MAYCON sendNotification");
            }, true);

            // callback
            finishCallback(null, listArray);
        });
    },

    /**
     * Método para definir todos os arquivos que serão validados pelo player
     *
     * @method processFilesList
     * @param  {Object} listData
     * @param  {function} finishCallback
     * @return {void}
     */
    HttpFileSync.prototype.processFilesList = function(listData, finishCallback)
    {
        // buscando os arquivos para o player baixar
        filesEssentialPlayer(listData, function _doneFilesEssentialPlayer(arrayFiles) {
            // callback
            finishCallback(null, arrayFiles);
        });

        /**
         * Método para retornar quais são os arquivos essenciais para que o player funcione
         * @param  {Function} callback
         * @return {void}
         */
        function filesEssentialPlayer (medias, callback) {
            //Lista de arquivos essenciais ao player
            var essentialFiles = [
                'layout.' + thatHttpFileSync.player.id + '.json',  // arquivo com as configurações do layout
                'rss.player.' + thatHttpFileSync.player.id + '.json',
                // o arquivo seleção deve ser baixado somente se o manager atualiza-lo
                // 'selecao.' + thatHttpFileSync.player.id + '.json',  // lista de todos os arquivos de media que deverão está no player
            ];

            // verifica se o arquivo é válido
            if (_.isObject(medias) && 'playlists' in medias && medias.playlists) {
                medias = medias.playlists;
            } else {
                medias = [];
            }

            // download de todos os arquivo medias
            var daysInWeek = 7;
            for(var i = 0; i< daysInWeek; i++){
                essentialFiles.push('media.' + thatHttpFileSync.player.id + '.' + i + '.json');
            };

            // Lista os arquivos em disco na pasta [4YouSee] default_project/json/
            FileSync.listFiles(window.rootDirEntry, window.projectName + '/' + FileSync.PLAYER_DIR_JSON, function(filesInDisk, error){
                // em caso de erros
                if(filesInDisk == undefined || error) {
                    _logError('Error ao listar arquivos do diretório', error);
                    //Faz a união da lista de arquivos recebidas do list com os arquivos essenciais ao player
                    medias = _.union(medias, essentialFiles);
                }else{
                    //Cria uma lista com todos os arquivos essencias ao player que não foram baixados
                    var essentialsNeedDownload = _.difference(essentialFiles, filesInDisk);

                    /**
                     * Faz a união da lista arquivos a serem baixados recebidos de pelo <list.id-player.json>
                     * com a lista de arquivos essenciais ao player que ainda não foram baixados.
                     */
                    medias = _.union(medias, essentialsNeedDownload);
                }

                // callback
                callback(medias);
            });
        }
    }

    /**
     * Método para processar todos os arquivos de media.<idPlayer>.json
     *
     * @method processFilesMedia
     * @param  {JSON} listData
     * @param  {function} finishCallback
     * @return {void}
     */
    HttpFileSync.prototype.processFilesMedia = function(listData, finishCallback)
    {

        var fileMediaUpdate = false;

        // verifica a lista de arquivos
        async.each(listData, function _eachFilesMedia (file, next) {

            // só arquivo medias
            if(!file.match('media\.[0-9]+\.[0-6]\.json')) {
                // callback
                return next();
            }

            // download do arquivo media
            FileSyncMedia.readFile(file, function _doneReadFile () {
                // flag
                fileMediaUpdate = true;

                // callback
                return next();
            });
        },
        function _doneEachFilesMedia (error) {
            // callback
            finishCallback(null, listData);

            // reseta o ponteiro da playlist
            if (fileMediaUpdate && !thatHttpFileSync.aux) {
                // reseta a playlist
                thatHttpFileSync.appWindow.playlistManager.resetPlaylistCurrent();
                // flag
                fileMediaUpdate = true;
            }
        });
    }

    /**
     * Método para processar os arquivos de media e carrossel
     *
     * @method processFileCarrossel
     * @param  {JSON} listData
     * @param  {function} finishCallback
     * @return {void}
     */
    HttpFileSync.prototype.processFileCarrossel = function(listData, finishCallback)
    {
        var typeList = true;
        var fileCarrosselUpdate = false;
        var that = this;
        var carrosselAllow = [];

        if (thatHttpFileSync.appWindow !== undefined) {
            carrosselAllow = thatHttpFileSync.appWindow.playlistManager.getCarrocelPlayList();
            carrosselAllow = _.map(carrosselAllow, function _eachCarrossel(item) {
                // nome do arquivo para download
                return FileSyncCarrossel.getNameFile(item.arquivo);
            });
        }

        // verifica a lista de arquivos
        async.each(listData, function _eachFileCarrossel (file, next) {
            typeList = true;

                // gamb, devido o manager enviar categorias que não são utilizadas pelo player
            if (carrosselAllow.indexOf(file) == -1 || !file.match('categoria\.[0-9]+\.json')) {
                // só arquivo medias ou categoria
                if(!file.match('media\.[0-9]+\.[0-6]\.json')) {
                    // callback
                    return next();
                }

                // caso não seja carrossel
                typeList = false;
            }


            // download do arquivo media
            FileSyncCarrossel.readFile(file, thatHttpFileSync.player, typeList, function _doneReadFile () {
                // floag
                fileCarrosselUpdate = true;

                // callback
                return next();
            });
        },
        function _doneEachFilesCarrossel (error) {

            // callback
            finishCallback(null, listData);

            // reseta o ponteiro da playlist
            if (fileCarrosselUpdate && !thatHttpFileSync.aux) {
                // reseta a playlist
                if ('player' in thatHttpFileSync.appWindow) {
                    thatHttpFileSync.appWindow.player.resetPlayList();
                }
            }
        });
    }

    /**
     * Método para realizar o download dos templates e fontes dos layout
     *
     * @method processFileTemplate
     * @param  {Array} listData
     * @param  {Function} finishCallback
     * @return {void}
     */
    HttpFileSync.prototype.processFileTemplate = function(listData, finishCallback)
    {
        var typeList = true;
        var trocaLayout = false;
        var that = this;

        // verifica a lista de arquivos
        async.eachSeries(listData, function _eachFileTemplate (file, next) {
            typeList = true;

            // só arquivo medias ou categoria
            if(_.isNull(file.match('media\.[0-9]+\.[0-6]\.json')) && _.isNull(file.match('layout\.[0-9]+\.json'))) {
                // callback
                return next();
            }

            // caso seja arquivo media ele deve procurar o layout a ser aplicado
            if(!_.isNull(file.match('media\.[0-9]+\.[0-6]\.json'))) {
                typeList = false;
            }

            // download do arquivo media
            FileSyncLayoutTemplate.readFile(file, function _doneReadFile (listFile) {

                if (listFile) {
                    // troca o layout
                    trocaLayout = true;

                    // caso seja arquivo media
                    if (!typeList) {
                        return FileSyncLayoutTemplate.processMedia(listFile, next);
                    }

                    // processa layout
                    return FileSyncLayoutTemplate.processLayout(listFile, next);
                }

                // callback
                return next();
            });
        },
        function _doneEachFileTemplate (error) {

            // callback
            finishCallback(null, listData);

            // exclue todos os arquivos de templates desnecessários
            EventLayout.trigger('verifyFilesLayoutForDelete');

            // carrega layout, o layout padrão será carregado
            if (trocaLayout && erroFileNewsLayout && !thatHttpFileSync.aux) {
                EventLayout.trigger('loadLayout', [thatHttpFileSync.appWindow, true, undefined, true]);
            }

            // flag. Está no escopo global
            erroFileNewsLayout = true;

        });
    }

    /**
     * Método para processar os arquivos de notícia da playlist principal
     *
     * @method processFileNewsPlayList
     * @param  {Object} listData
     * @param  {Function} finishCallback
     * @return {void}
     */
    HttpFileSync.prototype.processFileNewsPlayList = function(listData, finishCallback)
    {
        var flagExec = false;

        // verifica a lista de arquivos
        async.each(listData, function _eachFileNewsPlayList (file, next) {
            // só arquivo medias ou categoria
            if(!file.match('rss\.player\.[0-9]+\.json')) {
                // callback
                return next();
            }

            // download do arquivo media
            FileSyncRss.readFile(file, function _doneReadFile () {

                // flag
                flagExec = true;

                // callback
                return next();
            });
        },
        function _doneEachFileNewsPlayList (error) {

            // callback
            finishCallback(null, listData);

            if (flagExec && !thatHttpFileSync.aux) {
                // evento para excluir os arquivos de notícia desnecessários
                EventDownload.trigger('verifyFilesNewsForDelete')
            }
        });
    }

    /**
     * Método para processar os arquivos de notícia dos templates do layout
     *
     * @method processFileNewsTemplate
     * @param  {Object} listData
     * @param  {Function} finishCallback
     * @return {void}
     */
    HttpFileSync.prototype.processFileNewsTemplate = function(listData, finishCallback)
    {

        // semaforo
        var fileProcess = false;

        // verifica a lista de arquivos
        async.eachSeries(listData, function _eachFilesNewsTemplate (file, next) {
            // só arquivos de notícias
            if(!file.match('rss\.[0-9]+\.xml')) {
                // callback
                return next();
            }

            // download do arquivo
            FileSyncRssNews.readFile(file, function _doneReadFile (errorRead) {

                fileProcess = true;

                // callback
                return next();
            });
        },
        function _doneEachFilesNewsTemplate (error) {

            // callback
            finishCallback(null, listData);

            // caso algum arquivo tenha sido processado
            if (fileProcess && !thatHttpFileSync.aux) {
                // evento para atualizar fonte de notícias do layout
                EventLayout.trigger('updatePlayListNewsLayout');
            }
        });
    }

    /**
     * Método para verificar quais arquivos de conteúdo não são mais necessários para o player
     *
     * @method processFileSelecao
     * @param  {Object} listData
     * @param  {Function} finishCallback
     * @return {void}
     */
    HttpFileSync.prototype.processFileSelecao = function(listData, finishCallback)
    {
        var proccessFile = false;

        // verifica a lista de arquivos
        async.each(listData, function _eachFilesSelecao (file, next) {
            // só arquivos de notícias
            if(!file.match('selecao\.[0-9]+\.json')) {
                // callback
                return next();
            }

            proccessFile = true;

            // callback
            return next();
        },
        function _doneEachFilesSelecao (error) {
            // callback
            finishCallback(null, listData);

            // caso seja necessário processar os arquivos selecao
            if (proccessFile) {
                // evento
                EventDownload.trigger('finishDownloadFilesSelecao');
            }

        });
    }



    HttpFileSync.prototype.setProjectFile = function(metadata, finishCallback)
    {
        var isDir = metadata.mimeType == DRIVE_DIR;
        this.setProjectTreeLeaf(metadata.title, this.projectName, isDir);
        if (isDir) {
            this.mapSubdirTree(metadata, this.projectName + "/" + metadata.title, finishCallback);
        } else {
            this.setFileToSync(metadata, this.projectName);
            finishCallback();
        }
    };

    /**
     * Download de arquivos JSON e gravalo em disco
     *
     * @method downloadFileJson
     * @param  {string} pathMedia   Endereço do arquivo para dowload
     * @param  {string} nameMedia   Nome do arquivo
     * @param  {function} endCallback
     * @return {void}
     */
    HttpFileSync.prototype.downloadFileJson = function (pathMedia, dirPathFile, endCallback)
    {
        var that = this;

        // caminho para download
        var nameMedia = pathMedia.split("/").pop();

        // caso seja informado o caminho para o arquivo
        var dirPathFile = (dirPathFile? dirPathFile : that.projectName + '/' + FileSync.PLAYER_DIR_JSON);

        // download do arquivo de media
        $.ajax({
            url: pathMedia + '?_=' + (new Date().getTime()),
            dataType: "json",
            cache: false

        })
        .done(function _successDownloadFile(mediaData, status, event) {
            // logs
            // console.info("TAG_MAYCON downloadFileJson successDownloadFile 1");

            // parse do arquivo
            FileSync.JSONParse(mediaData, function (mediaData) {
                // logs
                // console.info("TAG_MAYCON downloadFileJson successDownloadFile JSONParse 2: mediaData", mediaData);

                // em caso de erros
                if (!mediaData) {
                    // callback
                    return endCallback(false, event);
                }

                // grava conteúdo do arquivo
                thatHttpFileSync.writeContentFile(dirPathFile, nameMedia, JSON.stringify(mediaData), 'text/plain', function _doneWriteFile(content) {
                    // info
                    // console.info("downloadFileJson successDownloadFile writeContentFile 3");

                    // callback
                    endCallback(content, event);
                });
            });
        })
        .fail(function _failDownloadFile(event) {
            // console.log('TAG_MAYCON failDownloadFile');
            // console.info("TAG_MAYCON downloadFileJson failDownloadFile");

            // callback
            endCallback(false, event);
        });
    }

    /**
     * Método para gravar o conteúdo em um arquivo
     *
     * @method writeContentFile
     * @param  {string} dirPathFile Caminho do arquivo
     * @param  {string} nameMedia   Nome do arquivo
     * @param  {string} mediaData   JSON a ser persisto
     * @param  {Function} endCallback
     * @return {void}
     */
    HttpFileSync.prototype.writeContentFile = function (dirPathFile, nameMedia, mediaData, mimeType, endCallback)
    {

        // caso o mimeType não seja informado utiliza como padrão
        mimeType = (mimeType? mimeType : 'text/plain');

        // grava os medias jsons em disco
        thatHttpFileSync.writeDownloadedFile(rootDirEntry, dirPathFile, nameMedia, mediaData, mimeType, function _doneWriteDownloadedFile(status) {

            // logs
            // console.info("HttpFileSync writeContentFile writeDownloadedFile: status", status);

            // em caso de erro
            if (!status) {
                return endCallback(null);
            }

            // callback
            endCallback(mediaData);
        });
    }

    HttpFileSync.prototype.saveUnzipedFiles = function(rootDirEntry, path, filename, endCallback)
    {
        var pathFilesTo = path.split(".")[0];
        var nameFolder = pathFilesTo.split("/").pop();

        pathFilesTo = pathFilesTo.replace(nameFolder, '');

        // _log(
        //     {
        //         'Class': 'HttpFileSync',
        //         'Method': 'saveUnzipedFiles',
        //         'from:path': path,
        //         'to:pathFilesTo': pathFilesTo
        //     }
        // );

        // descompacta o conteúdo
        player4uc.unzip.start({
            from: path,
            to: pathFilesTo,
            folder: nameFolder
        }, function(error, data){
            // callback
            endCallback(data, error);

            // _log('HttpFileSync.saveUnzipedFiles: error, data', error, data);
        });

    };

    HttpFileSync.prototype.unzipFile = function(rootDirEntry, pathunPack, zipurl, endCallback)
    {
        var that = this;
        var uri = URI(zipurl);
        var nameFile = uri.filename();

        // unzip
        this.saveUnzipedFiles(rootDirEntry, pathunPack, nameFile, function __doneUnzipFile(data, error) {

            // em caso de erro
            if (error) {
                return endCallback(false);
            }

            // callback
            endCallback(true);
        });
    };

    /**
     * Método para escrever um arquivo no disto
     * @param  {Object} rootDirEntry Absoleto
     * @param  {string} path         Caminho do arquivo
     * @param  {string} fileName     Nome do arquivo
     * @param  {string} fileContent  Conteúdo do arquivo
     * @param  {string} contentType  Absoleto
     * @param  {function} endCallback  Callback
     * @return {void}
     */
    HttpFileSync.prototype.writeDownloadedFile = function(rootDirEntry, path, fileName, fileContent, contentType, endCallback)
    {

        var pathFile = path + '/' + fileName;

        // tive problemas com o callback
        // que executava 2 vezes
        // endCallback
        if (_.isUndefined(endCallback)) {
            endCallback = function () {};
        }

        player4uc.filesystem.writeFile(pathFile, fileContent, function __doneWriteDownloadedFile_WriteFile(error, data) {
            // callback
            endCallback(data, error);
        });
    };



    // declarando a função
    window.HttpFileSync = HttpFileSync;
})();
