/*
TODO:

FileSync:
    Sincroniza os arquivos locais com os arquivos do GoogleDrive - OK
    - Precisa receber o nome do projeto. - OK
    - Verifica se o projeto existe no Google Drive e se não existir dispara erro - OK
    - Se existir o projeto, busca os arquivos da pasta json e da pasta videos (futuramente outras) - OK
    - Enquanto percorre os arquivos, monta na memória a estrutura atual de arquivos no Drive - OK
    - Para cada pasta encontrada, percorre todos os arquivos e coloca para baixar para a pasta local - OK
Falta
    - Antes de baixar, verifica a última atualização do arquivo e só baixa se estiver desatualizado
    - Após preparar a estrutura de arquivos do Drive percorre os arquivos locais
    - Monta a estrutura atual dos arquivos locais
    - Verifica se há arquivos locais que não são necessários e apaga
*/


if (typeof FileSync !== "undefined") {
    _logError(FileSync);
    throw new Error('FileSync API already defined.');
}

(function() {

    var DRIVE_FILE = "application/vnd.google-apps.file";
    var DRIVE_DIR = "application/vnd.google-apps.folder";
    var PROJECT_PREFIX = '4YouSee_';

    function fileErrorHandler(e, context) {
        _logError('Error: ', e.name, e.message);
        _logError('Context: ', context);
        // throw new Error(e.message);
    }

    /**
     * ------------------------------------------------------------------
     * class to handle the syncronization of local files with remote files
     * @class FileSync
     */
    function FileSync(projectName, player, athenticate) {

        this.filesToSync = []; // {path: 'string', driveObj: Object}
        this.projectName = projectName;
        this.player = player;

        this.projectTree = {
            dir: projectName,
            files: []
        };

        this.localProjectTree = {
            dir: projectName,
            files: []
        };

        if (athenticate && typeof gapi !== "undefined") {
            // authenticate if not authenticated yet
            gapi.auth.authorize({immediate: false}, function(token) {
                if (!token) {
                // TODO: better error handling
                    _logError(err);
                    return;
                }
                // console.log('authenticado', token.access_token);
            });
        }

        // this.getLocalProjectFileTree();
    }

    /**
     * Método para seta o player
     *
     * @method setPlayer
     * @param {Object} player
     */
    FileSync.setPlayer = function __doneSetPlayer(player) {
        this.player = player;
    }


    /**
     * Inserts a file to filesToSync queue
     *
     * @method setFileToSync
     * @param GoogleDriveFileObject file
     */
    FileSync.prototype.setFileToSync = function(file, path) {
        // console.log('setFileToSync', file, path);

        if (!this.fileAlreadySyncing(file)) {
            this.filesToSync.push({file: file, path: path});
        }
    };

    FileSync.prototype.fileAlreadySyncing = function(file) {
        // console.log('fileAlreadySyncing', file);

        for (var i = 0; i < this.filesToSync.length; i++) {
            if (file.id == this.filesToSync[i].id) {
                return true;
            }
        }
        return false;
    };

    /**
     * Method that sets the tree leaf of the projectTree variable
     *
     * @method setProjectTreeLeaf
     * @param String fileName the dir or file name
     * @param String path The path where to put the file/dir
     * @param Boolean isDir indicates wether the fileName is a dir or not
     */
    FileSync.prototype.setProjectTreeLeaf = function(fileName, path, isDir) {
        // console.log('setProjectTreeLeaf', fileName, path, isDir);

        if (typeof path === 'undefined' || !path)
            path = this.projectName;

        if (typeof isDir === 'undefined')
            isDir = false;

        var dir = this.getProjectTreeDir(path);
        if (isDir) {
            dir.files.push({dir: fileName, files: []});
        } else {
            dir.files.push(fileName);
        }
    };

    /**
     * Returns the dir object reference from projectTree
     */
    FileSync.prototype.getProjectTreeDir = function(path, root) {
        // console.log('getProjectTreeDir', path, root);

        var that = this;

        if (typeof root == "undefined" || !root)
            root = this.projectTree;

        var pathparts = path.split("/");

        // if root of path is the root
        if (root.dir === pathparts[0]) {
            // if path isn't just the root, searchs inside it for the next subitens
            if (pathparts.length > 1) {
                return this.getProjectTreeDir(pathparts.splice(1).join("/"), root);
            } else {
                return root;
            }
        }

        // searches the path inside the "root" tree
        for (var i = 0; i < root.files.length; i++) {
            if (typeof root.files[i] !== "string") {
                if (root.files[i].dir === pathparts[0]) {
                    // if path isn't just the root, searchs inside it for the next subitens
                    if (pathparts.length > 1) {
                        return this.getProjectTreeDir(pathparts.splice(1).join("/"), root.files[i]);
                    } else {
                        return root.files[i];
                    }
                }
            }
        }
    };

    FileSync.prototype.getLocalProjectFileTree = function() {
        // TODO: needs to map the local project file tree for, in future, remove unecessa ry files
    };

    /**
     * Reads a file on disc
     *
     * @method readFile
     * @param String Path
     * @param Function finishCallback With two params: fileContent, errors
     */
    FileSync.readFile = function(folderAbsoleto, path, finishCallback) {
        FileSync.getFile(folderAbsoleto, path, finishCallback);
    };

    /**
     * Reads a file on disc like a text File
     *
     * @method readAsText
     * @param Object fileEntry
     * @param Function finishCallback With two params: fileContent, errors
     */
    FileSync.readAsText = function (fileEntry, finishCallback) {
        fileEntry.file(function(file) {
            var reader = new FileReader();
            reader.onerror = function(e){
                finishCallback(null, e);
            };
            reader.onload = function(e) {
                finishCallback(e.target.result);
            };
            reader.readAsText(file);
        });
    }

    /**
     * Reads a JSON file and passes the JSON object to a callback
     *
     * @method readJSONFile
     * @param String Path
     * @param Function finishCallback With two params: fileContent, errors
     */
    FileSync.readJSONFile = function(folderAbsoleto, path, finishCallback) {
        // console.log("============= Called: readJSONFile", path, finishCallback);
        FileSync.readFile(folderAbsoleto, path, function(fileContent, error) {
            // tratar caso aconteça algum erro
            if (!error) {
                // caso venha string
                FileSync.JSONParse(fileContent, finishCallback);

                // log
                // console.info('readJSONFile');
            } else {
                // callback
                finishCallback(fileContent, error);

                // log
                _logError('readJSONFile ERROR', path);
            }

        });
    };

    /**
     * Método para realizar o parse do xml para josn
     *
     * @method readXmlFile
     * @param  {string} path           Caminho do arquivo
     * @param  {function} finishCallback
     * @return {void}
     */
    FileSync.readXmlFile = function (folderAbsoleto, path, finishCallback) {
        // console.log("============= Called: readJSONFile", path, finishCallback);
        FileSync.readFile(folderAbsoleto, path, function(fileContent, error) {

            var fileContentJson;

            // em caso de erros
            if (!error) {
                try {
                    // caso o conteudo seja enviado
                    if(typeof fileContent == 'string') {
                        // Create x2js instance with default config
                        var x2js = new X2JS();
                        // realizar o parse para xml
                        fileContentJson = x2js.xml_str2json(fileContent);
                    }
                } catch (exception) {
                    console.error(exception);
                    error = exception.message;
                }
            }

            // callback
            finishCallback(fileContentJson, error);
        });
    }

    /**
     * Get a FileEntry object for the file informed
     *
     * @method getFile
     * @param String Path
     * @param Function finishCallback With one params: FileEntry fileentry
     */
    FileSync.getFile = function(folderAbsoleto, path, finishCallback) {

        player4uc.filesystem.readFile(path, function (error, data) {
            // em caso de erros
            if (error) {
                _logError(error, path);
            }

            return finishCallback(data, error);
        });
    };

    /**
     * Método para verificar se o arquivo existe
     *
     * @method exists
     * @param  {string} path
     * @param  {Function} finishCallback
     */
    FileSync.exists = function(path, finishCallback) {

        player4uc.filesystem.exists(path, function (objectError) {
            // em caso de erros
            if (!objectError) {
                // _logError('File not exists', path);
                return finishCallback(false, null);
            }

            finishCallback(true, null);
        });
    };


    /**
     * List archives entry of directory
     *
     * @method listFilesEntries
     * @param DirectoryEntry rootDirEntry
     * @param String directoryToList
     * @param Function listFilesCallback with one params: Array of FileEntry|DirectoryEntry
     */
    FileSync.listFilesEntries = function(rootDirEntry, directoryToList, listFilesEntriesCallback){
        player4uc.filesystem.ls(directoryToList, function __doneLs(error, list) {
            listFilesEntriesCallback(list, error);
        });
    };

    FileSync.listFiles = function(rootDirEntry, directoryToList, listFilesCallback){
        FileSync.listFilesEntries(rootDirEntry, directoryToList, function(entries, error){
            if(entries === null || error)
                listFilesCallback(null, error);
            else{
                var files = [];
                var i;
                for (i in entries) {
                    if(entries[i].isDirectory)
                        files[files.length] = entries[i].name + '.zip';
                    else
                        files[files.length] = entries[i].name;
                };

                // _log({
                //     'FileSync.listFiles: path': directoryToList,
                //     'FileSync.listFiles: error': error,
                //     'FileSync.listFiles: files': files
                // });

                listFilesCallback(files);
            }
        });
    };

    FileSync.deleteFiles = function(filesToDelete){
        var that = this;

        this.deleteFile(filesToDelete);
    }

    /**
     * Método para excluir somente 1 arquivo
     *
     * @method deleteFile
     * @param  {FileEntity} file Entidade que representa o arquivo
     * @return {void}
     */
    FileSync.deleteFile = function(file){

        if (!_.isObject(file)) {
            return false;
        }

        // caso se ja uma pasta
        if(file.isDirectory){
            return player4uc.filesystem.rmdir(file.name, {"recursive": true}, function __doneDeleteFileRmDir(){
                _log('Arquivo deletado com sucesso ', file.name + '.zip');
            });
        }

        // exclue o arquivo
        player4uc.filesystem.rmdir(file.name, {"recursive": true}, function _doneRemoveFileRm(){
            _log('Arquivo deletado com sucesso ', file.name);
        });
    }

    /**
     * Método para realizar o parse de JSON
     *
     * @method JSONParse
     * @param {string}   string
     * @param {Function} callback(Object|null, error)
     */
    FileSync.JSONParse = function (string, callback) {
        var exceptionError = null;

        try {
            if(typeof string === 'string') {
                string = JSON.parse(string);
            }
        } catch (errorEx) {
            exceptionError = errorEx;
            string = null;

            // log
            _logError('JSONParse ERROR');
        }

        return callback(string, exceptionError);
    }

    /**
     * Método para listar todos os arquivos
     *
     * @method listAllFiles
     * @param  {array}   listFolders Array com o caminho para as pastas
     * @param  {Function} callback
     * @return {void}
     */
    FileSync.listAllFiles = function (listFolders, callback) {
        var i,
            listFiles = [];

        async.each(listFolders, function _eachListFolders (folder, eachNext) {
            FileSync.listFilesEntries(window.rootDirEntry, folder, function(filesEntity, error){
                // add files
                listFiles.push(filesEntity);

                // next folder
                eachNext();
            });
        }, function _doneEachListFolder() {
            // _log({
            //     'FileSync.listAllFiles: error': error,
            //     'FileSync.listAllFiles: listFiles': listFiles
            // });

            // callback
            callback(listFiles);
        });
    }

    FileSync.PROJECT_PREFIX = PROJECT_PREFIX;
    FileSync.PLAYER_DIR_JSON = 'json';
    FileSync.PLAYER_DIR_MEDIA = 'media';
    FileSync.PLAYER_DIR_AUDIO = 'mp3';
    FileSync.PLAYER_DIR_RSS_TEMPLATE = 'template';
    FileSync.PLAYER_DIR_RSS_IMG = 'imgnew';
    FileSync.PLAYER_DIR_XML = 'xml';
    FileSync.PLAYER_DIR_LAYOUT = 'layout';
    FileSync.PLAYER_DIR_LAYOUT_TEMPLATE = 'layout/template';
    FileSync.PLAYER_DIR_LAYOUT_RSS_IMG = 'layout/imgnew';
    FileSync.PLAYER_DIR_LAYOUT_RSS_XML = 'layout/xml';

    FileSync.FILE_TYPE_AUDIO = 'audio';
    FileSync.FILE_TYPE_MEDIA = 'media';
    FileSync.FILE_TYPE_RSS_IMG = 'img';
    FileSync.FILE_TYPE_RSS_TEMPLATE = 'template';
    FileSync.FILE_TYPE_XML = 'xml';
    FileSync.FILE_TYPE_LAYOUT_TEMPLATE = 'layout';
    FileSync.FILE_TYPE_LAYOUT_RSS_IMG = 'layout_rss_img';

    FileSync.TYPE_CARROSSEL = 'car';
    FileSync.TYPE_MOVIE = 'i';
    FileSync.TYPE_LAYOUT = 'l';
    FileSync.TYPE_NEWS = 'n';

    window.FileSync = FileSync;
})();
