// scripts/FileSync/FileSyncCarrossel.js
(function (window) {
    'use strict';

    /**
     * Classe para gerenciar a leitura e download dos carrosseis
     *
     * @class FileSyncCarrossel
     * @type {Function}
     */
    var FileSyncCarrossel = FileSyncInterface(window).clone();

    /**
     * Nome do arquivo de categoria
     *
     * @property {string} nameFile
     * @type {String}
     */
    FileSyncCarrossel.nameFile = 'categoria.<idplayer>.json';

    /**
     * Lista de carrosséis
     * @type {Array}
     */
    FileSyncCarrossel.prototype.listCarrosseis = [];

    /**
     * Método para adicionar itens do arquivo média para que o carrossel possa ser validado
     * @param {Object} item
     */
    FileSyncCarrossel.prototype.addMediaListCarrossel = function __doneAddMediaListCarrossel(item) {
        var validItem = _.findWhere(this.listCarrosseis, {"arquivo": item.arquivo});

        // caso não exista ele será adicionado
        if (!validItem) {
            this.listCarrosseis.push(item);
        }
    };

    /**
     * Método para ler o arquivo media
     *
     * @method readFile
     * @param  {Object} file
     * @param  {Object} playerObject Objeto do player
     * @param  {boolean} carrossel   Caso seja true o arquivo de media que está rodando é o de carrossel
     * @param  {Function} callbackReadFile
     * @return {void}
     */
    FileSyncCarrossel.prototype.readFile = function (file, playerObject, carrossel, callbackReadFile)
    {
        // caminho mais o nome do arquivo
        var idCarrossel = file.replace(/(categoria)+\.(\d+)\.(json)+/g, "$2");
        var media = window.projectName + '/json/' + file;
        var that = this;

        // ler o arquivo xml
        FileSync.readJSONFile(window.rootDirEntry, media, function doneReadJSONFile(jsonFile, error) {

            // em caso de erros
            if (_.isNull(jsonFile) || error) {

                // callback
                callbackReadFile();

                // tenta recurar a lista
                return that.recuperar(file, 'Arquivo não encontrado.');
            }

            var itens = jsonFile.itens;

            /**
             * Verifica cada item da lista para saber já foi feito o download
             * @param  {array} item  Lista dos arquivos
             * @return {void}
             */
            _async.each(itens,
                function processItemMedia(item, next) {
                    // verifica se é um objeto válido
                    if(_.isObject(item) && item.tipo === FileSync.TYPE_CARROSSEL && !item.arquivo) {

                        _log('Objeto inválio.', item);
                        return next();
                    }

                    var validItemCarrossel = _.findWhere(that.listCarrosseis, {"arquivo": idCarrossel});
                    if (validItemCarrossel) {
                        var validItem = _.findWhere(validItemCarrossel.playlistSchedule, {"arquivo": item.arquivo});
                        if (validItem) {
                            item.schedule = validItem.schedule;
                            item.startDate = validItem.startDate;
                            item.endDate = validItem.endDate;
                        }
                    }

                    // caso o item esteja com a programação vencida
                    // o download do arquivo não deverá ser feito
                    if (!that.dateValidateDownload(item)) {
                        _log('Conteúdo com a programação vencida.', item.arquivo);
                        return next();
                    }

                    // TESTANDO OS TIPOS
                    switch (item.tipo) {

                        // ARQUIVOS
                        case FileSync.TYPE_MOVIE:
                            // adiciona arquivo para download
                            that.addDownloadFile(item, next);
                            break;

                        // CARROSSEL
                        // TODO: ISSO AQUI NÃO ESTÁ FUNCIONANDO
                        // DEVE VERIFICAR O MOTIVO
                        case FileSync.TYPE_CARROSSEL:
                            var fileCarrossel = that.getNameFile(item.arquivo);

                            // adiciona arquivo de lista para download
                            that.downloadFileMedia(sync.urlDownloadJson + '/' + fileCarrossel, function () {
                                // processar o arquivo
                                that.readFile(fileCarrossel, true, carrossel, next);
                            })
                            break;

                        // caso seja outro formato
                        default:
                            // callback
                            next();
                    }
                },
                function doneProcessItemMedia() {
                    callbackReadFile();
                }
            );
        });
    }

    /**
     * Método para retornar o nome do arquivo
     *
     * @method getNameFile
     * @param  {integer} idPlayer Id do player
     * @return {integer}
     */
    FileSyncCarrossel.prototype.getNameFile = function FileSyncCarrosselGetNameFile (idPlayer) {
        return FileSyncCarrossel.nameFile.replace("<idplayer>", idPlayer);
    }

    /**
     * Método para verificar se o download do arquivo devera ser feito
     *
     * @method dateValidateDownload
     * @param  {object} item
     * @return {boolean}
     */
    FileSyncCarrossel.prototype.dateValidateDownload = function (item) {
        var today = new Date();
        var regex = new RegExp('-', 'g');

        // data iniciar
        var startDate = (!item.startDate) ? undefined : new Date(item.startDate.replace(regex, '/'));

        // data final
        var endDate = undefined;
        if(item.endDate){
            endDate = new Date(item.endDate.replace(regex, '/'));
            endDate.setHours(23);
            endDate.setMinutes(59);
            endDate.setSeconds(59);
        }

        // caso não tenha programação o download deverá ser feito
        if (startDate == undefined && endDate == undefined) {
            return true;
        }

        // caso a data final seja maior que hoje o download deverá ser feito
        if (endDate == undefined || endDate > today) {
            return true;
        }

        return false;
    }

    window.FileSyncCarrossel = new FileSyncCarrossel();
}) (window);
