// scripts/FileSync/FileSyncInterface.js
window.FileSyncInterface = (function (window) {
    'use strict';

    /**
     * Método para realizar download dos arquivos de medias
     *
     * @class FileSyncInterface
     */
    function FileSyncInterface () {

    }

    /**
     * Tipo de extensões permitidas para download
     *
     * @property {array} typesAccepts
     * @type {Array}
     */
    FileSyncInterface.typesAccepts = ['mp3', 'mp4', 'html', 'htm', 'zip', 'jpeg', 'jpg', 'png', 'gif', 'bmp'];

    /**
     * Objeto utilizado para ser repassado ao método de download do arquivo
     *
     * @property {object} ObjectDownloadItem
     * @type {Object}
     */
    FileSyncInterface.ObjectDownloadItem = {
        file: undefined,
        type: undefined,
        url: null,
        notify: false,
        trying: 0
    };

    /**
     * Método para realizar o download dos arquivos de media
     *
     * @method downloadFileMedia
     * @param  {string} file        Caminho do arquivo para ser feito o download
     * @param  {Function} endCallback
     * @return {void}
     */
    FileSyncInterface.prototype.downloadFileMedia = function (file, endCallback) {
        var dirPathFile = window.projectName + '/' + FileSync.PLAYER_DIR_JSON;

        // download do arquivo json
        HttpFileSync.prototype.downloadFileJson(file, dirPathFile, endCallback);
    }

    /**
     * Método para recurar, caso exista algum erro
     *
     * @method recuperar
     * @param {string} file Nome do arquivo a ser processado
     * @param {string} msg Mensagem de erro
     * @return {void}
     */
    FileSyncInterface.prototype.recuperar = function recuperar(file, msg) {
        var media = window.sync.urlDownloadJson + file;
        var that = this;

        // reporta o erro
        _logError(msg);

        // download do arquivo
        this.downloadFileMedia(media, function downloadProcessFile(error, event) {

            // caso seja 404
            if (!error || _.isObject(event) && event.status == 404) {
                _log("Arquivo '" + file + "' não encontrado.");
                return;
            }

            // o processamento fica em paralelo
            that.readFile(file, function doneProcesseFile(error) {
                if (!error) {
                    _log("TAG_MAYCON processFileNewsTemplate: Arquivo " + file + " foi processado com sucesso.");
                }
            });
        });
    }

    /**
     * Método para ler o arquivo media
     *
     * @method readFile
     * @param  {Object} file
     * @return {void}
     */
    FileSyncInterface.prototype.readFile = function (file, callbackReadFile) {

        // caminho mais o nome do arquivo
        var media = window.projectName + '/json/' + file;

        var that = this;

        // ler o arquivo xml
        FileSync.readJSONFile(window.rootDirEntry, media, function doneReadJSONFile(jsonFile, error) {

            // em caso de erros
            if (_.isNull(jsonFile) || error) {

                // callback
                callbackReadFile();

                // tenta recurar a lista
                return that.recuperar(file, 'Arquivo não encontrado.');
            }

            var itens = jsonFile.itens;

            /**
             * Verifica cada item da lista para saber já foi feito o download
             * @param  {array} item  Lista dos arquivos
             * @return {void}
             */
            _async.each(itens,
                function processItemMedia(item, next) {
                    // verifica o tipo de arquivo
                    if((!item.arquivo || !('tipo' in item)) && item.tipo != FileSync.TYPE_NEWS) {

                        _logError('Objeto inválio.', item);
                        return next();
                    }

                    // caso o item esteja com a programação vencida
                    // o download do arquivo não deverá ser feito
                    if (!that.dateValidateDownload(item)) {
                        _log('Conteúdo com a programação vencida.', item.arquivo);
                        return next();
                    }

                    // TESTANDO OS TIPOS
                    switch (item.tipo) {

                        // ARQUIVOS
                        case FileSync.TYPE_MOVIE:
                            // adiciona arquivo para download
                            that.addDownloadFile(item, next);
                            break;

                        // caso não seja nenhum tipo para ser tratado
                        default:
                            // callback
                            next();
                    }
                },
                function doneProcessItemMedia() {
                    callbackReadFile();
                }
            );
        });
    }

    /**
     * Método para adicionar item a fila de download
     *
     * @method addDownloadFile
     * @param {Object} objectFile
     * @param {Function} endCallback
     */
    FileSyncInterface.prototype.addDownloadFile = function (objectFile, endCallback) {
        // pega a extenção do arquivo especificado
        var mediaType = objectFile.arquivo.split('.').pop();

        // verifica o tipo do arquivo
        if (_.indexOf(this.typesAccepts, mediaType)) {

            // adiciona na lista para download
            var downloadItem = _.clone(FileSyncInterface.ObjectDownloadItem);
            downloadItem.file = objectFile.arquivo;
            downloadItem.type = FileSync.FILE_TYPE_MEDIA;

            // adicionando o arquivo para download
            DownloadCenter.addDownload(downloadItem);
        }

        // callback
        return endCallback();
    }

    /**
     * Método para verificar se o download do arquivo devera ser feito
     *
     * @method dateValidateDownload
     * @param  {object} item
     * @return {boolean}
     */
    FileSyncInterface.prototype.dateValidateDownload = function (item) {
        var today = new Date();
        var regex = new RegExp('-', 'g');

        // data iniciar
        var startDate = (!item.startDate) ? undefined : new Date(item.startDate.replace(regex, '/'));

        // data final
        var endDate = undefined;
        if(item.endDate){
            endDate = new Date(item.endDate.replace(regex, '/'));
            endDate.setHours(23);
            endDate.setMinutes(59);
            endDate.setSeconds(59);
        }

        // caso não tenha programação o download deverá ser feito
        if (startDate == undefined && endDate == undefined) {
            return true;
        }

        // caso a data final seja maior que hoje o download deverá ser feito
        if (endDate == undefined || endDate > today) {
            return true;
        }

        return false;
    }

    /**
     * Método para unir todos os arquivos selecao.json
     *
     * @method joinFiles
     * @param  {[type]} listFiles Lista de arquivos a serem testados
     * @return {void}
     */
    FileSyncInterface.prototype.joinFiles = function FileSyncInterfaceJoinFiles (listFiles, filesSearch, finishCallback) {

        // caso não seja um array
        if (!_.isArray(listFiles)) {
            // logs
            // console.log('TAG_MAYCON FileSyncInterface.joinFiles: joinFiles', listFiles);
            return finishCallback([]);
        }

        /**
         * Lista de arquivos
         *
         * @var {array} listAllFilesSelecao
         * @type {Array}
         */
        var listAllFilesSelecao = [];

        /**
         * Lista de itens a ser comparados
         *
         * @var {RegExp} listRegex
         * @type {RegExp}
         */
        var listRegex = new RegExp(filesSearch.join('|'), "ig");

        var that = this;

        // verifica a lista de arquivos
        async.each(listFiles, function eachFilesSelecao (file, nextEachListFile) {
            // só arquivos de notícias
            // AQUI DEVE VIR UMA COMPARAÇÃO DE ARRAY UTILIZANDO EXPRESSÃO REGULAR
            if(!file.match(listRegex)) {
                // nextEachListFile
                return nextEachListFile();
            }

            // Lê o arquivo
            that.readFile(file, function doneReadFile (contentFile) {
                // caso seja array
                if (_.isArray(contentFile)) {
                    // verifica itens
                    return async.each(contentFile, function doneEach(item, next) {
                        // adiciona
                        listAllFilesSelecao.push(item);

                        // callback
                        next();
                    }, function doneEachCotentFile() {
                        // nextEachListFile
                        nextEachListFile();
                    });
                }

                // caso seja Objeto
                if (_.isObject(contentFile)) {
                    // adiciona
                    listAllFilesSelecao.push(contentFile);
                }


                return nextEachListFile();
            });
        },
        function doneEachJoinFiles (error) {

            // callback
            finishCallback(listAllFilesSelecao);

            // logs
            // console.info('Finish joinFiles');
        });
    }

    // set a classe
    return FileSyncInterface;
});
