// scripts/FileSync/FileSyncLayoutTemplate.js
(function (window) {
    'use strict';

    /**
     * Classe para gerenciar o processamento dos arquivos de layout
     *
     * @class FileSyncLayoutTemplate
     * @type {Function}
     */
    var FileSyncLayoutTemplate = FileSyncInterface(window).clone();

    /**
     * Tipo de extensões permitidas para download
     *
     * @property {array} typesAccepts
     * @type {Array}
     */
    FileSyncLayoutTemplate.typesAccepts = ['zip'];

    /**
     * Quando não se deve realizar o download da imagem
     *
     * @property {string} NOT_FOUND_IMAGE
     * @type {String}
     */
    FileSyncLayoutTemplate.NOT_FOUND_IMAGE = '0.png';

    /**
     * Variável com o caminho dos arquivos a serem lidos
     *
     * @property {string} filePath
     * @type {String}
     */
    FileSyncLayoutTemplate.filePath = '';

    /**
     * Variável com nome do arquivo de layout
     *
     * @property {string} fileName
     * @type {String}
     */
    FileSyncLayoutTemplate.fileName = 'layout.<IDPLAYER>.json'

    /**
     * Método para realizar o download dos arquivos de notícia
     *
     * @method downloadFileMedia
     * @param  {string} file        Caminho do arquivo para ser feito o download
     * @param  {Function} endCallback
     * @return {void}
     */
    FileSyncLayoutTemplate.prototype.downloadFileMedia = function (file, endCallback) {
        var dirPathFile = window.projectName + '/' + FileSync.PLAYER_DIR_JSON;

        // download do arquivo json
        window.sync.downloadFileJson(file, dirPathFile, function _doneDownloadFileJsonTemplate(json) {
            // callback
            endCallback(json);
        });
    }

    /**
     * Método para adicionar item a fila de download
     *
     * @method addDownloadFile
     * @param {Object} objectFile
     * @param {Function} endCallback
     */
    FileSyncLayoutTemplate.prototype.addDownloadFile = function (objectFile, endCallback) {

        var that = this;

        // _log("FileSyncLayoutTemplate.addDownload: objectFile.area", objectFile.area);

        async.each(objectFile.area, function _downloadFilesLayout (item, next) {

            //////////////////////////
            // download do template //
            //////////////////////////
            if ('template' in item) {
                var downloadItem = _.clone(FileSyncLayoutTemplate.ObjectDownloadItem);
                downloadItem.file = item.template;
                downloadItem.type = FileSync.FILE_TYPE_LAYOUT_TEMPLATE;

                // _log("FileSyncLayoutTemplate.addDownload template: ", downloadItem);

                // adicionando o arquivo para download
                DownloadCenter.addDownload(downloadItem);
            }

            /////////////////////////
            // download da notícia //
            /////////////////////////
            if ('fonte' in item) {
                downloadItem = _.clone(FileSyncLayoutTemplate.ObjectDownloadItem);
                downloadItem.file = "rss." + item.fonte + ".xml";
                downloadItem.type = FileSync.FILE_TYPE_XML;

                // console.log("FileSyncLayoutTemplate.addDownload fonte: ", downloadItem);

                // adicionando o arquivo para download
                DownloadCenter.addDownload(downloadItem);

                // processa o arquivo para encontrar as imagens
                var obj = {image: [], fonte: [downloadItem.file]}
                that.processFileXml(obj, function __doneProcessFileXml(listImage) {
                    async.each(
                        listImage.image,
                        function __eachListImage(image, next) {
                            downloadItem = _.clone(FileSyncLayoutTemplate.ObjectDownloadItem);
                            downloadItem.file = image;
                            downloadItem.type = FileSync.FILE_TYPE_LAYOUT_RSS_IMG;

                            // adicionando o arquivo para download
                            DownloadCenter.addDownload(downloadItem);
                        }
                    );
                });
            }

            // callback each
            return next();
        }, function _doneDownloadFilesLayout (error) {
            // callback
            endCallback();
        });
    }

    /**
     * Método para ler os arquivos
     *
     * @method readFile
     * @param  {Object} file
     * @param  {Boolean} typeFile   Caso seja true o tipo de arquivo será do tipo layout, caso false será um media
     * @param  {Function} callbackReadFile
     * @return {void}
     */
    FileSyncLayoutTemplate.prototype.readFile = function (file, callbackReadFile) {
        var that = this;

        // caminho mais o nome do arquivo
        file = window.projectName + '/' + FileSync.PLAYER_DIR_JSON + '/' + file;

        // console.log('FileSyncLayoutTemplate.readFile: file', file);

        // ler o arquivo
        FileSync.readJSONFile(window.rootDirEntry, file, function _doneReadJSONFile(jsonFile, error) {

            // em caso de erros
            if (error) {
                jsonFile = false;
            }

            // callback
            return callbackReadFile(jsonFile);
        });
    }

    /**
     * Método para realizar o download do layout padrão
     *
     * @method processLayout
     * @param  {object}   jsonFile
     * @param  {Function} callback
     * @return {void}
     */
    FileSyncLayoutTemplate.prototype.processLayout = function FileSyncLayoutTemplateProcessLayout (jsonFile, callback) {

        if (!_.isObject(jsonFile) || !('layouts' in jsonFile)) {
            return callback(false);
        }

        // pega o id do layout padrão e baixar a fonte
        var idLayoutDefault = jsonFile.layouts.default;
        var list = jsonFile.layouts.layout;
        var i;
        var that = this;

        // download do conteúdo
        async.each(list, function _eachList (item, next) {

            // download
            that.addDownloadFile(item, next);
        }, function _doneEachList() {
            // callback
            callback();
        });
    }

    /**
     * Método para realizar o download de todos os layout da plalist principal
     *
     * @method processMedia
     * @param  {Object}   itens
     * @param  {Function} callback
     * @return {void}
     */
    FileSyncLayoutTemplate.prototype.processMedia = function FileSyncLayoutTemplateProcessMedia (itens, callback) {

        var that = this;

        // caso esteja vazio
        if ('itens' in itens && !itens.itens.length) {
            // logs
            _logError('Arquivo media está vazio.', itens);

            return callback();
        }

        /**
         * Verifica cada item da lista para saber já foi feito o download
         * @param  {array} item  Lista dos arquivos
         * @return {void}
         */
        _async.each(itens.itens,
            function processItemMedia(item, next) {
                // verifica o tipo de arquivo
                if(_.isObject(item) && item.tipo === FileSync.TYPE_LAYOUT && !item.arquivo) {

                    _logError('Objeto inválio.', item);
                    return next();
                }

                // caso o item esteja com a programação vencida
                // o download do arquivo não deverá ser feito
                if (!that.dateValidateDownload(item)) {
                    _log('Conteúdo com a programação vencida.', item.arquivo);
                    return next();
                }

                // TESTANDO OS TIPOS
                switch (item.tipo) {

                    // ARQUIVOS
                    case FileSync.TYPE_LAYOUT:
                        // adiciona arquivo para download
                        // TODO: AQUI FUDEU, PORQUE SÓ TENHO O ID DO LAYOUT
                        that.addDownloadFile(item, next);
                        break;

                    // caso não seja nenhum tipo para ser tratado
                    default:
                        // callback
                        next();
                }
            },
            function doneProcessItemMedia() {
                callback();
            }
        );
    }


    /**
     * Método para verificar todos os arquivos de xml e zip que não mais são necessários e depois excluílos
     *
     * @method verifyFilesLayoutForDelete
     * @return {void}
     */
    FileSyncLayoutTemplate.prototype.verifyFilesLayoutForDelete = function FileSyncLayoutTemplateVerifyFilesLayoutForDelete() {
        // caminho para pasta onde estão os arquivos layout.<idplayer>.json
        var pathFilesLayout = window.projectName + '/' + FileSync.PLAYER_DIR_JSON;
        // this
        var that = this;

        // iniciando o callback Hell
        FileSync.listFiles (window.rootDirEntry, pathFilesLayout, function _doneListFiles(listFiles) {

            var listFilesTemplates = {
                fonte: [],
                template: [],
                image: [],
            };

            // caso não seja um array
            if (!_.isArray(listFiles)) {
                // logs
                // console.log('verifyFilesLayoutForDelete: listFiles', listFiles);
                return false;
            }

            //Uni todos os arquivos layout.<>.json
            that.joinFiles(listFiles, ['layout\.[0-9]+\.json'], function _doneJoinFiles(listObjectFileLayout) {

                // verificar cada area de cada layout objeto layouts
                async.each(listObjectFileLayout, function _doneEachObjects(object, nextFilesLayouts) {
                    var layouts = object.layouts.layout;

                    // verifica todas as áreas
                    async.each(layouts, function _doneEachLayouts(ObjectLayout, nextLayouts) {
                        var AllArea = ObjectLayout.area;

                        // verifica cada área do layout
                        async.each(AllArea, function _doneEachArea(area, nextArea) {
                            // caso seja área principal
                            if (('fonte' in area) || ('template' in area)) {
                                // fonte
                                var nameFonte = 'rss.' + area.fonte + '.xml';
                                if (listFilesTemplates.fonte.indexOf(nameFonte) == -1) {
                                    listFilesTemplates.fonte.push(nameFonte);
                                }

                                // template
                                var nameTemplate = area.template;
                                if (listFilesTemplates.template.indexOf(nameTemplate) == -1) {
                                    listFilesTemplates.template.push(nameTemplate);
                                }
                            }

                            // next area
                            return nextArea();
                        }, function _doneAllArea() {
                            // next layout
                            return nextLayouts();
                        });

                    }, function _nextFileLayout() {
                        // next file layout
                        nextFilesLayouts();
                    });
                }, function _doneEachAllLayouts(errors) {

                    // processa todas as fontes para obter a lista de imagens do layout
                    that.processFileXml(listFilesTemplates, function __done(listFilesTemplatesImage) {
                        // excluindo arquivos fonte
                        that.deleteFiles(listFilesTemplatesImage, function _doneDeleteFile() {
                            _log("Arquivos de template, imagem e xml excluídos com sucesso.");
                        });
                    })


                    // log
                    // console.log('verifyFilesLayoutForDelete: listFilesTemplates', listFilesTemplates);
                });
            });
        });
    }

    /**
     * Método para processar os arquivos de xml
     * @param  {Object} objectList
     * @param  {Function} endCallback
     * @return {void}
     */
    FileSyncLayoutTemplate.prototype.processFileXml = function __doneProcessImage(objectList, endCallback) {
        // ler todos os arquivos de fonte
        var pathXml = window.projectName + '/' + FileSync.PLAYER_DIR_LAYOUT_RSS_XML + '/';

        async.each(
            objectList.fonte,
            function __eachProcessFonteForImage(item, endNext) {
                FileSync.readXmlFile(window.rootDirEntry, pathXml + item, function __doneEachProcessFonteForImage(json, error) {
                    // em caso de erros
                    if (error) {
                        return endNext();
                    }

                    // se existe item
                    if (_.isObject(json) && 'root' in json && 'item' in json.root) {
                        // verifica item por item
                        if (!_.isArray(json.root.item)) {
                            json.root.item = [json.root.item];
                        }

                        async.each(
                            json.root.item,
                            function __eachJsonRoot(item, next) {
                                // lista das imagens
                                if (('_imagem' in item) && (_.indexOf(objectList.image, item._imagem) === -1) && item._imagem != '0.png') {
                                    objectList.image.push(item._imagem);
                                }

                                return next();
                            },
                            function __done() {
                                return endNext();
                            }
                        );
                    } else {
                        return endNext();
                    }

                });
            },
            function __doneEach() {
                endCallback(objectList);
            }
        );
    };

    /**
     * Método para retornar o nom do arquivo de layout
     *
     * @method getNameFile
     * @param  {integer} playerId Id do player
     * @return {string}
     */
    FileSyncLayoutTemplate.prototype.getNameFile = function FileSyncLayoutTemplateGetNameFile (playerId) {
        return FileSyncLayoutTemplate.fileName.replace("<IDPLAYER>", playerId)
    }

    /**
     * Método para excluir os arquivos desnecessários
     *
     * @method deleteFiles
     * @param  {Array} itens              Lista de arquivos que devem ficar no player
     * @param  {Function} callbackDeleteFile
     * @return {void}
     */
    FileSyncLayoutTemplate.prototype.deleteFiles = function (itens, callbackDeleteFile) {

        // caso esteja vazio ou não seja um array
        if (!_.isObject(itens) || _.isEmpty(itens)) {
            return callbackDeleteFile();
        }

        // caminho das pastas
        var pathFilesTemplates = window.projectName + '/' + FileSync.PLAYER_DIR_LAYOUT_TEMPLATE;
        // pasta das fontes
        var pathFilesXml = window.projectName + '/' + FileSync.PLAYER_DIR_LAYOUT_RSS_XML;
        // pasta das imagens
        var pathFilesImage = window.projectName + '/' + FileSync.PLAYER_DIR_LAYOUT_RSS_IMG;

        // lista de arquivos para excluir
        var deleteFiles = {
            template: [],
            xml: [],
            image: [],
        }

        // verifica as pasta e exclui os arquivos
        async.series([
            function verifyFilesTemplateDelete (next) {
                // Lista os arquivos em disco na pasta [4YouSee] default_project/layout/template
                FileSync.listFiles(window.rootDirEntry, pathFilesTemplates, function _doneListFiles(filesInDisk, error) {

                    // caso tenha algum erro
                    if (!error) {
                        // verificar quais arquivos tem a mais na pasta 'template', dos que tem na lista
                        deleteFiles.template = _.filter(filesInDisk, function (item) {
                            return _.indexOf(itens.template, item) == -1;
                        });
                    }

                    // callback
                    return next();
                });
            },
            function verifyFilesImageDelete (next) {
                // Lista os arquivos em disco na pasta [4YouSee] default_project/layout/imgnew
                FileSync.listFiles(window.rootDirEntry, pathFilesImage, function _doneListFiles(filesInDisk, error) {

                    // caso tenha algum erro
                    if (!error) {
                        // verificar quais arquivos tem a mais na pasta 'image', dos que tem na lista
                        deleteFiles.image = _.filter(filesInDisk, function (item) {
                            return _.indexOf(itens.image, item) == -1;
                        });
                    }

                    // callback
                    return next();
                });
            },
            function verifyFilesXmlDelete (next) {
                // Lista os arquivos em disco na pasta [4YouSee] default_project/layout/xml
                FileSync.listFiles(window.rootDirEntry, pathFilesXml, function _doneListFiles(filesInDisk, error) {
                    // caso tenha algum erro
                    if (!error) {
                        // verificar quais arquivos tem a mais na pasta 'xml', dos que tem na lista
                        deleteFiles.xml = _.filter(filesInDisk, function (item) {
                            return _.indexOf(itens.fonte, item) == -1;
                        });
                    }

                    // callback
                    return next();
                });
            }
        ],
        function doneVerifyFilesMediaAndAudioForDelete() {

            // callback
            callbackDeleteFile();

            var i, nameFile;
            // caminho das pastas
            var pathFilesTemplates = window.projectName + '/' + FileSync.PLAYER_DIR_LAYOUT_TEMPLATE + '/';
            // pasta das fontes
            var pathFilesXml = window.projectName + '/' + FileSync.PLAYER_DIR_LAYOUT_RSS_XML + '/';
            // pasta das imagens
            var pathFilesImage = window.projectName + '/' + FileSync.PLAYER_DIR_LAYOUT_RSS_IMG + '/';

            // excluindo os arquivos de template
            FileSync.listFilesEntries(window.rootDirEntry, pathFilesTemplates, function(filesEntity, error){
                for (i in filesEntity) {
                    nameFile = !filesEntity[i].isDirectory? filesEntity[i].name : filesEntity[i].name + '.zip';

                    if (_.indexOf(deleteFiles.template, nameFile) != -1) {
                        filesEntity[i].name = pathFilesTemplates + filesEntity[i].name;
                        FileSync.deleteFile(filesEntity[i]);
                    }
                }
            });

            // excluindo os arquivos de imagens
            FileSync.listFilesEntries(window.rootDirEntry, pathFilesImage, function(filesEntity, error){
                for (i in filesEntity) {
                    if (_.indexOf(deleteFiles.image, filesEntity[i].name) != -1) {
                        filesEntity[i].name = pathFilesImage + filesEntity[i].name;
                        FileSync.deleteFile(filesEntity[i]);
                    }
                }
            });

            // excluindo os arquivos xml
            FileSync.listFilesEntries(window.rootDirEntry, pathFilesXml, function(filesEntity, error){
                for (i in filesEntity) {
                    if (_.indexOf(deleteFiles.xml, filesEntity[i].name) != -1) {
                        filesEntity[i].name = pathFilesXml + filesEntity[i].name;
                        FileSync.deleteFile(filesEntity[i]);
                    }
                }
            });
        }
        );
    };

    window.FileSyncLayoutTemplate = new FileSyncLayoutTemplate();
}) (window);
