// scripts/FileSync/FileSyncRss.js
(function (window) {
    'use strict';

    /**
     * Classe para processar todos os arquivos de rss
     *
     * @class FileSyncRss
     * @type {[type]}
     */
    var FileSyncRss = FileSyncInterface(window).clone();

    /**
     * Tipo de extensões permitidas para download
     *
     * @property {array} typesAccepts
     * @type {Array}
     */
    FileSyncRss.typesAccepts = ['zip'];

    /**
     * Quando não se deve realizar o download da imagem
     * @property {string} NOT_FOUND_IMAGE
     * @type {String}
     */
    FileSyncRss.NOT_FOUND_IMAGE = '0.png';

    /**
     * Variável com o caminho dos arquivos a serem lidos
     * @property {string} filePath
     * @type {String}
    */
    FileSyncRss.filePath = window.projectName + '/json';

    /**
     * Método para adicionar item a fila de download
     *
     * @method addDownloadFile
     * @param {Object} objectFile
     * @param {Function} endCallback
     */
    FileSyncRss.prototype.addDownloadFile = function (objectFile, endCallback) {
        // pega a extenção do arquivo especificado
        var mediaType = objectFile.template.split('.').pop();

        // verifica o tipo do arquivo
        if (_.indexOf(FileSyncRss.typesAccepts, mediaType) !== -1) {

            //////////////////////////
            // download do template //
            //////////////////////////
            var downloadItem = _.clone(FileSyncRss.ObjectDownloadItem);
            downloadItem.file = objectFile.template;
            downloadItem.type = FileSync.FILE_TYPE_RSS_TEMPLATE;
            downloadItem.force = true;

            // adicionando o arquivo para download
            DownloadCenter.addDownload(downloadItem);

            ////////////////////////
            // download da imagem //
            ////////////////////////

            //Verique se imagem usada na notíca não é 0.png
            if(objectFile.imagem != FileSyncRss.NOT_FOUND_IMAGE){
                var downloadItem = _.clone(FileSyncRss.ObjectDownloadItem);
                downloadItem.file = objectFile.imagem;
                downloadItem.type = FileSync.FILE_TYPE_RSS_IMG;
                downloadItem.force = true;

                // adicionando o arquivo para download
                DownloadCenter.addDownload(downloadItem);
            }
        }

        // callback
        return endCallback();
    }

    /**
     * Método para ler o arquivo media
     *
     * @method readFile
     * @param  {Object} file
     * @param  {Function} callbackReadFile
     * @return {void}
     */
    FileSyncRss.prototype.readFile = function (file, callbackReadFile) {
        // caminho mais o nome do arquivo
        var media = window.projectName + '/json/' + file;

        var that = this;

        // ler o arquivo xml
        FileSync.readJSONFile(window.rootDirEntry, media, function doneReadJSONFile(jsonFile, error) {

            // em caso de erros
            if (_.isNull(jsonFile) || error) {

                // callback
                callbackReadFile(false);

                // tenta recurar a lista
                return that.recuperar(file, 'Arquivo não encontrado.');
            }

            var itens = jsonFile.itens;

            /**
             * Verifica cada item da lista para realizar o download
             * @param  {array} item  Lista dos arquivos
             * @return {void}
             */
            _async.each(itens,
                function __processItemMedia(item, next) {
                    // verifica o tipo de arquivo
                    if(!('texto' in item) || !('template' in item)) {

                        _logError('Objeto inválio.', item);
                        return next();
                    }

                    // caso o item esteja com a programação vencida
                    // o download do arquivo não deverá ser feito
                    if (!that.dateValidateDownload(item)) {
                        _log('Conteúdo com a programação vencida.', item.arquivo);
                        return next();
                    }

                    // download do conteúdo
                    that.addDownloadFile(item, next);
                },
                function __doneReadJSONFile() {
                    callbackReadFile();
                }
            );
        });
    }

    /**
     * Método para ler todos os arquivos de notícia
     * TODO: esse método deve ser refatorado junto com o método readFile
     *
     * @method readAllFile
     * @param  {Function} callbackReadAllFile
     * @return {void}
     */
    FileSyncRss.prototype.readAllFile = function doneReadAllFile (callbackReadAllFile) {
        // buscar todos os arquivos json de notícia
        // caminho mais o nome do arquivo
        var pathJson = window.projectName + '/json/';

        var that = this;

        // encadeia as funções
        async.waterfall([
            /**
             * Verifica todos os arquivos de notícias do app
             * @param  {function} callbackWaterfall
             * @return {Array}                   Lista com todos os arquivos de notícias
             */
            function __listFilesNews(callbackWaterfall) {
                var files = [];

                // Lista os arquivos em disco na pasta [4YouSee] default_project/json
                FileSync.listFiles(window.rootDirEntry, pathJson, function __doneListFiles(filesInDisk, error) {

                    // caso tenha algum erro
                    if (!error) {
                        // verificar quais arquivos tem a mais na pasta 'json', dos que tem na lista
                        files = _.filter(filesInDisk, function (item) {
                            return /rss\.player.\d+\.json/.test(item);
                        });
                    }

                    // callback
                    return callbackWaterfall(null, files);
                });
            },

            /**
             * Método para processar os arquivos e gerar a lista com template e imagens
             * @param  {array} listFile          Lista dos arquivos de notícia
             * @param  {function} callbackWaterfall
             * @return {object}                   Lista com todos os arquivos das notícias
             */
            function __processFile (listFile, callbackWaterfall) {
                // lista com todos os arquivos dos templates
                var listFiles = {
                    template: [],
                    image: []
                };

                // percorre cara arquivo
                async.each(listFile, function __eachProcessFileList (media, nextEach) {
                    // ler o arquivo xml
                    FileSync.readJSONFile(window.rootDirEntry, pathJson + media, function doneReadJSONFile(jsonFile, error) {

                        // em caso de erros
                        if (_.isNull(jsonFile) || error) {

                            // callback
                            return nextEach();
                        }

                        var itens = jsonFile.itens;

                        /**
                         * Verifica cada item da lista
                         * @param  {array} item  Lista dos arquivos
                         * @return {void}
                         */
                        async.each(itens,
                            function __processItemNews(item, next) {
                                // verifica o tipo de arquivo
                                if(('imagem' in item) && ('template' in item)) {
                                    // adiciona o template
                                    listFiles.template.push(item.template);
                                    // adiciona as imagens
                                    listFiles.image.push(item.imagem);
                                }

                                return next();
                            },
                            function __doneReadJSONFileNews() {
                                nextEach();
                            }
                        );
                    });
                },
                function __doneEachProccessFile () {
                    callbackWaterfall(null, listFiles);
                });
            }
        ],
        /**
         * Método para retornar finalizar a lista
         * @param  {object} listFile lista de arquivos das notícias
         * @return {void}
         */
        function __doneWaterFall(error, listFiles) {
            callbackReadAllFile(listFiles);
        });
    }

    /**
     * Método para excluir todos os templates e json das notícias que não são mais necessárias para o player
     * TODO: esse método deve ser refatorado junto com o método readFile
     *
     * @method verifyFileForDelete
     * @param  {Function} callbackVerifyFileForDelete
     * @return {void}
     */
    FileSyncRss.prototype.verifyFileForDelete = function doneVerifyFileForDelete (listFiles, callbackVerifyFileForDelete) {
        var pathTemplateNews = window.projectName + '/' + FileSync.PLAYER_DIR_RSS_TEMPLATE;
        var pathImageNews = window.projectName + '/' + FileSync.PLAYER_DIR_RSS_IMG;

        /**
         * Método para lista os arquivos da pasta
         * @param  {string}   folder
         * @param  {Function} callback
         * @return {void}
         */
        function listFolder (folder, callback) {
            // Lista os arquivos em disco
            FileSync.listFiles(window.rootDirEntry, folder, callback);
        }

        var filesDelete = {
            template: [],
            image: []
        }

        async.series([
            /**
             * Método para realizar o diff entre a lista de arquivos válido e os excedentes dos templates
             * @param  {function} nextSerie
             * @return {void}
             */
            function __listFileTemplateNews (nextSerie) {
                listFolder (pathTemplateNews, function (filesInDisk, error) {
                    // caso não haja erros
                    if (!_.isNull(filesInDisk) && !error) {
                        // verificar quais arquivos tem a mais na pasta de template
                        filesDelete.template = _.filter(filesInDisk, function (item) {
                            return _.indexOf(listFiles.template, item) == -1;
                        });
                    }

                    // callback
                    return nextSerie();
                });
            },

            /**
             * Método para realizar o diff entre a lista de arquivos válido e os excedentes das imagens
             * @param  {function} nextSerie
             * @return {void}
             */
            function __listFileImageNews (nextSerie) {
                listFolder (pathImageNews, function (filesInDisk, error) {
                    // caso não haja erros
                    if (!_.isNull(filesInDisk) || !error) {
                        // verificar quais arquivos tem a mais na pasta de imagens
                        filesDelete.image = _.filter(filesInDisk, function (item) {
                            return _.indexOf(listFiles.image, item) == -1;
                        });
                    }

                    // callback
                    return nextSerie();
                });
            },
        ],
        /**
         * Método para excluir todos os arquivos excedentes do template e imagem
         * @return {void}
         */
        function __doneEachFileList() {
            var nameFile, i;

            ///////////////////////////////////////
            // excluindo os arquivos de template //
            ///////////////////////////////////////
            FileSync.listFilesEntries(window.rootDirEntry, pathTemplateNews, function(filesEntity, error){
                for (i in filesEntity) {
                    // gamb para que eu ainda não sei
                    // mas o motivo, é de terem adicionar .zip no nome da pasta
                    nameFile = !filesEntity[i].isDirectory? filesEntity[i].name : filesEntity[i].name + '.zip';

                    if (_.indexOf(filesDelete.template, nameFile) != -1) {
                        FileSync.deleteFile(filesEntity[i]);
                    }
                }
            });

            /////////////////////////////////////
            // excluindo os arquivos de imagem //
            /////////////////////////////////////
            FileSync.listFilesEntries(window.rootDirEntry, pathImageNews, function(filesEntity, error){
                for (i in filesEntity) {
                    // gamb para que eu ainda não sei
                    // mas o motivo, é de terem adicionar .zip no nome da pasta
                    nameFile = !filesEntity[i].isDirectory? filesEntity[i].name : filesEntity[i].name + '.zip';

                    if (_.indexOf(filesDelete.image, nameFile) != -1) {
                        FileSync.deleteFile(filesEntity[i]);
                    }
                }
            });

            callbackVerifyFileForDelete(true);
        });
    }

    window.FileSyncRss = new FileSyncRss();
}) (window);
