// scripts/FileSync/FileSyncRssNews.js
(function (window) {
    'use strict';

    /**
     * Classe para gerenciar o processamento dos arquivos de nóticia
     *
     * @class FileSyncRssNews
     * @type {Function}
     */
    var FileSyncRssNews = FileSyncInterface(window).clone();

    /**
     * Tipo de extensões permitidas para download
     *
     * @property {array} typesAccepts
     * @type {Array}
     */
    FileSyncRssNews.typesAccepts = ['zip', 'html', 'htm'];

    /**
     * Quando não se deve realizar o download da imagem
     *
     * @property {string} NOT_FOUND_IMAGE
     * @type {String}
     */
    FileSyncRssNews.NOT_FOUND_IMAGE = '0.png';

    /**
     * Nome do arquivo de categoria
     * @type {String}
     */
    FileSyncRssNews.nameFile = '';

    /**
     * Objeto para registrar todos os arquivos de retornaram 404
     *
     * @property {object} ifStop
     * @type {Object}
     */
    FileSyncRssNews.ifStop = {};

    /**
     * Quantidade máxima de tentativas de download
     *
     * @property {integer} maxStop
     * @type {Number}
     */
    FileSyncRssNews.maxStop = 2;

    /**
     * Método para realizar o download dos arquivos de media
     *
     * @method downloadFileMedia
     * @param  {string} file        Caminho do arquivo para ser feito o download
     * @param  {Function} endCallback
     * @return {void}
     */
    FileSyncRssNews.prototype.downloadFileMedia = function (file, endCallback) {
        var downloadItem = _.clone(FileSyncRssNews.ObjectDownloadItem);
        downloadItem.file = file;
        downloadItem.type = FileSync.FILE_TYPE_XML;
        downloadItem.force = true;

        // adicionando o arquivo para download
        DownloadCenter.addDownload(downloadItem);

        return endCallback();
    };

    /**
     * Método para adicionar item a fila de download
     *
     * @method addDownloadFile
     * @param {Object} objectFile
     * @param {Function} endCallback
     */
    FileSyncRssNews.prototype.addDownloadFile = function (objectFile, endCallback) {
        // pega a extenção do arquivo especificado
        var mediaType = objectFile._template.split('.').pop();

        // verifica o tipo do arquivo
        if (_.indexOf(FileSyncRssNews.typesAccepts, mediaType)) {

            //////////////////////////
            // download do template //
            //////////////////////////
            var downloadItem = _.clone(FileSyncRssNews.ObjectDownloadItem);
            downloadItem.file = objectFile._template;
            downloadItem.type = FileSync.FILE_TYPE_LAYOUT_TEMPLATE;
            downloadItem.force = true;

            // adicionando o arquivo para download
            DownloadCenter.addDownload(downloadItem);

            ////////////////////////
            // download da imagem //
            ////////////////////////

            //Verique se imagem usada na notíca não é 0.png
            if(objectFile._imagem != FileSyncRssNews.NOT_FOUND_IMAGE){
                downloadItem = _.clone(FileSyncRssNews.ObjectDownloadItem);
                downloadItem.file = objectFile._imagem;
                downloadItem.type = FileSync.FILE_TYPE_LAYOUT_RSS_IMG;
                downloadItem.force = true;

                // adicionando o arquivo para download
                DownloadCenter.addDownload(downloadItem);
            }
        }

        // callback
        return endCallback();
    };

    /**
     * Método para ler o arquivo media
     *
     * @method readFile
     * @param  {Object} file
     * @param  {Function} callbackReadFile
     * @return {void}
     */
    FileSyncRssNews.prototype.readFile = function (file, callbackReadFile) {
        // caminho mais o nome do arquivo
        var media = window.projectName + '/' + FileSync.PLAYER_DIR_LAYOUT_RSS_XML + '/' +  file;

        var that = this;

        // verifica se o arquivo existe
        FileSync.exists (media, function (fileExists) {

            // caso o arquivo não exista
            if (!fileExists) {
                return callbackReadFile(undefined);
            }

            // ler o arquivo xml
            FileSync.readXmlFile(window.rootDirEntry, media, function __doneReadXmlFileReadFile(jsonFile, msgError) {

                // em caso de erros
                if (_.isNull(jsonFile) || msgError) {

                    // callback
                    callbackReadFile();

                    // tenta recurar a lista
                    return that.recuperar(file, 'Arquivo não encontrado.');
                }

                // caso não tenha notícias
                // if (!('item' in jsonFile.root) && !('itens' in jsonFile.root)) {
                //     return callbackReadFile();
                // }

                // os arquivos rss.<id_fonte>.xml vem como item
                var itens = [];
                if ('item' in jsonFile.root) {
                    itens = jsonFile.root.item;

                    // caso não seja um array
                    if (!_.isArray(itens)) {
                        // download do item
                        downloadItem(itens, callbackReadFile);
                        return;
                    }
                }

                // o arquivo rss.player.<id-player>.json, as notícias vem com itens
                if ('itens' in jsonFile.root) {
                    itens = jsonFile.itens;
                }

                /**
                 * Método para realizar o download do conteúdo da notícia
                 * @param  {Object} item
                 * @param  {Function} next
                 */
                function downloadItem(item, next) {
                    // verifica o tipo de arquivo
                    // if(!('texto' in item) || !('template' in item)) {
                    if(!('_fonte' in item) || !('_template' in item)) {

                        _logError('Objeto inválio.', item);
                        return next();
                    }

                    // caso o item esteja com a programação vencida
                    // o download do arquivo não deverá ser feito
                    if (!that.dateValidateDownload(item)) {
                        _log('Conteúdo com a programação vencida.', item.arquivo);
                        return next();
                    }

                    // download do conteúdo
                    that.addDownloadFile(item, next);
                }

                // caso esteja vazio
                if (!itens.length) {
                    return callbackReadFile();
                }

                /**
                 * Verifica cada item da lista para realizar o download
                 * @param  {array} item  Lista dos arquivos
                 * @return {void}
                 */
                async.each(itens,
                    downloadItem,
                    function __doneReadXmlFile() {
                        callbackReadFile();
                    }
                );
            });
        });
    };

    /**
     * Método para recurar, caso exista algum erro
     *
     * @method recuperar
     * @param {string} file Nome do arquivo a ser processado
     * @param {string} msg Mensagem de erro
     * @return {void}
     */
    FileSyncRssNews.prototype.recuperar = function recuperar(file, msg) {
        var media = file;
        var that = this;

        // reporta o erro
        _logError(msg);

        var fileNameCount = file.replace(/\./g, '');

        // conta quantos erros
        if (fileNameCount in FileSyncRssNews.ifStop) {
            FileSyncRssNews.ifStop[fileNameCount]++;
        } else {
            FileSyncRssNews.ifStop[fileNameCount] = 0;
        }

        // condição de parada
        if (FileSyncRssNews.maxStop <= FileSyncRssNews.ifStop[fileNameCount]) {
            _logError('Nº máximo de tentativas de download excedido para o arquivo ' + file + '.');
            return;
        }

        // download do arquivo
        this.downloadFileMedia(media, function downloadProcessFile() {

            // o processamento fica em paralelo
            that.readFile(file, function doneProcesseFile(error) {
                if (!error) {
                    // console.log("processFileNewsTemplate: Arquivo " + file + " foi processado com sucesso.");

                    // reseta contador
                    FileSyncRssNews.ifStop[file] = 0;
                }

            });
        });
    };

    window.FileSyncRssNews = new FileSyncRssNews();
}) (window);
