// scripts/FileSync/FileSyncSelecao.js
(function (window) {
    'use strict';

    /**
     * Classe para gerenciar o processamento dos arquivos selecao
     *
     * @class FileSyncSelecao
     * @type {Function}
     */
    var FileSyncSelecao = FileSyncInterface(window).clone();

    /**
     * Nome do arquivo de categoria
     *
     * @property {string} nameFile
     * @type {String}
     */
    FileSyncSelecao.nameFile = '';

    /**
     * Variável com o caminho dos arquivos a serem lidos
     * @property {string} filePath
     * @type {String}
     */
    FileSyncSelecao.filePath = '';


    /**
     * Método para ler o arquivo
     *
     * @method readFile
     * @param  {Object} file
     * @param  {Function} callbackReadFile
     * @return {void}
     */
    FileSyncSelecao.prototype.readFile = function (file, callbackReadFile) {
        // caso não venha o nome do arquivo
        if (_.isEmpty(file)) {
            return callbackReadFile();
        }

        var that = this;

        // caminho mais o nome do arquivo
        var media = window.projectName + '/json/' + file;

        // ler o arquivo
        FileSync.readJSONFile(window.rootDirEntry, media, function doneReadJSONFile(jsonFile, error) {
            // em caso de erros
            if (_.isNull(jsonFile) || error) {

                // callback
                callbackReadFile();

                // tenta recurar a lista
                return that.recuperar(file, 'Arquivo não encontrado.');
            }

            // json
            var itens = jsonFile.selecao;

            // callback
            callbackReadFile(itens);
        });
    }

    /**
     * Método para excluir os arquivos desnecessários
     *
     * @method deleteFiles
     * @param  {Array} itens              Lista de arquivos que devem ficar no player
     * @param  {Function} callbackDeleteFile
     * @return {void}
     */
    FileSyncSelecao.prototype.deleteFiles = function (itens, callbackDeleteFile) {

        // caso esteja vazio ou não seja um array
        if (!_.isArray(itens) || _.isEmpty(itens)) {
            return callbackDeleteFile();
        }

        // caminho das pastas
        var pathMedias = window.projectName + '/' + FileSync.PLAYER_DIR_MEDIA;
        var pathAudio = window.projectName + '/' + FileSync.PLAYER_DIR_AUDIO;

        // lista de arquivos para excluir
        var deleteFiles = {
            media: [],
            audio: [],
        }

        // verifica as pasta e exclui os arquivos
        async.series([
            function verifyFilesMediaDelete (next) {
                // Lista os arquivos em disco na pasta [4YouSee] default_project/media
                FileSync.listFiles(rootDirEntry, pathMedias, function doneListFiles(filesInDisk, error) {

                    // caso tenha algum erro
                    if (!error) {
                        // verificar quais arquivos tem a mais na pasta 'media', dos que tem na lista
                        deleteFiles.media = _.filter(filesInDisk, function (item) {
                            return (_.indexOf(itens, item) == -1 && !/playlistVideo.*|.*\.tmp|.*\.backup/.test(item));
                        });
                    }

                    // callback
                    return next();
                });
            },
            function verifyFilesAudioDelete (next) {
                // Lista os arquivos em disco na pasta [4YouSee] default_project/mp3
                FileSync.listFiles(rootDirEntry, pathAudio, function doneListFiles(filesInDisk, error) {
                    // caso tenha algum erro
                    if (!error) {
                        // verificar quais arquivos tem a mais na pasta  'mp3', dos que tem na lista
                        deleteFiles.audio = _.filter(filesInDisk, function (item) {
                            return _.indexOf(itens, item) == -1;
                        });
                    }

                    // callback
                    return next();
                });
            }
        ],
        function doneVerifyFilesMediaAndAudioForDelete() {

            // callback
            callbackDeleteFile();

            // excluindo os arquivos medias
            FileSync.listFilesEntries(rootDirEntry, pathMedias, function(filesEntity, error){
                var i, nameFile;
                var pathMedias = window.projectName + '/' + FileSync.PLAYER_DIR_MEDIA + '/';
                var pathAudio = window.projectName + '/' + FileSync.PLAYER_DIR_AUDIO + '/';

                for (i in filesEntity) {

                    // gamb para que eu ainda não sei o
                    // motivo de terem adicionar .zip no nome da pasta
                    nameFile = !filesEntity[i].isDirectory? filesEntity[i].name : filesEntity[i].name + '.zip';

                    if (_.indexOf(deleteFiles.media, nameFile) != -1) {
                        filesEntity[i].name = pathMedias + filesEntity[i].name;
                        FileSync.deleteFile(filesEntity[i]);
                    }
                }
            });

            // excluindo os arquivos de audio
            FileSync.listFilesEntries(rootDirEntry, pathAudio, function(filesEntity, error){
                var i;
                for (i in filesEntity) {
                    if (_.indexOf(deleteFiles.audio, filesEntity[i].name) != -1) {
                        filesEntity[i].name = pathAudio + filesEntity[i].name;
                        FileSync.deleteFile(filesEntity[i]);
                    }
                }
            });
        }
        );
    };

    /**
     * Método para verificar quais arquivos de conteúdo não são mais necessários para o player
     *
     * @method processFileSelecao
     * @param  {Object} listData
     * @param  {Function} finishCallback
     * @return {void}
     */
    FileSyncSelecao.prototype.processFileSelecao = function(finishCallback) {
        // callback
        finishCallback = (typeof finishCallback === 'function'? finishCallback : function() {});

        // caminho para pasta onde estão os arquivos categoria.<idplayer>.json
        var pathFilesSelecao = window.projectName + '/' + FileSync.PLAYER_DIR_JSON;

        var that = this;

        // juntar todos os conteúdos de ambos os players
        // excluir conteúdo

        // iniciando o callback Hell
        FileSync.listFiles (window.rootDirEntry, pathFilesSelecao, function doneListFiles(listFiles) {

            //Uni todos os arquivos selecao.<>.json
            that.joinFiles(listFiles, ['selecao\.[0-9]+\.json'], function doneJoinFiles(listFilesDelete) {

                // excluindo arquivos
                that.deleteFiles(listFilesDelete, function doneDeleteFile() {
                    // console.log("Arquivos excluídos com sucesso.");

                    // callback
                    finishCallback();
                });
            });
        });
    }

    window.FileSyncSelecao = new FileSyncSelecao();
}) (window);
