// script/Layout/LayoutArea.js
'use strict';

(function (window) {

    var idElements = 0;

    /**
     * Classe para gerenciar as áreas do layout
     *
     * @class LayoutArea
     * @constructor
     * @param {Object} jsonArea Json da área
     */
    function LayoutArea(jsonArea) {
        // elemento html
        this.jsonArea = jsonArea;
        // caso a área seja a principal
        this.main;

        // elementHtml
        this.elementHtml;
    }


    /**
     * Método para criar os elementos do layout
     *
     * @method createElement
     * @return {Element}             Elemento html
     */
    LayoutArea.prototype.createElement = function (areaExist) {
        var idElement = 'area_' + ((idElements++) + '');

        // caso a área já exista
        if (!areaExist) {
            // cria elemento
            this.elementHtml = document.createElement('div');
            // define um id
            this.elementHtml.id = idElement;
            // cor de fundo
            if ('background' in this.jsonArea) {
                // this.elementHtml.style.background = this.jsonArea.background;
                // this.elementHtml.style.background = '#fff';
            }
            // área principal
            // this.elementHtml.setAttribute('allowtransparency', 'true');
            // this.elementHtml.style.backgroundColor = 'transparent';
            this.elementHtml.style.border = 0; //'1px solid yellow';
            this.elementHtml.setAttribute('data-principal', this.jsonArea.main);
            this.setMain(this.jsonArea.main);

            // atributos de áreas auxiliares
            if (!this.main) {
                if ('fonte' in this.jsonArea) {

                    // atributo
                    this.elementHtml.setAttribute('data-fonte', this.jsonArea.fonte);
                    // setando a fonte
                    this.setFonte(this.jsonArea.fonte);
                }
                if ('template' in this.jsonArea) {
                    this.elementHtml.setAttribute('data-template', this.jsonArea.template);
                    // setando o template
                    this.setTemplate(this.jsonArea.template);
                }
            }
            // executa a criação do conteudo
            var elementVideos = this.createWebView(this.jsonArea);
            var elementWebview = this.createVideo(this.jsonArea);

            // adiciona como filho do element
            // webview
            this.elementHtml.appendChild(elementWebview);
            // vídeo
            this.elementHtml.appendChild(elementVideos);

            // limpa a memória
            var elementWebview = null;
            var elementVideos = null;
        }

        // posição elemento
        // div
        this.elementHtml = this.positionElement(this.elementHtml, this.jsonArea);


        return this.elementHtml;
    };

    /**
     * Método para limpar todos os elementos dentro da div
     *
     * @method clearElement
     * @param  {ElementHtml} elementContainer
     * @return {void}
     */
    LayoutArea.prototype.clearElement = function (elementContainer) {
        var i, area;
        var countArea = elementContainer.children.length;
        for (i = 0 ; i < countArea; i++) {
            // remover o conteúdo da div container
            area = elementContainer.children[0];

            if (area != undefined) {
                area.remove();
            }
        }
    }

    /**
     * Método para criar o elemento da webview
     *
     * @method createWebView
     * @param  {json} jsonArea
     * @return {Element}
     */
    LayoutArea.prototype.createWebView = function (jsonArea) {
        // cria a webview
        var elementWebview = document.createElement('iframe');
        // trocando o partition
        elementWebview.setAttribute('partition', 'webview_' + (Math.random() * 10));
        // elementWebview.setAttribute('allowtransparency', 'true');
        elementWebview.style.opacity = 0;
        elementWebview.style.position = 'absolute';
        elementWebview.style.top = 0;
        elementWebview.style['z-index'] = 1;
        elementWebview.style.width = '100%';
        // elementWebview.style.right  = 0;
        // elementWebview.style.backgroundColor = 'transparent';
        // elementWebview.style.background = '#000000';
        elementWebview.style.border = 0; //'1px solid red';

        return elementWebview;
    };

    /**
     * Método para criar o elemento do video
     *
     * @method createVideo
     * @param  {Element} Element
     * @param  {json} jsonArea
     * @return {Element}
     */
    LayoutArea.prototype.createVideo = function (jsonArea) {
        // cria o video
        var elementVideos = document.createElement('video');

        // autoplayer
        // elementVideos.setAttribute('autoplay', true);
        // elementVideos.setAttribute('allowtransparency', 'true');
        elementVideos.style.opacity = 0;
        elementVideos.style.position = 'absolute';
        elementVideos.style.top = 0;
        elementVideos.style['z-index'] = 9;
        elementVideos.style.backgroundColor = '#000';
        elementVideos.style.border = 0; //'1px solid green';

        return elementVideos;
    };

    /**
     * Método para definir a posição do elemento
     *
     * @method positionElement
     * @param  {Element} element
     * @param  {json} jsonArea
     * @return {Element}
     */
    LayoutArea.prototype.positionElement = function (element, jsonArea) {

        if (!('rect' in jsonArea)) {
            _logError('json Inválido. Está faltando a chave Rect.');
            return element;
        }

        // valores
        var values = jsonArea.rect.split(',');
        // É obrigatório ter 4 elementos
        if (values.length != 4) {
            _logError('Quantidade de elementos inválidos.');
            return element;
        }

        // relação dos itens
        // 0: left | 1: top | 2: width | 3: height
        element.style.left = values[0];
        element.style.top = values[1];
        element.style.width = values[2];
        element.style.height = values[3];
        element.style.position = 'absolute';

        return element;
    }

    /**
     * Método para retornar o element html
     *
     * @method getElementHtml
     * @return {Element}
     */
    LayoutArea.prototype.getElementHtml = function () {
        return this.elementHtml;
    }

    /**
     * Método para setar a fonte
     *
     * @method setFonte
     * @param {string} fonte
     */
    LayoutArea.prototype.setFonte = function (fonte) {
        this.fonte = fonte;
    };

    /**
     * Método para retornar qual é a fonte
     *
     * @method getFonte
     * @return {string}
     */
    LayoutArea.prototype.getFonte = function () {
        return this.fonte;
    };

    /**
     * Método para setar o template
     *
     * @method setTemplate
     * @param {string} template
     */
    LayoutArea.prototype.setTemplate = function (template) {
        this.template = template;
    };

    /**
     * Método para retornar qual é o template
     *
     * @method getTemplate
     * @return {string}
     */
    LayoutArea.prototype.getTemplate = function () {
        return this.template;
    };


    /**
     * Método para setar o json da área
     *
     * @method setJsonArea
     * @param {string} template
     */
    LayoutArea.prototype.setJsonArea = function (jsonArea) {
        this.jsonArea = jsonArea;
    };

    /**
     * Método para retornar qual é o json da área
     *
     * @method getJsonArea
     * @return {string}
     */
    LayoutArea.prototype.getJsonArea = function () {
        return this.jsonArea;
    };

    /**
     * Método para setar se a área é principal ou não
     *
     * @method setMain
     * @param {string} value
     */
    LayoutArea.prototype.setMain = function (value) {
        var valueMain = false;
        if (value === "true") {
            valueMain = true;
        }


        this.main = valueMain;
    }

    /**
     * Método para retornar se uma área é a principal
     *
     * @method getMain
     * @return {boolean}
     */
    LayoutArea.prototype.getMain = function () {
        return this.main;
    }

    window.LayoutArea = LayoutArea;
}) (window);
