(function (){
    /**
     * Classe para download dos arquivos de template
     *
     * @class LayoutTemplateDownload
     * @param {integer} _id
     * @param {Directory} rootDirEntry
     * @param {string} projectName
     * @param {string} env4YouSee
     */
    function LayoutTemplateDownload(_id, rootDirEntry, projectName, env4YouSee){
        // console.log('Iniciando LayoutTemplateDownload ', _id, new Date());
        this.index = _id;
        LayoutTemplateDownload.commonUrl = env4YouSee + '/common/templates/';
        LayoutTemplateDownload.rootDirEntry = rootDirEntry;
        LayoutTemplateDownload.projectName = projectName;
    }

    /**
     * Método para iniciar o download do conteúdo
     *
     * @method download
     * @return {void}
     */
    LayoutTemplateDownload.prototype.download = function (genericItem, callback){

        if(genericItem != undefined){
            var fType = genericItem.type;
            var extFile = genericItem.file.split('.')[1];
            var nameFileZip = genericItem.file.split('.')[0] + '/index.html';

            // save generic files

            // nome do arquivo
            // caso seja zip o nome da pasta deve ser removido
            var nameFile = (extFile === 'zip'? nameFileZip  : genericItem.file);
            var pathFilesMedia = '/' + window.projectName + '/' + FileSync.PLAYER_DIR_LAYOUT + '/' + FileSync.FILE_TYPE_RSS_TEMPLATE + '/';


            // caso seja para forçar
            if ('force' in genericItem && genericItem.force) {
                // video
                if (extFile != 'zip') {
                    player4uc.filesystem.rm(pathFilesMedia + nameFile, function __doneDeleteFile() {});
                }
                else {
                    // zip
                    player4uc.filesystem.rm(pathFilesMedia + genericItem.file, function __doneDeleteFileZip() {});

                    // excluindo zips
                    player4uc.filesystem.rmdir(pathFilesMedia + genericItem.file.split('.')[0], {recursive: true}, function __doneDeleteFile(error, data) {
                        _log({
                            'Arquivos de templates excluídos':genericItem.file,
                            'error': error,
                            'data': data,
                        });
                    });
                }

                // força o download
                return downloadContent(callback, true);
            }

            // verifica se o arquivo já existe
            FileSync.exists(
                pathFilesMedia + nameFile,
                function (fileExist) {
                    // caso o arquivo já exista
                    if (!fileExist) {

                        // _log('Iniciando o download do arquivo:' + genericItem.file);
                        return downloadContent(callback);
                    }

                    // callback
                    _log('O arquivo ', genericItem.file, ' já foi baixado.');
                    callback(genericItem, DownloadCenter.FOUND);
                },
                false
            );

            /**
             * Método para realizar download do conteúdo
             * @param  {Function} callback
             * @return {void}
             */
            function downloadContent(callback, force) {
                var that = this;
                var hash = '';

                // forçar o download
                if (force) {
                    hash = '?' + new Date().getTime();
                }

                // url para download
                var urlDownload = LayoutTemplateDownload.commonUrl + genericItem.file + hash;

                // inicia o download dos arquivos
                player4uc.download.start({
                    url: urlDownload,
                    path: pathFilesMedia,
                }, function __doneDownloadStart(error, data){
                    var statusDownload = DownloadCenter.COMPLETED

                    // em caso de erro
                    if (error) {
                        statusDownload = DownloadCenter.FAILED;
                    }

                    // caso o conteúdo seja zip
                    if (extFile == 'zip') {
                        // descompacta o conteúdo zip
                        window.sync.unzipFile(LayoutTemplateDownload.rootDirEntry, pathFilesMedia + genericItem.file, urlDownload, function (result) {
                            // em caso de erro
                            if (!result) {
                                statusDownload = DownloadCenter.FAILED;
                            }

                            // _log("Conteúdo " + genericItem.file + " foi descompactado. Status: " + statusDownload);

                            // excluindo zips
                            player4uc.filesystem.rmdir(pathFilesMedia + genericItem.file, {recursive: true}, function __doneDeleteFile(error, data) {
                                _log({
                                    'Arquivos de templates excluídos':genericItem.file,
                                    'error': error,
                                    'data': data,
                                    'Status': statusDownload,
                                });
                            });

                            // callback
                            callback(genericItem, statusDownload);
                        });
                        return;
                    }

                    // callback
                    callback(genericItem, statusDownload);
                });
            }
        }
    };

    window.LayoutTemplateDownload = LayoutTemplateDownload;
})();
