(function() {
	function ExtractDomainUrl(url){
    	var url = $.trim(url);
    	if(url.search(/^https?\:\/\//) != -1)
        	url = url.match(/^https?\:\/\/([^\/?#]+)(?:[\/?#]|$)/i, "");
    	else
        	url = url.match(/^([^\/?#]+)(?:[\/?#]|$)/i, "");
    	return url[1];
	}

	function ManagerConnect(playerId, env4YouSee) {
		var that = this;
		this.playerId = playerId;
		this.env4YouSee = env4YouSee || '';
		this.playLogList = [];

		//Extrair da url do manager o endereço de conexão do socket
		var urlConnection = ExtractDomainUrl(this.env4YouSee);
		_log('urlConnection',urlConnection);

		var socketURL = this.env4YouSee.replace(/^\/|\/$/g, '');
		_log('socketURL',socketURL);

		this.socket = new io('http://' + urlConnection + ':3000', {
			'query': 'playerId=' + playerId + '&socketURL=' + socketURL.replace('http://', ''),
			'reconnection': true,
			'reconnectionDelay': 1000,
			'forceNew':true
        });

        this.log = createLogger(this.socket);

        //Registra o listener para os eventos (on_message, playlog, playerlog)
        this.listenUpdate();
        this.log.listen('playlog');
        this.log.listen('playerlog');

        this.socket.on('connect', function() {
            _log( 'Manager connection successfully');
        });

        // fica vigiando o socket para restaurar a conexão caso caia
    	startServerPing(that.socket);
	}

	function startServerPing(socket) {
		var that = this;
		var count = 0;
		that._pingTimeout = setInterval(function(){

			if(!socket.connected) {
				_log('Trying reconnect socket');
				socket.disconnect();
				socket.connect();
			}

			// Força a conexão em caso de o "connected" não for setado corretamente
			if(count >= 5) {
				_log('Trying reconnect socket');
				socket.disconnect();
				socket.connect();
				count = 0;
			}

			count ++;
			socket.emit('ping', {}, function() {
				count = 0;
				// _log('-- startServerPing --');
			});


		}, 30000);
	}

	function createLogger(socket, time, limit){
		var that = {
			socket : socket,

			logNameMap : {
				"playlog" : {
					"create" : "create_play_log",
					"saved"  : "play_log_saved"
				},

				"playerlog" : {
					"create" : "create_player_log",
					"saved"  : "player_log_saved"
				}
			},

			get: function (logName, callback) {

				player4uc.storage.get(['settings.log.'+logName], function (storageObj) {
					var logs = storageObj['settings.log.'+logName] || [];
					callback(logs);
				});
			},

			set : function (logName, log, callback) {

				var storageObj = {};

				that.get(logName, function(logs){
					if(!logs)
						logs = [];

					logs.push(log);
					storageObj['settings.log.'+logName] = logs;

                    // console.log("managerConnect set: logName, storageObj", logName, storageObj);

					player4uc.storage.set(storageObj);

					if(typeof(callback) == 'function')
						callback(logs);
				});
			},

			save : function(logName, arquivo, tipo) {

				if((!logName in that.logNameMap))
					return;

				var mapedLogName = that.logNameMap[logName].create;


				var log = {
					code: CryptoJS.SHA1((new Date().getTime() * Math.random()).toString()).toString(),
					tstam: moment(new Date()).format('YYYY-MM-DD HH:mm:ss'),
					tipo: tipo.toUpperCase() || 'I',
					status: 'pending'
				};


				if(logName == 'playlog')
					log['conteudo'] = arquivo;

				else if(logName == 'playerlog')
					log['complemento'] = arquivo;

				that.set(logName, log);
			},

			remove : function(logName, logCode){
				var log;
				var storageObj = {};

				that.get(logName, function(logs) {
					log = _.findWhere(logs, {code: logCode});
					for (var i = 0; i < logs.length; i++) {
						if(logs[i] == log) {
							logs.splice(i,1);
							break;
						}
					};

					storageObj['settings.log.'+logName] = logs;
					player4uc.storage.set(storageObj);
				});
			},

			listen : function (logName){

				var mapedLogName = that.logNameMap[logName].saved;
				that.socket.on(mapedLogName, function (data) {
					that.remove(logName, data.code);
				});
			},

			send : function (logName) {

				if(!that.socket.connected)
					return;

				var mapedLogName = that.logNameMap[logName].create;
				var storageObj = {};

				that.get(logName, function(logs) {
					var logsPending = _.filter(logs, function (log){
						return log.status == 'pending';
					});

					if(logsPending.length == 0)
						return;

					var total = (limit <= logsPending.length) ? limit : logsPending.length;

					for (var i = 0; i < total; i++) {
						delete logsPending[i].status;
						that.socket.emit(mapedLogName, logsPending[i]);

                        // console.log('managerConnect send: mapedLogName, logsPending[i]', mapedLogName, logsPending[i]);
					};

					storageObj['settings.log.'+logName] = logs;
					player4uc.storage.set(storageObj);
				});
			},

			changeStatus : function(logName) {
				var storageObj = {};

				that.get(logName, function(logs) {
					var logsToSend = _.filter(logs, function (log){
						return !log.status;
					});

					if(logsToSend.length == 0)
						return;

					var total = (limit <= logsToSend.length) ? limit : logsToSend.length;

					for (var i = 0; i < total; i++) {
						logsToSend[i].status = 'pending';
					};

					storageObj['settings.log.'+logName] = logs;
					player4uc.storage.set(storageObj);
				});
			}
		}

		time = time || 2000;
		limit = limit || 10;
		var timer = setInterval(function _timer(){
			that.send('playlog');
			that.send('playerlog');
		}, time);

		// Altera status dos logs que não foram enviados
		var timerToStatusChange = setInterval(function _timerToStatusChange(){
			that.changeStatus('playlog');
			that.changeStatus('playerlog');
		}, 15000);

		if(typeof(callback) == 'function')
			callback();

		return that;
	}

    /**
     * Setar o id do player
     *
     * @method setPlayerId
     * @return {void}
     */
    ManagerConnect.prototype.setPlayerId = function __donePlayerId(playerId) {
        this.playerId = playerId;
    }

    /**
     * Setar o ambiente do player
     * @method setEnv4YouSee
     * @return {void}
     */
    ManagerConnect.prototype.setEnv4YouSee = function __doneEnv4YouSee(env4YouSee) {
        this.env4YouSee = env4YouSee || '';
    }


	ManagerConnect.prototype.sendNotification = function(fileName, callback, notifySocket) {

		var fileList = [];
		var filesInUrl ='';
		var urlToNotificate ='';

		if(!fileName)
			return _logError(fileName + ' is not defined');

		if(typeof(fileName) == 'string')
			fileList = [fileName];
		else
			fileList = fileName;

		if(notifySocket) {
			for (var i = 0; i < fileList.length; i++) {

				var fileName = fileList[i];
		        this.socket.emit('on_message_complete', {
		            fileName: fileName
		        });
			};
		}

		filesInUrl = fileList.join(';');

		// http://dse.4yousee.com.br/flash/rotinas/processa_resposta_android.php?player_id=3&files=media.1.0.json;
		urlToNotificate = this.env4YouSee + '/flash/rotinas/processa_resposta_android.php?player_id=' + this.playerId + '&files=' + filesInUrl + ';';

        // caso não exista nenhum arquivo
        if (_.isNull(filesInUrl) || fileList.length == 0) {
            return;
        }

		// var http = new HttpRequest();
		_$.get(urlToNotificate, function(data, status) {

			if(callback && typeof(callback) == 'function')
				callback();
		})
        .fail(function failSendNotification() {
            // console.log('failSendNotification');

            // callback
            if(callback && typeof(callback) == 'function')
                callback();
        });
	}

	ManagerConnect.prototype.listenUpdate = function() {

		var that = this;
		if(!this.listenUpdateCb || typeof(this.listenUpdateCb) != 'function')
			return _logErro('callback is not a function');

		//Trata eventos vindos do socket. Atualização do list.<idPlayer>.json, alteração de videos os templates
		this.socket.on('on_message', function on_message(data){
			_log('sync on_message', data);

			// Veririca se alteração ocorrida foi no list e dispara o callback para tratar.
			// Atualização de playlists e lista de notícias
			if(data.fileType == 'list')
				return that.listenUpdateCb(data);

			else if (data.fileType == 'video') {
                // verificar se o arquivo já existe
                verifyFileExistVideo(data, function(error){
                    // caso o arquivo não exista
                    if(!error){
                        return;
                    }

                    // download do conteúdo
                    downloadContent(data, FileSync.FILE_TYPE_MEDIA);
                });
            }
            else if (data.fileType == 'template'){
                // verificar se o arquivo já existe
                verifyFileExistTemplate(data, function(layout, main, foundFile){

                    _log({
                        'Class': 'ManagerConnect',
                        'layout': layout,
                        'main': main,
                        'foundFile': foundFile,
                    })

                    if (layout) {
                        // download do conteúdo
                        downloadContent(data, FileSync.FILE_TYPE_LAYOUT_TEMPLATE, data.fileName);

                        // TODO: RESETAR O LAYOUT
                        setTimeout(function() {
                            EventLayout.trigger('loadLayout', [player4uc.window.current(), undefined, undefined, 1]);
                        }, 10000);
                    }

                    if (main) {
                        // download do conteúdo
                        downloadContent(data, FileSync.FILE_TYPE_RSS_TEMPLATE, data.fileName);
                    }

                });
			};

            // avisa o socket
           that.onMessageComplete(data.fileName);
		});

        // TODO: SERÁ NECESSÁRIO IMPLEMENTAR O EVENTO 'player_content_list'

        // menu de configuração do socket
        this.socket.on('get_settings', function __done(data) {
            ConfigurationSocket.listen(data);
        });

        this.socket.on('save_settings',function(settings){
            _log('save_settings', settings);

            ConfigurationSocket.saveSetting(settings);
        });
	}

    /**
     * Método para realizar o download do conteúdo de template ou media
     * @param  {json} data
     * @return {void}
     */
    function downloadContent (data, fType) {
        var downloadItem = {file: data.fileName, type: fType, url: null, notify: true, trying: 0, force: true};
        DownloadCenter.addDownload(downloadItem);
    }

     /**
     * Método para verificar se o arquivo existe de vídeo
     * @param  {Object} data
     * @param  {Function} endCallback
     * @return {void}
     */
    function verifyFileExistVideo (data, endCallback) {
        var fileName = data.fileName;
        var extFile = fileName.split('.').pop();

        // caminho para o arquivo index
        var pathFile = window.projectName + '/' + FileSync.PLAYER_DIR_MEDIA;

        // caso o arquivo seja do tipo zip
        if (extFile === 'zip') {
            fileName = fileName.split('.')[0] + '/index.html';
        }

        // busca o arquivo
        FileSync.exists(pathFile + '/' + fileName, endCallback);
    }

    /**
     * Método para verificar se o arquivo existe
     * @param  {Object} data
     * @param  {Function} endCallback
     * @return {void}
     */
    function verifyFileExistTemplate (data, endCallback) {

        // caminho para o arquivo index
        var pathFileIndexMain = window.projectName + '/' + FileSync.PLAYER_DIR_RSS_TEMPLATE + '/' + data.fileName.split('.')[0];
        var pathFileIndexLayout = window.projectName + '/' + FileSync.PLAYER_DIR_LAYOUT_TEMPLATE + '/' + data.fileName.split('.')[0];

        var layout, main, File;

        _log({
            'method': 'verifyFileExistTemplate',
            'pathFileIndexMain': pathFileIndexMain,
            'pathFileIndexLayout': pathFileIndexLayout,
        });

        // verifica se o arquivo existe
        async.series([
            function verifyTemplateMain(callbackAsync) {
                // busca o arquivo do template da área principal
                FileSync.listFilesEntries(window.rootDirEntry, pathFileIndexMain, function (foundFile, error) {
                    // caso o arquivo não exista
                    if(!error) {
                        main = true;

                        // objeto do arquivo
                        File = foundFile;
                    }

                    // next
                    callbackAsync();
                });
            },

            function verifyTemplateLayout(callbackAsync) {
                // busca o arquivo do template do layout
                FileSync.listFilesEntries(window.rootDirEntry, pathFileIndexLayout, function (foundFile, error) {
                    // caso o arquivo não exista
                    if(!error) {
                        layout = true;

                        // objeto do arquivo
                        File = foundFile;
                    }

                    // next
                    callbackAsync();
                })
            }
        ], function doneAsyncVerifyFileExistTemplate() {

            _log({
                'method': 'verifyFileExistTemplate',
                'layout': layout,
                'main': main,
                'File': File,
            });

            // callback
            endCallback(layout, main, File);
        });
    }

	ManagerConnect.prototype.onMessageComplete = function(file){
		_log('onMessageComplete', file);
		this.socket.emit('on_message_complete',{
			fileName: file
		});
	}

	// Subscribe this function to access listenUpdate Callback
	ManagerConnect.prototype.listenUpdateCb = function() {}

	window.ManagerConnect = ManagerConnect;
})();
