/**
 * PlayListManager class, that controls the playlist state
 */

(function() {

	function PlayListManager(playerId, playerUrl) {
		this.playerId = playerId;
		this.playerUrl = playerUrl;

		PlayListManager.curr = this;

		this.currentPlaylist = [];
		this.loadedFiles = [];
		this.playlistStatus = {
			current: false,
			playing: false
		};

		// this.getPlaylist(); // remover
		// this.getLoadedFiles(); // remover
		this.loadFiles();

		// listen the onfiledownload event to reload the json files whenever

		// chrome.runtime.onMessage.addListener(function(request, sender) {
		// 	// TODO: reload files only when the player media.x.y.json file is downloaded
		// 	if (typeof request["onfiledownload"] !== "undefined") {
		// 		// reload only if it is a json file
		// 		if (request.onfiledownload.match(/json$/)) {
		// 			PlayListManager.curr.loadFiles();
		// 		}
		// 	}
		// });
	}

    /**
     * Método para setar o id do player
     *
     * @method setPlayerId
     * @param {integer} playerId
     */
    PlayListManager.prototype.setPlayerId = function __doneSetPlayerId(playerId) {
        this.playerId = playerId;
    }

    /**
     * Método para setar a url do player
     *
     * @method setPlayerUrl
     * @param {string} playerUrl
     */
    PlayListManager.prototype.setPlayerUrl = function __doneSetPlayerUrl(playerUrl) {
        this.playerUrl = playerUrl;
    }

	PlayListManager.prototype.getPlaylist = function(callback) {
		// // TODO: still fake playlist
		// var currentPlaylist = [
		// 	'12' /*1151*/, '11'/*1156*/, '1'/*1208*/
		// ];

		// this.currentPlaylist = currentPlaylist;

		if (callback)
			callback(currentPlaylist);
	};

	PlayListManager.prototype.setPlaylist = function(loadedFile) {

		if(!loadedFile || typeof loadedFile != 'object')
			return;

		if (typeof loadedFile.seq != "undefined") {
			if(typeof(loadedFile.seq) == 'string')
				loadedFile.seq = loadedFile.seq.split(',');

			this.currentPlaylist = loadedFile.seq;
		} else if (loadedFiles.length) {
			this.currentPlaylist = loadedFile;
		}
	};

	PlayListManager.prototype.getLoadedFiles = function() {
		return this.loadedFiles;
	};

	PlayListManager.prototype.setLoadedFiles = function(loadedFiles) {
		// this.loadedFiles = loadedFiles;
		if (loadedFiles && 'itens' in loadedFiles && typeof loadedFiles.itens !== "undefined") {
			this.loadedFiles = loadedFiles.itens;
		} else if (loadedFiles && 'items' in loadedFiles && typeof loadedFiles.items !== "undefined") {
			this.loadedFiles = loadedFiles.items;
		} else {
			this.loadedFiles = loadedFiles;
		}
	};

	PlayListManager.prototype.loadFiles = function(day, endCallback) {
		var that = this;
		var playerId = this.playerId;
		var today = day || (new Date()).getDay();
		that.day = today;

		var path = [window.projectName, 'json', ['media.', playerId, '.', today, '.json'].join("")].join("/")

		window.FileSync.readJSONFile(window.rootDirEntry, path, function(contentFile, error) {
			// console.log("setting the loaded files");

            // caso o arquivo esteja conrrompido
            if( error || !contentFile || typeof contentFile != 'object' || !('seq' in contentFile)) {
                _logError("PlayListManager:", error);
                return;
            }

            // tudo ok com o arquivo de media
			that.setLoadedFiles(contentFile);
			that.setPlaylist(contentFile);

            // callback
            if (_.isFunction(endCallback)) {
                endCallback();
            }
		});
	};

    /**
     * Método para resetar o ponteiro da playlist
     * TODO: o legal é que o método para realizar o player saiba que eu informei 0
     * @return {void}
     */
    PlayListManager.prototype.resetPlaylistCurrent = function __doneresetPlaylistCurrent() {
        this.playlistStatus.current = -1;
    }

	/**
	 * Returns the next media in the list
	 */
	PlayListManager.prototype.getNext = function(preventUpdate) {
		var currentDay = (new Date()).getDay();

		if(!preventUpdate) {
			if (currentDay != this.day) {
				// console.warn('Happy New Day!')
				this.day == currentDay;
				this.loadFiles();
				return this.playlistStatus.current = 0;
			}
		}

		// if(currentDay != this.day)

		// if not playing, start from 0
		if (this.playlistStatus.current === false) {
			return this.playlistStatus.current = 0;

		}
		// if last item, restart from 0
		if (this.playlistStatus.current >= (this.currentPlaylist.length - 1)) {
			return this.playlistStatus.current = 0;;

		}

		return ++this.playlistStatus.current;
	};

	PlayListManager.prototype.getPlaylistItem = function(itemIndex) {

		var itemNum = this.currentPlaylist[itemIndex];
		// searches the playlist Item Object
		var item;

		if(!this.loadedFiles)
			return false;

		for (var i = 0; i < this.loadedFiles.length; i++) {
			if (this.loadedFiles[i].seq == itemNum) {
				return this.loadedFiles[i];
			}
		}

		return false;
	};

    /**
     * Método para retornar todos os carrosseis da playlist
     * @return {array}
     */
    PlayListManager.prototype.getCarrocelPlayList = function getCarrocelPlayList() {
        var i, list = [];

        for (i in this.loadedFiles) {
            if (this.loadedFiles[i].tipo == 'car') {
                list.push(this.loadedFiles[i]);
            }
        }

        return list;
    }

	window.PlayListManager = PlayListManager;
})();
