if (typeof Player !== "undefined") {
	// console.log(Player);
	throw new Error('player API already defined.');
}

(function() {
	// project name
	var proj, projPath, classRss, stopMovie, classCarrossel;
    var thisRoot;

	/**
	 * Class constructor
     *
     * @class Player
     * @constructor
	 * @param String proj Project name
	 * @param String projPath Project path
	 * @param Window window The window where the player must be injected
	 */
	function Player (projPath, appWindow, env4YouSee, playerId) {
        // root
        thisRoot = this;

		// mapping of 4yousee types to internal types
		this.typeMapping = {
            "mp4": "video",
            "zip": "html",
            "html": "html",
            "rss": "rss",
            "car": "carrossel",
            "gif": "imagem",
            "jpg": "imagem",
            "jpeg": "imagem",
			"png": "imagem",
		};

		// loads the container
        // this.app = appWindow;
		this.appWindow = appWindow;
		this.loading   = this.appWindow.document.getElementById('loading');
		this.errorMsg  = this.appWindow.document.getElementById('errorMsg');
		this.projPath = projPath;
		this.env4YouSee = env4YouSee;
		this.playerId = playerId;
        classRss = new Rss(projPath, true);
		classCarrossel = new Carrossel(projPath);

        this.connected = false;

        /**
         * Controla se o player está executando algum conteúdo
         * @type {Boolean}
         */
		this.playerActive = false;
	}

    /**
     * Método para setar o id do player
     *
     * @method setPlayerId
     * @param {integer} playerId
     */
    Player.prototype.setPlayerId = function __doneSetPlayerId(playerId) {
        this.playerId = playerId;
    }

    /**
     * Método para exibir o conteúdo
     *
     * @method showContainer
     * @param  {Element} container         Elemento html
     * @param  {String} placeholderToShow Qual conteúdo será exibido
     * @return {void}
     */
	function showContainer (container, placeholderToShow){
		placeholders = ['video', 'iframe'];

        thisRoot.loading.innerHTML = '';

        // exibe o conteúdo
        if(container.querySelector(placeholderToShow)) {
            container.querySelector(placeholderToShow).style.opacity = 1;
        }

        // suavisar a transação
        setTimeout(function() {
            for (var i = placeholders.length - 1; i >= 0; i--) {

                // caso não seja o item a ser exibido
                if(placeholderToShow != placeholders[i] && container.querySelector(placeholders[i])) {
                    container.querySelector(placeholders[i]).style.opacity = 0;

                    // caso seja conteúdo html
                    if (placeholders[i] === 'iframe') {
                        jQuery(container.querySelector(placeholders[i])).trigger('resetIframe');
                    }
                }
            };
        }, 500);
	};

    Player.prototype.showContainer = showContainer;


	function addOnHeadersReceivedListener (env4YouSee, webview, endCallback){
		if(!webview.request.onHeadersReceived.hasListeners()){
			// console.log('Adicionando listener onHeadersReceived ao webview');

			webview.request.onHeadersReceived.addListener(function(details){
				if(details.url.indexOf('index.html') > 0 && details.statusCode == 404){
					_logError('Arquivo não foi descompactado - ', details.url);
					endCallback(null, mediaFile);
				}
			},{urls: ['<all_urls>']});
		}
	}

	Player.prototype.play = function (type, item, endCallback) {
        var that = this;
		var mediaFile = item.arquivo;
		var time = item.tempo;

        this.clearTimeOutPlayer();



        // if (this.playerActive) {
        //     return;
        // }

        // solução paliativa
        // tive que adicionar o debounce para segurar o problema de loop
        // endCallback = _.debounce(endCallback, 1000);

        // caso o layout ainda não tenha sido iniciado;
        if (_.isUndefined(this.getContainer())) {
            // caso seja erro de layout o arquivo deve ser substituído
            _logError('Layout ainda não carregado.');

            // todos os arquivos precisam ser verificados
            sync.init(true, function _finishCheckProject() {
                _log('Função para tentar recuperar executada.');
            });

            return setTimeout(function() { that.play(type, item, endCallback); }, 4000);
            // return setTimeout(function() { endCallback(true, 'layout'); }, 4000);
        }

        this.playerActive = true;

        // data de inicio do conteúdo
        var a = new Date();
        var datePlay = a.getHours() + ":" + a.getMinutes() + ":" + a.getSeconds();

		if (type == 'video') {
			if(dateValidate(item)){
				_log('--> VIDEO: ', mediaFile, datePlay);
                // playe conteúdo
				this.playVideo(mediaFile, this.getContainer(), time, endCallback);
			} else {
                // TODO:excluir o conteúdo

                // callback
				endCallback('Invalid scheduling', null);
            }

		} else if (type == 'html') {
			if(dateValidate(item)){
				_log('--> HTML: ', mediaFile, datePlay);
                // playe conteúdo
				this.playHtml(mediaFile, this.getContainer(), time, endCallback, item);
			} else {
                // TODO:excluir o conteúdo

                // callback
				endCallback('--> HTML: Invalid scheduling', null);
            }

		} else if (type == 'rss') {
            _log('--> NOTÍCIA: ', item, datePlay);
            // playe conteúdo
			this.playRss(this.getContainer(), time, endCallback, item);

        } else if (type == 'carrossel') {
            _log('--> CARROSSEL: ', mediaFile, datePlay);

            // verifica se o conteúdo é valido
            if (dateValidate(item)) {
                // playe conteúdo
                this.playCarrossel(mediaFile, this.getContainer(), time, endCallback);
            } else {
                // TODO:excluir o conteúdo

                // callback
                return endCallback('--> CARROSSEL: Invalid scheduling', null);
            }

        } else if (type == 'imagem') {
            if(dateValidate(item)){
                _log('--> IMAGE: ', mediaFile, datePlay);
                // playe conteúdo
                this.playImage(mediaFile, this.getContainer(), time, endCallback);
            } else {
                endCallback('--> IMAGE: Invalid scheduling', null);
            }
        } else {
			var error = "4yousee can't recognize ";
			_logError(error, type, " as a valid media type");
			endCallback(error, null);

		}
	};

    /**
     * Método para verificar se o conteúdo a ser tocado é válido
     *
     * @method dateValidate
     * @param  {Object} item
     * @return {boolean}
     */
	function dateValidate (item) {
		var today = new Date();
		var regex = new RegExp('-', 'g');

		var startDate = (!item.startDate) ? undefined : new Date(item.startDate.replace(regex, '/'));

		var endDate = undefined;
		if(item.endDate){
			endDate = new Date(item.endDate.replace(regex, '/'));
			endDate.setHours(23);
			endDate.setMinutes(59);
			endDate.setSeconds(59);
		}

		if(startDate != undefined && today < startDate)
			return false;

		if(endDate != undefined && today > endDate)
			return false;

		var scheduleList = item.schedule;

		if(scheduleList.length == 0)
			return true;

		var isValid = false;
		scheduleList.forEach(function(schedule){

			if(!isValid && schedule.weekDays.indexOf(today.getDay()) != -1){
				var time = schedule.startTime.split(':');

				var startTime = new Date();
				startTime.setHours(time[0]);
				startTime.setMinutes(time[1]);
				startTime.setSeconds(time[2]);

				time = schedule.endTime.split(':');
				var endTime = new Date();
				endTime.setHours(time[0]);
				endTime.setMinutes(time[1]);
				endTime.setSeconds(time[2]);

				if(today >= startTime && today <= endTime)
					isValid =  true;

			}
		});

		// console.log('dateValidate nenhum filtro encontrado', item);
		return isValid;
	}
    Player.prototype.dateValidate = dateValidate; // torna a funação pública

    /**
     * Método para resetar o timeout de todos os tipos de conteúdo
     *
     * @method clearTimeOutPlayer
     * @return {void}
     */
    Player.prototype.clearTimeOutPlayer = function clearTimeOutPlayer() {
        this.appWindow.clearTimeout(this.appWindow.timeout);

        var listClear = [
            // imagem
            classImagePlayer,
            // html
            classHtmlPlayer,
            // rss
            classRssPlayer,
        ];
        var i;

        for (i in listClear) {
            if (typeof listClear[i] == 'object') {
                listClear[i].clearTimeout();
            }
        }

        stopMovie = false;
    }

	/**
	 * Maps the given argument (4yousee types) to internal type (ex.: i => video) or returns
	 * the map object
     *
     * @method getTypeMapping
	 * @param String 4yousee type (optional)
	 */
	Player.prototype.getTypeMapping = function() {
	 	if (arguments.length == 0) {
	 		return this.typeMapping;
	 	}
        // validate type
	 	var type = arguments[1];
        var ext = arguments[0];

        switch (type) {
            // carrossel
            case 'car':
                return this.typeMapping[type];
                break;

            // rss
            case 'n':
                return this.typeMapping['rss'];
                break;
        }

        // caso não seja as exceções
	 	if (this.typeMapping[ext]) {
	 		return this.typeMapping[ext];
	 	}
	};

	/***
	 * Play functions (private)
     *
     * @method playVideo
	 */
	Player.prototype.playVideo = function(mediaFile, container, time, endCallback) {
	 	var that = this;
	 	var path = this.projPath + '/' + FileSync.PLAYER_DIR_MEDIA + '/';
		// console.log("preparing to play ", mediaFile);
        var loadVideo = false;

        // lê arquivo
        FileSync.exists(path + mediaFile, generateVideoTag);

		function generateVideoTag(FileExist) {

            // em caso de erros
            if (!FileExist) {
				var downloadItem = {file: mediaFile, type: FileSync.FILE_TYPE_MEDIA, url: null, notify: false, trying: 0, force: true};
				DownloadCenter.addDownload(downloadItem);

				var errorMsg = "Can't play video";
				_logError(errorMsg, mediaFile);

                // aguarda 2 segundos
                setTimeout(function() {
                    endCallback(errorMsg, mediaFile);
                }, 2000);

                return;
			};

            /**
             * Método para remover os eventos
             * @return {void}
             */
            function removeEventVideo() {
                // ElementVideo.removeEventListener('ended', eventEnd, false);
                ElementVideo.removeEventListener('error', eventError, false);
                ElementVideo.removeEventListener('loadstart', loadedmetadata, false);
                ElementVideo.removeEventListener('pause', eventEnd, false);
            }

            /**
             * Evento que é executa quando o vídeo acaba
             * @param  {integer} typeSamsung Valor misterioso da Samsung, ainda não sei o link onde está os valores
             * @return {void}
             */
            function eventEnd(typeSamsung) {
                _log("---> FIM VIDEO: ", mediaFile);
                // console.debug("video end", mediaFile);

                // remove os eventos
                removeEventVideo();


                // callback
                endCallback(null, mediaFile);

                // if (stopMovie) {
                //    // evento da finalização do vídeo
                //    EventPlayerAudio.trigger('playlistAudioPlayer', that.appWindow);
                // }
            }

            /**
             * Evento de erro ao executar o vídeo
             * @param  {Object} event
             * @return {void}
             */
            function eventError(event) {
                _logError('video error', {
                    'message': event.message,
                    'filename': event.filename,
                });
                // console.debug("video error", mediaFile);

                // remove os eventos
                removeEventVideo();

                // permite que o conteúdo seja executado
                window.controllPlayerContent = undefined;

                endCallback('video error on play', mediaFile);
            }

            /**
             * Evento para ser iniciado quando os dados do vídeo for carregado
             * @return {void}
             */
            function loadedmetadata () {
                // redimenciona o vídeo
                that.resizeVideo(container.querySelector('video'));

                // melhorar a transições de conteúdo
                setTimeout(function() {
                    // exibe o vídeo
                    that.showContainer(container, 'video');

                    _log('Exibindo o vídeo.');
                }, 1500);

                // flag
                loadVideo = true;

                // executa o vídeo
                // ElementVideo.setAttribute('autoplay', true);
                ElementVideo.play();

                // logs
                // console.debug("video loadstart", mediaFile);
            }

            // dados da área principal
            var postionElementMain = container.getBoundingClientRect();

            // elemento html do vídeo
            var ElementVideo = container.querySelector('video');

            // redimenciona o vídeo
            this.resizeVideo(ElementVideo);

            var fileMedia = "../content/" + path + mediaFile;
            // var fileMedia = window.rootDirEntry + path + mediaFile;
            // var fileMedia = "http://checklist.4yousee.com.br/common/videos/" + mediaFile;
            // var fileMedia = window.hostPlayerHttp + path + mediaFile;

            ElementVideo.setAttribute('src', fileMedia);

            /////////////
            // eventos //
            /////////////

            // remove todos os eventos
            removeEventVideo();

            // adicionando eventos
            // ElementVideo.addEventListener('ended', eventEnd, false);
            ElementVideo.addEventListener('error', eventError, false);

            // quando o vídeo estiver pronto para tocar
            ElementVideo.addEventListener('loadstart', loadedmetadata, false);
            ElementVideo.addEventListener('pause', eventEnd, false);

            // carregando vídeo
            ElementVideo.load();
            // ElementVideo.play();

            // medida de contenção caso o vídeo não seja exibido
            // setTimeout(function() {
            //     if (ElementVideo.currentTime == 0 || !loadVideo) {
            //         _logError('Medida de contenção, o evento load do vídeo não foi disparado.');

            //         // dispara o evento load
            //         ElementVideo.load();
            //     }
            // }, 1000);

            // GAMB
            // PARA EVITAR QUE O VIDEO FAÇA A PLAYLIST TER CONTEÚDOS TOCANDO DE FORMAR ENCAVALADA
            // TODO: REFATORA
            stopMovie = true;

            /**
             * Método para verificar se o playlist de áudio deve ser pausada
             * @return {void}
             */
            function pauseAudio () {
                // evento para quando o vídeo for iniciado
                // console.log("TAG_MAYCON loadedmetadata: hasAudio", hasAudio(container.children[1]));
                if (hasAudio(container.children[1])) {
                    EventPlayerAudio.trigger('playlistAudioPause', that.appWindow);
                }
            }
		}
	}


    var classImagePlayer;
    /***
     * Play image
     *
     * @method playImage
     */
    Player.prototype.playImage = function (mediaFile, container, time, endCallback) {
        var that = this;
        var path = window.hostPlayerHttp + this.projPath + '/' + FileSync.PLAYER_DIR_MEDIA + '/';
        var localPath = this.projPath + '/' + FileSync.PLAYER_DIR_MEDIA + '/';


        classImagePlayer = new ImagePlayer (this.appWindow, mediaFile, container, time);

        classImagePlayer.play(path, localPath, endCallback);
    }

    var classHtmlPlayer;
    /***
	 * Play functions (private)
     *
     * @method playHtml
	 */
	Player.prototype.playHtml = function (mediaFile, container, time, endCallback) {
	 	var that = this;
	 	var path = window.hostPlayerHttp + this.projPath + '/' + FileSync.PLAYER_DIR_MEDIA + '/';
		var localPath = this.projPath + '/' + FileSync.PLAYER_DIR_MEDIA + '/';


        classHtmlPlayer = new HtmlPlayer (this.appWindow, mediaFile, container, time);

        classHtmlPlayer.play(path, localPath, endCallback);
	}

    var classRssPlayer;

    /**
     * Método para executar coteúdo rss
     *
     * @method playRss
     * @param  {ElementHtml} container   [description]
     * @param  {integer} time        [description]
     * @param  {function} endCallback [description]
     * @param  {object} item        [description]
     * @return {void}
     */
	Player.prototype.playRss = function(container, time, endCallback, item) {
        var that = this;
        var path = this.projPath + '/json/';
        var mediaFile = 'rss.player.' + that.playerId + '.json'

        // seta o path na classe Rss
        classRss.setProjPath(this.projPath);

        // console
        _log('--> RSS: ', mediaFile);

        // classe para player do rss
        // TODO: ISSO DEVE SER MELHORADO
        classRssPlayer = new RssPlayer(window);
        classRssPlayer = new classRssPlayer(this.appWindow, classRss, container, time, path);

        // seta o id do player
        classRssPlayer.setIdPlayer(that.playerId);

        // seta o arquivo de playlist
        classRssPlayer.setPlayList(mediaFile);

        // gamb para da tempo do chrome ler o arquivo
        setTimeout(function() {
            // player rss
            classRssPlayer.play(endCallback);
        }, 500);
	}

    /**
     * Método para executar o carrossel
     *
     * @method playCarrossel
     * @param  {object} mediaFile   Objeto com item a ser tocado
     * @param  {ElementHtml} container   Objeto html do conteudo
     * @param  {integer} time        Time out
     * @param  {function} endCallback
     * @return {void}
     */
    Player.prototype.playCarrossel = function(mediaFile, container, time, endCallback) {
        var that = this;
        var path = this.projPath + '/json/';
        var nameFile = 'categoria.' + mediaFile + '.json'

        // busca o arquivo json do carrossel
        FileSync.readJSONFile(window.rootDirEntry, path + nameFile, generateCarrosselTag);

        /**
         * Método para tocar o conteudo
         * @param  {json} rssJson Conteúdo da lista
         * @param  {object} error
         * @return {void}
         */
        function generateCarrosselTag(rssJson, error) {

            // verifica se existe erro
            if (error || !rssJson) {
                var errorMsg = "Can't play carrossel";
                _logError('Carrossel -->', errorMsg, error);
                that.clearTimeOutPlayer();

                // callback
                setTimeout(function() {
                    endCallback(errorMsg, mediaFile);
                }, 5000);

                return ;
            }

            // total de itens do carrossel
            var countItens = rssJson.itens.length;
            var cntCarrossel = 1;

            // seta o código do carrossel
            classCarrossel.setCodeCarrossel(mediaFile);


            // envia a lista de itens para classe
            classCarrossel.setItens(rssJson.itens);


            // busca o proxímo item a ser tocado
            classCarrossel.getNext(function(item) {
                playerCarrossel(item, endCallback);
            }, null);

            /**
             * Método para tocar o carrossel
             * @param  {JSON} item
             * @return {void}
             */
            function playerCarrossel(item, callbackPlayerCarrossel){

                // valida o conteudo
                if(typeof item === 'string' ){
                    // reseta conteúdos
                    that.clearTimeOutPlayer();

                    _logInfo('CARROSSEL :', item, error);
                    return callbackPlayerCarrossel(item, mediaFile);
                }

                if (dateValidate(item)) {

                    var fileExt = item.arquivo.split('.').pop();
                    var type = that.getTypeMapping(fileExt, item.tipo);

                    // play conteúdo
                    that.play(type, item, callbackPlayerCarrossel);

                } else {
                    var msgErro = '--> CARROSSEL: Conteúdo inválido.';
                    _logInfo(msgErro, item.arquivo);

                    // caso o item seja inválido
                    if (cntCarrossel === countItens) {
                        // callback
                        return callbackPlayerCarrossel(item, null);
                    }

                    cntCarrossel++;

                    // próximo item
                    classCarrossel.getNext(function(item) {
                        playerCarrossel(item, callbackPlayerCarrossel);
                    }, null);

                }
            }

        }
    }

    /**
     * Método para verificar se o arquivo no carrossel é válido
     *
     * @method validedFormatMediaCarrossel
     * @param  {JSON} item
     * @return {boolean}
     */
    function validedFormatMediaCarrossel(item) {
        var ext = item.arquivo.split(".").pop();

        if (ext != 'mp4') {
            return false;
        }

        return true;
    }


	Player.prototype.resizeVideo = function(element) {
		// call method in main.js
		resizeVideo(element);
	}

    /**
     * Método para redimencionar o conteudo
     *
     * @method resizeContainer
     * @param  {Element} element   Element a ser redimencionado
     * @param  {Element} container Elemento do container
     * @return {void}
     */
	Player.prototype.resizeContainer = function(element, container) {
		element.style.width = jQuery(container).width() + 'px';
		element.style.height = jQuery(container).height() + 'px';
	}

    /**
     * Método para retornar a container em uso
     *
     * @method getContainer
     * @return {Element}
     */
    Player.prototype.getContainer = function () {
        return this.appWindow.ClassLayout.getAreaMain();
    }

    /**
     * Método para resetar o conteudo do container
     *
     * @method resetContent
     * @param  {Element} container
     * @return {void}
     */
    Player.prototype.resetContent = function(container) {

        // verifica se foi enviado um objeto válido
        if (_.isUndefined(container)) {
            return _logError('O container enviado não é válido.');
        }

        var i;

        if (_.isArray(container)) {
            for (i in container) {
                resetItens(container);
            }

            return;
        }

        resetItens(container);

        /**
         * Método para resetar o conteúdo
         *
         * @method resetItens
         * @param  {Element} container
         * @return {void}
         */
        function resetItens(container) {
            container.querySelector('video').setAttribute('src', '');
            container.querySelector('iframe').setAttribute('src', ' ');
        }
    }

    /**
     * Método para resetar a playlist do player
     *
     * @method resetPlayList
     * @return {void}
     */
    Player.prototype.resetPlayList = function __doneResetPlayList() {

        // reseta o carrossel
        classCarrossel.resetPlaylistCurrent();

        // reseta a playlist principal
        this.appWindow.playlistManager.resetPlaylistCurrent();
    }

	window.Player = Player;
})();
