/* globals Rss, Carrossel, jQuery, _, _log, _logError, sync, playlistVideo, FileSync, ImagePlayer */

if (typeof Player !== "undefined") {
    // console.log(Player);
    throw new Error('player API already defined.');
}

var classCarrossel;
var classHtmlPlayer;

(function() {
    'use strict';
	// project name
	var classRss, stopMovie;
    var thisRoot;

	/**
	 * Class constructor
     *
     * @class Player
     * @constructor
	 * @param String proj Project name
	 * @param String projPath Project path
	 * @param Window window The window where the player must be injected
	 */
	function Player (projPath, appWindow, env4YouSee, playerId) {
        // root
        thisRoot = this;

		// mapping of 4yousee types to internal types
		this.typeMapping = {
            "mp4": "video",
            "zip": "html",
            "html": "html",
            "rss": "rss",
            "car": "carrossel",
            "gif": "imagem",
            "jpg": "imagem",
            "jpeg": "imagem",
			"png": "imagem",
		};

		// loads the container
        // this.app = appWindow;
		this.appWindow = appWindow;
		this.loading   = this.appWindow.document.getElementById('loading');
		this.errorMsg  = this.appWindow.document.getElementById('errorMsg');
		this.projPath = projPath;
		this.env4YouSee = env4YouSee;
		this.playerId = playerId;
        classRss = new Rss(projPath, true);
		classCarrossel = new Carrossel(projPath);

        this.connected = false;

        /**
         * Controla se o player está executando algum conteúdo
         * @type {Boolean}
         */
		this.playerActive = false;
	}

    /**
     * Método para setar o id do player
     *
     * @method setPlayerId
     * @param {integer} playerId
     */
    Player.prototype.setPlayerId = function __doneSetPlayerId(playerId) {
        this.playerId = playerId;
    };

    /**
     * Método para exibir o conteúdo
     *
     * @method showContainer
     * @param  {Element} container         Elemento html
     * @param  {String} placeholderToShow Qual conteúdo será exibido
     * @return {void}
     */
	function showContainer (container, placeholderToShow, areaAux){
		var placeholders = ['video', 'iframe'];

        if (!areaAux) {
            thisRoot.loading.innerHTML = '';
            thisRoot.loading.style.display = 'none';
        }

        // exibe o conteúdo
        if(container.querySelector(placeholderToShow)) {
            container.querySelector(placeholderToShow).style.opacity = 1;
        }

        // suavisar a transação
        setTimeout(function() {
            for (var i = placeholders.length - 1; i >= 0; i--) {

                // caso não seja o item a ser exibido
                if(placeholderToShow != placeholders[i] && container.querySelector(placeholders[i])) {
                    container.querySelector(placeholders[i]).style.opacity = 0;

                    // caso seja conteúdo html
                    if (placeholders[i] === 'iframe') {
                        jQuery(container.querySelector(placeholders[i])).trigger('resetIframe');
                    }
                }
            }
        }, 500);
	};

    Player.prototype.showContainer = showContainer;


	Player.prototype.play = function (type, item, endCallback, notNextVideo) {
        var that = this;
		var mediaFile = item.arquivo;
		var time = item.tempo;

        this.clearTimeOutPlayer();

        // caso o layout ainda não tenha sido iniciado;
        if (_.isUndefined(this.getContainer())) {
            // caso seja erro de layout o arquivo deve ser substituído
            _logError('Layout ainda não carregado.');

            // todos os arquivos precisam ser verificados
            sync.init(true, function _finishCheckProject() {
                _log('Função para tentar recuperar executada.');
            });

            return setTimeout(function() { that.play(type, item, endCallback); }, 4000);
            // return setTimeout(function() { endCallback(true, 'layout'); }, 4000);
        }


        /**
         * Função de callback, para preecher a tag vídeo com a playlist
         *
         * TODO: Remover isso
         * @param  {string} msgError
         * @param  {object} mediaFile
         */
        function __endCallback(msgError, mediaFile) {
            EventPlayer.trigger('updatePlaylistVideo', [
                undefined, //element html
                function __callbackHtmlPlayer() { // callback
                    // callback
                    // O timeout é para que dê tempo da tv
                    // realizar o load do vídeo
                    endCallback(msgError, mediaFile);
                }
            ]);
        }

        this.playerActive = true;

        // logs
        console.log('%c Será tocado um novo conteúdo!', 'background: #222; color: #bada55', JSON.stringify(item));


		if(!dateValidate(item)){

            return setTimeout(function() {
                var msgError = 'Invalid scheduling';
                // caso o conteúdo seja inválido não pode trocar a playlist
                // Vídeo deve tocar o proxímo vídeo
                if (['video'].indexOf(type) === -1) {
                    // impede que a playlist seja alterada
                    playlistVideo.authChangePlayList = false;

                    __endCallback(msgError, null);
                } else {
                    endCallback(msgError, null);
                }
            }, 200);
        }

        switch (type) {
            case 'video':
				this.playVideo(item, this.getContainer(), time, endCallback);
                break;
            case 'html':
				this.playHtml(mediaFile, this.getContainer(), time, __endCallback, item);
                break;
            case 'rss':
			     this.playRss(this.getContainer(), time, __endCallback, item);
                break;
            case 'carrossel':
                // solução parcial
                // Faz que caso o conteúdo do carrossel seja html
                // o player faça a troca da playlist de vídeo 2vezes
                playlistVideo.authChangePlayList = true;

                this.playCarrossel(item, this.getContainer(), time, endCallback, __endCallback);
                break;
            case 'imagem':
                this.playImage(mediaFile, this.getContainer(), time, __endCallback);
                break;
            default:
    			var error = "4yousee can't recognize ";
    			_logError(error, type, " as a valid media type");
    			__endCallback(error, null);
        }
	};

    /**
     * Método para verificar se o conteúdo a ser tocado é válido
     *
     * @method dateValidate
     * @param  {Object} item
     * @return {boolean}
     */
	function dateValidate (item) {
		var today = new Date();
		var regex = new RegExp('-', 'g');

		var startDate = (!item.startDate) ? undefined : new Date(item.startDate.replace(regex, '/'));

		var endDate;
		if(item.endDate){
			endDate = new Date(item.endDate.replace(regex, '/'));
			endDate.setHours(23);
			endDate.setMinutes(59);
			endDate.setSeconds(59);
		}

		if(startDate !== undefined && today < startDate)
			return false;

		if(endDate !== undefined && today > endDate)
			return false;

		var scheduleList = item.schedule;

		if(!scheduleList || scheduleList.length === 0)
			return true;

		var isValid = false;
		scheduleList.forEach(function(schedule){

			if(!isValid && schedule.weekDays.indexOf(today.getDay()) != -1){
				var time = schedule.startTime.split(':');

				var startTime = new Date();
				startTime.setHours(time[0]);
				startTime.setMinutes(time[1]);
				startTime.setSeconds(time[2]);

				time = schedule.endTime.split(':');
				var endTime = new Date();
				endTime.setHours(time[0]);
				endTime.setMinutes(time[1]);
				endTime.setSeconds(time[2]);

				if(today >= startTime && today <= endTime)
					isValid =  true;

			}
		});

		// console.log('dateValidate nenhum filtro encontrado', item);
		return isValid;
	}
    Player.prototype.dateValidate = dateValidate; // torna a funação pública

    /**
     * Método para resetar o timeout de todos os tipos de conteúdo
     *
     * @method clearTimeOutPlayer
     * @return {void}
     */
    Player.prototype.clearTimeOutPlayer = function clearTimeOutPlayer() {
        this.appWindow.clearTimeout(this.appWindow.timeout);

        var listClear = [
            // html
            classHtmlPlayer,
            // image
            classImagePlayer,
            // rss
            classRssPlayer,
        ];
        var i;

        for (i in listClear) {
            if (listClear[i]) {
                listClear[i].clearTimeout();
            }
        }

        stopMovie = false;
    };

	/**
	 * Maps the given argument (4yousee types) to internal type (ex.: i => video) or returns
	 * the map object
     *
     * @method getTypeMapping
	 * @param String 4yousee type (optional)
	 */
	Player.prototype.getTypeMapping = function() {
	 	if (arguments.length === 0) {
	 		return this.typeMapping;
	 	}
        // validate type
	 	var type = arguments[1];
        var ext = arguments[0];

        switch (type) {
            // carrossel
            case 'car':
                return this.typeMapping[type];

            // rss
            case 'n':
                return this.typeMapping.rss;
        }

        // caso não seja as exceções
	 	if (this.typeMapping[ext]) {
	 		return this.typeMapping[ext];
	 	}
	};

	/***
	 * Play functions (private)
     *
     * @method playVideo
	 */
	Player.prototype.playVideo = function(mediaFile, container, time, endCallback) {
	 	// var that = this;
	 	// var path = this.projPath + '/' + FileSync.PLAYER_DIR_MEDIA + '/';
		// console.log("preparing to play ", mediaFile);
        // var loadVideo = false;

        // elemento html do vídeo
        var ElementVideo = container.querySelector('video');

        // variável playlistVideo definida no arquivo background.js

        // seta a tag html a ser utilizada pelo vídeo
        playlistVideo.setElementPlayer(ElementVideo);

        // seta o container a ser utilizado
        playlistVideo.setElementContainer(container);

        //set o objeto do vídeo a ser tocado
        playlistVideo.setObjectItem(mediaFile);

        //seta callback para quando o conteúdo for concluído
        playlistVideo.setCallback(endCallback);

        // executa o conteúdo
        playlistVideo.execFn();
	};

    var classImagePlayer;
    /***
     * Play image
     *
     * @method playImage
     */
    Player.prototype.playImage = function (mediaFile, container, time, endCallback)
    {
        var path = window.hostPlayerHttp + this.projPath + '/' + FileSync.PLAYER_DIR_MEDIA + '/';
        var localPath = this.projPath + '/' + FileSync.PLAYER_DIR_MEDIA + '/';

        // instance class
        classImagePlayer = new ImagePlayer (this.appWindow, mediaFile, container, time);

        // show content
        classImagePlayer.play(path, localPath, endCallback);
    };


    /***
	 * Play functions (private)
     *
     * @method playHtml
	 */
	Player.prototype.playHtml = function (mediaFile, container, time, endCallback)
    {
	 	var that = this;
	 	var path = window.hostPlayerHttp + this.projPath + '/' + FileSync.PLAYER_DIR_MEDIA + '/';
		var localPath = this.projPath + '/' + FileSync.PLAYER_DIR_MEDIA + '/';

        // instance class
        classHtmlPlayer = new HtmlPlayer (this.appWindow, mediaFile, container, time);

        // set method callback
        classHtmlPlayer.setCallback(endCallback);

        // exec content
        classHtmlPlayer.play(path, localPath);
	};

    var classRssPlayer;

    /**
     * Método para executar coteúdo rss
     *
     * @method playRss
     * @param  {ElementHtml} container   [description]
     * @param  {integer} time        [description]
     * @param  {function} endCallback [description]
     * @param  {object} item        [description]
     * @return {void}
     */
	Player.prototype.playRss = function(container, time, endCallback, item) {
        var that = this;
        var path = this.projPath + '/json/';
        var mediaFile = 'rss.player.' + that.playerId + '.json'

        // seta o path na classe Rss
        classRss.setProjPath(this.projPath);

        // console
        _log('--> RSS: ', mediaFile);

        // classe para player do rss
        // TODO: ISSO DEVE SER MELHORADO
        classRssPlayer = new RssPlayer(window);
        classRssPlayer = new classRssPlayer(this.appWindow, classRss, container, time, path);

        // seta o id do player
        classRssPlayer.setIdPlayer(that.playerId);

        // seta o arquivo de playlist
        classRssPlayer.setPlayList(mediaFile);

        // gamb para da tempo do chrome ler o arquivo
        setTimeout(function() {
            // player rss
            classRssPlayer.play(endCallback);
        }, 500);
	};

    /**
     * Método para executar o carrossel
     *
     * @method playCarrossel
     * @param  {object} mediaFile   Objeto com item a ser tocado
     * @param  {ElementHtml} container   Objeto html do conteudo
     * @param  {integer} time        Time out
     * @param  {function} endCallback
     * @param  {function} endCallbackContentInvalid   Foi necessário isso ser adicionado assim, porque o carrossel tem uma validação propria em outra classe
     * @return {void}
     */
    Player.prototype.playCarrossel = function(item, container, time, endCallback, endCallbackContentInvalid) {
        var that = this;
        var mediaFile = item.arquivo;
        var path = this.projPath + '/json/';
        var nameFile = 'categoria.' + mediaFile + '.json'

        // busca o arquivo json do carrossel
        FileSync.readJSONFile(window.rootDirEntry, path + nameFile, generateCarrosselTag);

        /**
         * Método para tocar o conteudo
         * @param  {json} rssJson Conteúdo da lista
         * @param  {object} error
         * @return {void}
         */
        function generateCarrosselTag(rssJson, error) {

            // verifica se existe erro
            if (error || !rssJson) {
                var errorMsg = "Can't play carrossel";
                _logError('Carrossel -->', errorMsg, error);
                that.clearTimeOutPlayer();

                // callback
                setTimeout(function() {
                    endCallbackContentInvalid(errorMsg, mediaFile);
                }, 5000);

                return ;
            }

            // total de itens do carrossel
            var countItens = rssJson.itens.length;
            var cntCarrossel = 1;

            // seta o código do carrossel
            classCarrossel.setCodeCarrossel(mediaFile);

            // envia a lista de itens para classe
            classCarrossel.setItens(rssJson.itens);

            // seta as validades de conteúdos, com programação
            // dentro da playlist
            classCarrossel.setValidadeContent(item.playlistSchedule);

            // busca o proxímo item a ser tocado
            classCarrossel.getNext(function(item) {
                playerCarrossel(item, endCallback, endCallbackContentInvalid);
            }, null);

            /**
             * Método para tocar o carrossel
             * @param  {JSON} item
             * @return {void}
             */
            function playerCarrossel(item, callbackPlayerCarrossel, endCallbackContentInvalid){

                // valida o conteudo
                if(typeof item === 'string' ){
                    // reseta conteúdos
                    that.clearTimeOutPlayer();

                    _logError('CARROSSEL :', item, error);
                    return endCallbackContentInvalid(item, mediaFile);
                }

                if (dateValidate(item)) {

                    var fileExt = item.arquivo.split('.').pop();
                    var type = that.getTypeMapping(fileExt, item.tipo);

                    // play conteúdo
                    that.play(type, item, callbackPlayerCarrossel, true);

                } else {
                    var msgErro = '--> CARROSSEL: Conteúdo inválido.';
                    _logError(msgErro, item.arquivo);

                    // caso o item seja inválido
                    if (cntCarrossel === countItens) {
                        // callback
                        return endCallbackContentInvalid(item, null);
                    }

                    cntCarrossel++;

                    // próximo item
                    classCarrossel.getNext(function(item) {
                        playerCarrossel(item, callbackPlayerCarrossel, endCallbackContentInvalid);
                    }, null);

                }
            }

        }
    };

    /**
     * Método para verificar se o arquivo no carrossel é válido
     *
     * @method validedFormatMediaCarrossel
     * @param  {JSON} item
     * @return {boolean}
     */
    function validedFormatMediaCarrossel(item) {
        var ext = item.arquivo.split(".").pop();

        if (ext != 'mp4') {
            return false;
        }

        return true;
    }


	Player.prototype.resizeVideo = function(element) {
		// call method in main.js
		resizeVideo(element);
	};

    /**
     * Método para redimencionar o conteudo
     *
     * @method resizeContainer
     * @param  {Element} element   Element a ser redimencionado
     * @param  {Element} container Elemento do container
     * @return {void}
     */
	Player.prototype.resizeContainer = function(element, container) {
		element.style.width = jQuery(container).width() + 'px';
		element.style.height = jQuery(container).height() + 'px';
	};

    /**
     * Método para retornar a container em uso
     *
     * @method getContainer
     * @return {Element}
     */
    Player.prototype.getContainer = function () {
        return this.appWindow.ClassLayout.getAreaMain();
    };

    /**
     * Método para resetar o conteudo do container
     *
     * @method resetContent
     * @param  {Element} container
     * @return {void}
     */
    Player.prototype.resetContent = function(container) {

        // verifica se foi enviado um objeto válido
        if (_.isUndefined(container)) {
            return _logError('O container enviado não é válido.');
        }

        var i;

        if (_.isArray(container)) {
            for (i in container) {
                resetItens(container);
            }

            return;
        }

        resetItens(container);

        /**
         * Método para resetar o conteúdo
         *
         * @method resetItens
         * @param  {Element} container
         * @return {void}
         */
        function resetItens(container) {
            container.querySelector('video').setAttribute('src', '');
            container.querySelector('iframe').setAttribute('src', ' ');
        }
    };

    /**
     * Método para resetar a playlist do player
     *
     * @method resetPlayList
     * @return {void}
     */
    Player.prototype.resetPlayList = function __doneResetPlayList() {

        // reseta a playlist principal
        this.appWindow.playlistManager.resetPlaylistCurrent();
    };

	window.Player = Player;
})();
