'use strict';

(function (window) {

    /**
     * Classe para controlar a exibição dos conteúdos imagem
     *
     * @class ImagePlayer
     * @constructor
     * @param {AppWindow} appWindow Objeto appWindow do chrome
     * @param {JSON} mediaFile Json do conteúdo
     * @param {Element} container Elemento html
     * @param {integer} time      Tempo de espera do conteúdo
     */
    function ImagePlayer (appWindow, mediaFile, container, time) {
        this.appWindow = appWindow;
        this.mediaFile = mediaFile;
        this.container = container;

        // para de tocar o conteúdo
        this.stopContent = false;

        this.time = time;
    };

    /**
     * Método para o player do conteúdo
     *
     * @method play
     * @param  {string} path        Caminho padrão
     * @param  {string} localPath   Caminho do arquivo
     * @param  {function} endCallback Função de callback
     * @return {void}
     */
    ImagePlayer.prototype.play = function (path, localPath, endCallback) {
        var that = this;

        var mediaPath = localPath + that.mediaFile;
        var htmlUrl = path + that.mediaFile;

        _log("TAG_MAYCON ImagePlayer play: mediaPath", mediaPath);

        // lê o arquivo
        FileSync.exists(mediaPath, function(error){

            // em caso de errors
            if(!error){

                // baixa os arquivos novamente
                var downloadItem = {file: that.mediaFile, type: FileSync.FILE_TYPE_MEDIA, url: null, notify: false, trying: 0};
                DownloadCenter.addDownload(downloadItem);

                // console
                var errorMsg = "Can't play image";
                _logError('--> IMAGE:', errorMsg, that.mediaFile, error);

                // reseta o timeout
                that.clearTimeout();

                // callback
                setTimeout(function() {
                    endCallback(errorMsg, that.mediaFile);
                }, 5000);

                return;
            }

            // monta o conteúdo html na tela
            generateHtmlTag(htmlUrl, endCallback);
        });

        /**
         * Método para executar o html
         *
         * @method generateHtmlTag
         * @param  {string} htmlUrl     Url
         * @param  {function} endCallback Função de callback
         * @return {void}
         */
        function generateHtmlTag (htmlUrl, endCallback) {

            // element
            var webviewElement = that.container.querySelector('iframe');

            // configura os parâmetro da webview
            var stringHtmlImage = '<html> <head> <style> html {background: url("' + htmlUrl + '") no-repeat center center fixed; -webkit-background-size: cover; -moz-background-size: cover; -o-background-size: cover; background-size: cover; } </style> </head> <body> </body> </html>';
            htmlUrl = 'data:text/html;charset=utf-8,' + escapeHtml(stringHtmlImage);

            webviewElement.setAttribute('src', htmlUrl);
            webviewElement.style.width = window.screen.availWidth + "px";
            webviewElement.style.height = window.screen.availHeight + "px";

            // escreve o conteúdo no iframe
            var doc = webviewElement.contentDocument;
            doc.open();
            doc.writeln(stringHtmlImage);
            doc.close();

            Player.prototype.showContainer(that.container, 'iframe');
            Player.prototype.resizeContainer(webviewElement, that.container);

            // reseta o timeout
            that.clearTimeout();

            // time to out
            if(!that.time || that.time == 0)
                that.time = 15000;
            else
                that.time = that.time*1000;

            // toca o conteúdo
            that.stopContent = false;

            // registra a função
            that.timeOut = setTimeout(function(){
                _log('--> FIM IMAGE:', htmlUrl);

                // callback
                if (!that.stopContent) {
                    endCallback(null, that.mediaFile);
                }
            }, that.time);
        }
    }

    /**
     * Método para limpar o timeout
     *
     * @method clearTimeout
     * @return {void}
     */
    ImagePlayer.prototype.clearTimeout = function () {
        clearTimeout(this.timeOut);

        // para de tocar
        this.stopContent = true;
    }

    window.ImagePlayer = ImagePlayer;
}) (window);
