'use strict';

var RssPlayer = (function (window) {

    /**
     * Classe para executar o conteudo em Rss
     *
     * @class RssPlayer
     * @constructor
     * @param {Element} appWindow
     * @param {classRss} classRss
     * @param {Element} container Div da área
     * @param {float} time      Timeout
     */
    function RssPlayer (appWindow, classRss, container, time, path) {
        this.appWindow = appWindow;
        this.classRss = classRss;
        this.path = path;
        this.timeOut;
        this.time = time;
        this.appWindow.RssPlayerTimeoutRssPlayer = undefined;

        /**
         * Variável para controlar os timeouts
         *
         * @property {array} Timeout Array com os timeouts que estão sendo executados por área
         * @type {Array}
         */
        this.Timeout = [];

        // set container
        this.setContainer(container);

        // time to out
        if(!this.time || this.time == 0)
            this.time = 15000;
        else
            this.time *= 1000;
    };

    /**
     * Método para setar a janela
     *
     * @method setAppWindow
     * @param {window} appWindow
     */
    RssPlayer.prototype.setAppWindow = function (appWindow) {
        this.appWindow = appWindow;
    }

    /**
     * Método para setar o novo container
     *
     * @method setContainer
     * @param {Object} container Container
     */
    RssPlayer.prototype.setContainer = function (container) {
        this.container = container;
    }

    /**
     * Método para setar o id do player
     * @param  {integer} idPlayer
     * @return {void}
     */
    RssPlayer.prototype.setIdPlayer = function (idPlayer) {
        this.idPlayer = idPlayer;
    }

    /**
     * Método para setar a playlist do conteúdo a ser tocado
     *
     * @method setPlayList
     * @param {string} filePlayList nome do arquivo a ser incluído
     */
    RssPlayer.prototype.setPlayList = function (filePlayList) {
        var that = this;

        // limpa a playlist a ser utilizada
        that.classRss.setItens([]);

        // caso não seja válido
        if (!_.isString(filePlayList)) {
            return _logError('Playlist não é válida.');
        }

        /**
         * Busca o conteudo
         * @return {void}
         */
        // verifica a extenção
        var ext = filePlayList.split('.').pop();

        // _log({
        //     'Class': 'Player',
        //     'method': 'setPlayList',
        //     'ext': ext,
        //     'that.path + filePlayList': that.path + filePlayList
        // });

        // xml
        if (ext === 'xml') {
            FileSync.readXmlFile(appDir, that.path + filePlayList, gerateRssTagXml);
        }
        // json
        if (ext === 'json') {
            FileSync.readJSONFile(appDir, that.path + filePlayList, generateRssTag);
        }

        /**
         * Método traduzir o conteudo de xml para json
         *
         * @method gerateRssTagXml
         * @param  {xml} xmlRss
         * @param  {string} error
         * @return {void}
         */
        function gerateRssTagXml (objectJson, error) {
            var i, rssItens, item;

            // _log({
            //     'Class': 'Player',
            //     'method': 'playRss',
            //     'objectJson': objectJson
            // });

            // validando formato
            if (_.isUndefined(objectJson) || _.isNull(objectJson)) {

                // flag para que o layout não seja reiniciado
                erroFileNewsLayout = false;

                // processa o arquivo de template
                // Somente templates do layout utilizam arquivo xml
                EventLayout.trigger('processFileLayout', that.appWindow.contentWindow.playerData.id);

                return _logError('Json inválido', objectJson);
            }

            // caso não tenha notícia
            if (!('root' in objectJson) || !('item' in objectJson.root)) {
                return _logError('Json vazio', objectJson);
            }

            // verifica o objeto que chegou
            if (!_.isUndefined(objectJson) && ('root' in objectJson) && ('item' in objectJson.root)) {

                // caso exista mais de 1 item
                // Isso só existe pra atender uma gamb do manager
                rssItens = objectJson.root.item;
                if (('__cdata' in objectJson.root.item)) {
                    rssItens = [objectJson.root.item];
                }
            } else {
                // caso o objeto do item seja inválido
                // download
                sync.downloadNewsXmlLayout(filePlayList, function () {});
                // console
                _logError('Json inválido', objectJson);
            }

            // Tratando os dados
            if (!_.isUndefined(rssItens)) {
                rssItens = generateObjectRssItem(rssItens, function doneGenerateObjectRssItem(fileContentJson) {
                    // carrega o conteudo
                    generateRssTag(fileContentJson, error);
                });
            }
        }

        /**
         * Método para gerar objeto de notícia
         * @param  {array} itens Lista de itens
         * @return {array}
         */
        function generateObjectRssItem(rssItens, callback) {
            // traduz o conteudo para o formato esperado
            var fileContentJson = {
                itens: []
            };

            if (!_.isArray(rssItens)) {
                return callback([]);
            }

            async.each(rssItens, function(item, next){
                // adiciona novo elemento
                fileContentJson.itens.push(
                    {
                        "fonte": item._fonte,
                        "template": item._template,
                        "imagem": item._imagem,
                        "url": item._url,
                        "texto": item.__cdata,
                        "startDate": "",
                        "endDate": "",
                    }
                );

                // callback eachSeries
                return next();
            }, function doneEachSeries() {
                callback(fileContentJson);
            });
        }

        /**
         * Carrega o conteudo na área
         *
         * @method generateRssTag
         * @param  {json} rssJson
         * @param  {string} error
         * @return {void}
         */
        function generateRssTag(rssJson, error) {

            // em caso de error
            if (error || !rssJson) {

                var errorMsg = "Can't play rss";
                _logError(errorMsg, error);

                return;
            }

            // passa tudo para a variável playlist
            that.playlist = rssJson;

            /**
             * Seta o itens
             */
            that.classRss.setItens(rssJson.itens);
        }
    }

    /**
     * Método para setar o template do template
     *
     * @method setTemplate
     * @param {integer} template Fonte do template
     */
    RssPlayer.prototype.setTemplate = function (template) {
        this.IdTemplate = template;
        this.classRss.setTemplateLayout(template);
    }

    /**
     * Método para dar player no conteudo
     *
     * @method play
     * @param {function} endCallback Método de retorno para ser executado quando o conteúdo acabar
     * @param {integer} idLayout Id do layout que está sendo utilizado
     * @return {void}
     */
    RssPlayer.prototype.play = function (endCallback, idLayout) {
        var that = this;

        /**
         * Exibe o proximo item
         * @param  {json} item
         * @return {void}
         */
        that.classRss.getNext(execGetNext, null);

        /**
         * Método para tratar o proxímo item a ser tocado
         *
         * @method execGetNext
         * @param  {Object} item
         * @return {void}
         */
        function execGetNext (item){
            // reseta o timeout
            that.clearTimeout();

            // em caso de erro
            if(typeof item === 'string' ){
                _logError(item);

                // pula a notícia
                // Em conversa no dia 08/01/2016 com Herculano,
                // decidimos bloquear, porque não faz sentido exibir o template
                // e não tem notícia.
                setTimeout(function() {
                    endCallback(item);
                }, 5000);

                return;
            }

            // verifica url
            var htmlUrl = that.classRss.getUrl(item);

            // em caso de error
            if(!htmlUrl) {
                that.clearTimeout();
                var errorMsg = "Can't play SWF rss";
                _logError(errorMsg, item);

                // callback
                setTimeout(function() {
                    endCallback(errorMsg);
                }, 5000);

                return;
            }

            // class que executa conteúdo html
            var classHtmlPlayer = new HtmlPlayer (that.appWindow, item, that.container);

            // exibe conteúdo
            classHtmlPlayer.generateHtmlTag(htmlUrl, 'n', item.fonte, endCallback)
        }
    };

    /**
     * Método para resetar o timeout
     *
     * @method clearTimeout
     * @return {void}
     */
    RssPlayer.prototype.clearTimeout = function() {
        clearTimeout(this.timeOut);
        clearInterval(this.appWindow.RssPlayerTimeoutRssPlayer);
    }

    return RssPlayer;
});
