'use strict';
/* globals playlistVideo */

(function (_global) {

    /**
     * Classe para gerenciar a fila de conteúdo de carrossel
     *
     * @constructor CarrosselVideoPlayer
     */
    function CarrosselVideoPlayer()
    {
        // lista de todos os arquivos do carrossel
        this.listCarrossel = [];

        this.currentFile = [];
    }

    /**
     * Método para retornar o próximo item a seja tocado no player
     *
     * @method getNext
     * @param  {integer} idCarrossel Id do carrossel a tocado
     * @return {Object}
     */
    CarrosselVideoPlayer.prototype.getNext = function __done (idCarrossel)
    {
        var keyCarrossel = 'cat_' + idCarrossel;
        var currentPosition = this.currentFile[keyCarrossel] || 0;
        var listFiles = this.getListFiles(keyCarrossel);

        // caso a posição seja maior a atual
        if (!listFiles || currentPosition >= Object.keys(listFiles).length) {
            currentPosition = 0;
        }

        // TODO: CASO O CARROSSEL ESTEJA VAZIO, O QUE FAZER? UM CARROSSEL PODE ESTÁ VAZIO?

        // incrementa a variável de controle do carrossel
        this.currentFile[keyCarrossel] = currentPosition + 1;

        // dependencia da classe de vídeo
        return playlistVideo.getNameFileCarrossel(idCarrossel, currentPosition);
    };

    /**
     * Método para setar a playlist de conteúdo
     *
     * @method setListFiles
     * @param {Integer} idCarrossel
     * @param {Object} listFiles
     */
    CarrosselVideoPlayer.prototype.setListFiles = function __done (idCarrossel, listFiles)
    {
        // caso seja informado um objeto no lugar do id
        // será substituido todo o objeto
        if (typeof idCarrossel === 'object') {

            this.listCarrossel = idCarrossel;
        } else {
            this.listCarrossel[idCarrossel] = listFiles;
        }
    };

    /**
     * Método para obter a lista de arquivos da playlist
     *
     * @method getListFiles
     * @param  {Integer} idCarrossel
     * @return {Object}
     */
    CarrosselVideoPlayer.prototype.getListFiles = function __done (idCarrossel)
    {
        return this.listCarrossel[idCarrossel];
    };

    /**
     * Método para remover um carrossel
     *
     * @method removeListFiles
     * @param  {integer} idCarrossel
     */
    CarrosselVideoPlayer.prototype.removeListFiles = function __done (idCarrossel)
    {
        delete this.listCarrossel[idCarrossel];
    };

    /**
     * Método para resetar a lista de carrosseis
     *
     * @method resetListFiles
     * @return {void}
     */
    CarrosselVideoPlayer.prototype.resetListFiles = function __done ()
    {
        this.listCarrossel = [];
    };

    /**
     * Método inicializar posição inicial dos carrosseis
     *
     * @method createCurrentFile
     * @return {void}
     */
    CarrosselVideoPlayer.prototype.createCurrentFile = function __done ()
    {
        for (var idCarrossel in this.listCarrossel) {
            this.currentFile[idCarrossel] = 0;
        }
    };

    /**
     * Método para resetar playlist de carrosseis
     * @method resetPlaylist
     */
    CarrosselVideoPlayer.prototype.resetPlaylist = function __done ()
    {
        this.currentFile = [];
    }

    _global.CarrosselVideoPlayer = new CarrosselVideoPlayer();
}) (window);
