/* globals _log, player4uc, InputConfigurationSocket, ControllBrightness, ControllPowerOff */
(function(_global) {
    'use strict';

    /**
     * Class for manager comunication with 4Yousee Manager
     * @param {ManagerConnect} managerConnect
     */
    function ConfigurationSocket(managerConnect)
    {
        this.managerConnect = managerConnect;
    }

    /**
     * Set class ManagerConnect
     * @param {ManagerConnect} managerConnect
     */
    ConfigurationSocket.prototype.setClassManangerConnect = function __donesetClassManangerConnect(managerConnect)
    {
        this.managerConnect = managerConnect;
    };

    /**
     * Event in which save data of configuration
     * @param  {Object} data
     */
    ConfigurationSocket.prototype.saveSetting = function __doneSaveSetting(data)
    {
        var i;
        // format content for default
        var listInputs = [];
        if (typeof data === 'object') {
            for (i in data) {
                listInputs.push ((new InputConfigurationSocket(data[i].value, data[i].label, data[i].type, data[i].disabled, data[i].key, data[i].options)).toJSON());
            }
        }


        // TODO: DAQUI PRA BAIXO, QUANDO TIVER MAIS TEMPO
        // É MELHOR QUE SEJA REFATORADO

        var urlAccount = '';
        var playerId = '';
        var brightnessValues = '';
        var powerOffHourValues = '';

        for (i in listInputs) {
            switch (listInputs[i].key) {
                case 'urlAccount':
                    urlAccount = listInputs[i].value;
                    break;
                case 'pref_playerId':
                    playerId = listInputs[i].value;
                    break;
                case ControllBrightness.keyConfiguration: // brightness
                    brightnessValues = listInputs[i].value;
                    break;
                case ControllPowerOff.keyConfiguration: // power/off
                    powerOffHourValues = listInputs[i].value;
                    break;
            }
        }

        ////////////////
        // brightness //
        ////////////////
        if (typeof ControllBrightness === 'object') {
            // reset configuration
            ControllBrightness.clearConfiguration();

            // set configuration
            // Morning
            ControllBrightness.configuration(brightnessValues);
        }


        //////////////////
        // Power or Off //
        //////////////////
        if (typeof ControllPowerOff === 'object') {
            // reset configuration
            ControllPowerOff.clearConfiguration();

            // power
            ControllPowerOff.configuration(powerOffHourValues);
        }


        // confirmation settings
        var result = {sucess: true, message: player4uc.i18n.getMessage("settings_save_success")};
        this.managerConnect.socket.emit('save_settings_result',result, function(){
            // exerc function for apply settings
            // brightness
            window.ControllBrightness.init();

            // power/Off
            window.ControllPowerOff.init();

            _log("Configuration finish.");
        });
    };

    /**
     * Event in which listen the socket
     * @param  {Object} data
     */
    ConfigurationSocket.prototype.listen = function __doneListen(data)
    {
        if (!this.managerConnect.socket) {
            _log("Socket not setting in ConfigurationSocket.");
            return;
        }

        // send
       this.sendConfiguration(data);
    };

    /**
     * Send object of configuration
     */
    ConfigurationSocket.prototype.sendConfiguration = function __doneSendConfiguration()
    {
        if (!this.managerConnect.socket) {
            return;
        }

        var that = this;
        var keys = [ControllBrightness.keyConfiguration, ControllPowerOff.keyConfiguration];
        var dataPlayer = player4uc.window.current().contentWindow.playerData;

        // get settings storage local
        player4uc.storage.get(keys, function _sendSocketConfiguration(object) {

            var objBrightness;

            try {
                objBrightness = object[ControllBrightness.keyConfiguration][dataPlayer.id];
            } catch (exception) {
                objBrightness = object[ControllBrightness.keyConfiguration] = {};
                objBrightness = object[ControllBrightness.keyConfiguration][dataPlayer.id] = {itens: []};
            }

            var objScheduleOperations;
            try {
                objScheduleOperations = object[ControllPowerOff.keyConfiguration][dataPlayer.id];
            } catch (exception) {
                objScheduleOperations = object[ControllPowerOff.keyConfiguration] = {};
                objScheduleOperations = object[ControllPowerOff.keyConfiguration][dataPlayer.id] = {itens: []};
            }

            var urlAccount = (new InputConfigurationSocket(dataPlayer.url, player4uc.i18n.getMessage("urlAccount"), "TEXT", true, "urlAccount", [])).toJSON();
            var playerId = (new InputConfigurationSocket(dataPlayer.id, player4uc.i18n.getMessage("playerId"), "TEXT", true, "pref_playerId", [])).toJSON();
            var playerVersion = (new InputConfigurationSocket(window.versionApp, player4uc.i18n.getMessage("version_player"), "TEXT", true, "pref_playerVersion", [])).toJSON();

            var brightnessOptions = (new InputConfigurationSocket(objBrightness.itens, player4uc.i18n.getMessage("brightnessControll"), "BRIGHTNESS", false, ControllBrightness.keyConfiguration, [])).toJSON();

            // options "Power, off"
            var operationOptions = [{value: "ON", label: "Power ON"},{value: "OFF", label: "Power OFF"},{value: "REBOOT", label: "Reboot"}];
            var scheduleOperations = (new InputConfigurationSocket(objScheduleOperations.itens, player4uc.i18n.getMessage("schedule_operations"), "SCHEDULE_OPERATION", false, ControllPowerOff.keyConfiguration, operationOptions)).toJSON();

            // data player
            var settings = [urlAccount, playerId, playerVersion, brightnessOptions, scheduleOperations];

            _log('get_settings', settings);

            // send
            that.managerConnect.socket.emit('receive_settings', settings);
        });
    };

    _global.ConfigurationSocket = new ConfigurationSocket();

}) (window);
