/* globals _log */
(function(_global) {
    'use strict';

    /**
     * Class for create object JSON for comunication socket
     * @param {string} value
     * @param {string} label
     * @param {string} type
     * @param {boolean} disabled
     * @param {string} key
     * @param {string|Array} options
     */
    function InputConfigurationSocket(value, label, type, disabled, key, options)
    {
        this.value = '';
        this.label = '';
        this.type = '';
        this.disabled = false;
        this.key = '';
        this.options = [];

        this.setValue(value || '');
        this.setLabel(label || '');
        this.setType(type || '');
        this.setDisabled(disabled || false);
        this.setKey(key || '');
        this.setOptions(options || []);
    }

    /**
     * Variable with types input
     * @type {Array}
     */
    InputConfigurationSocket.prototype.typeInput = ['TEXT', 'OPTION', 'BOOL', 'TIME', 'BRIGHTNESS', 'SCHEDULE_OPERATION'];

    /**
     * Set value default of input
     * @param {string} value
     */
    InputConfigurationSocket.prototype.setValue = function __doneSetValue (value)
    {
        this.value = value;
    };

    /**
     * Set label of input
     * @param {string} value
     */
    InputConfigurationSocket.prototype.setLabel = function __doneSetLabel (value)
    {
        this.label = value;
    };

    /**
     * Set type of input
     * @param {string} value
     */
    InputConfigurationSocket.prototype.setType = function __doneSetType (value)
    {
        if (this.typeInput.indexOf(value) === -1) {
            return _log("Type not suported.");
        }

        this.type = value;
    };

    /**
     * Set if input disable or enable
     * @param {boolean} value
     */
    InputConfigurationSocket.prototype.setDisabled = function __doneSetDisabled (value)
    {
        if (typeof value === "boolean") {
            this.disabled = value;
        }
    };

    /**
     * Set id for input
     * @param {string} value
     */
    InputConfigurationSocket.prototype.setKey = function __doneSetKey (value)
    {
        this.key = value;
    };

    /**
     * Set options
     * @param {string|array} value
     */
    InputConfigurationSocket.prototype.setOptions = function __doneSetOptions (value)
    {
        switch (typeof value) {
            case 'string':
                if (value.length)
                    this.options.push(value);
                break;
            case 'object':
                this.options = this.options.concat(value);
                break;
            default:
                _log("Type not suported.");
        }
    };

    /**
     * Parse class for JSON
     * @return {Object}
     */
    InputConfigurationSocket.prototype.toJSON = function __doneToJson ()
    {
        return {
            value: this.value,
            label: this.label,
            type: this.type,
            disabled: this.disabled,
            key: this.key,
            options: this.options
        };
    };

    /**
     * Parse class for string
     * @return {string}
     */
    InputConfigurationSocket.prototype.toString = function __doneToString ()
    {
        return JSON.stringify(this.toJson());
    };

    _global.InputConfigurationSocket = InputConfigurationSocket;
}) (window);
