/* globals _log, player4uc, Power, _, console, moment, async, shutdownOrRebootPlayer */
(function(_global) {
    'use strict';

    /**
     * Class for control Power/Off screen player
     * @todo Multi players
     *
     * @constructor ControllPowerOff
     */
    function ControllPowerOff()
    {
        this.keyConfiguration = 'settings.schedule_operation';
        this.setIntervalConfig = [];
        this.dataPlayer = player4uc.window.current().contentWindow.playerData;
        this.stateTV = this.ON;
        this.schedulerOperation = [];

        // init configuration
        this.init();
    }

        /**
     * TV Power ON
     * @type {String}
     */
    ControllPowerOff.prototype.ON = 'ON';

    /**
     * TV Power Off
     * @type {String}
     */
    ControllPowerOff.prototype.OFF = 'OFF';

    /**
     * TV Reboot
     * @type {String}
     */
    ControllPowerOff.prototype.REBOOT = 'REBOOT';


    ControllPowerOff.prototype.TimerWeek = {
        "0": Power.TimerWeek.SUNDAY,
        "1": Power.TimerWeek.MONDAY,
        "2": Power.TimerWeek.TUESDAY,
        "3": Power.TimerWeek.WEDNESDAY,
        "4": Power.TimerWeek.THURSDAY,
        "5": Power.TimerWeek.FRIDAY,
        "6": Power.TimerWeek.SATURDAY,
        "7": Power.TimerWeek.EVERYDAY,
    };

    /**
     * Init configurations for brightness
     * @method init
     */
    ControllPowerOff.prototype.init = function __doneInit ()
    {
        var that = this;

        // clear timeouts
        this.clearTimeouts(function __callbackInit() {
            // get config
            that.getConfigurationStorage(function __doneGet (config) {
                // case object empty
                if (!(that.keyConfiguration in config) || _.isNull(config[that.keyConfiguration])) {
                    _log("ControllPowerOff: Settings empty.", config);
                    config[that.keyConfiguration] = {};
                    config[that.keyConfiguration][that.dataPlayer.id] = {itens: []};
                }

                config = config[that.keyConfiguration][that.dataPlayer.id].itens;

                // max date
                var maxDate = moment().add(7, "days").toDate();
                var now = moment().toDate();


                // order config
                config = _.sortBy(config, 'schedule');
                async.eachSeries(config, function __doneEachSeries(item, nextEach) {
                    // init
                    var init = moment(item.schedule, "DD/MM/YYYY HH:mm").toDate();
                    init.setSeconds(0);

                    // max one week
                    if (init > maxDate || init < now) {
                        return nextEach();
                    }

                    var options = {};
                    options.hour = init.getHours();
                    options.minute = init.getMinutes();
                    options.week = that.TimerWeek[init.getDay()];

                    // power
                    if (item.operation.value === that.ON) {
                        options.inputSource = "ext://hdmi:1";
                        return addOnTimer(options, nextEach);
                    }

                    // off
                    if (item.operation.value === that.OFF) {
                        return addOffTimer(options, nextEach);
                    }

                    // reboot
                    if(item.operation.value == that.REBOOT){

                        if(init >= now){
                            _log("Reboot schedule", init);
                            var timeout = init.getTime() - now.getTime();
                            setTimeout(function() {
                                // este método está no arquivo resetConfiguration.js
                                shutdownOrRebootPlayer(true);
                            }, timeout);
                        }

                    }

                }, function () {

                    // time out for 1h
                    setTimeout(function() {
                        that.init();
                    }, (1000 * 60 * 60) * 1);
                });
            });
        });
    };

    function failureCbAddTime(cbObject) {
        var errorCode = cbObject.errorCode;
        var errorText = cbObject.errorText;

        console.log ("Error Code [" + errorCode + "]: " + errorText);
    }

    function addOnTimer (options, callback) {
        var power = new Power();
        power.addOnTimer(callback, failureCbAddTime, options);
    }

    function addOffTimer (options, callback) {
        var power = new Power();
        power.addOffTimer(callback, failureCbAddTime, options);
    }

    /**
     * Set status of screen player
     * @method set
     *
     * @param {float} status
     */
    ControllPowerOff.prototype.set = function __doneSet (status)
    {
        var options = {};
        options.displayMode = (status? Power.DisplayMode.DISPLAY_ON : Power.DisplayMode.DISPLAY_OFF);

        var power = new Power();
        power.setDisplayMode(function(){}, function(e){ console.warn("ERROR:", e); }, options);
    };

    /**
     * Set configuration
     * @method configuration
     *
     * @param  {string} controllPowerOff       Format: [{init: '10/10/2016', action: 'ON/OFF', }]
     */
    ControllPowerOff.prototype.configuration = function __doneConfiguration (controllPowerOff)
    {
        var that = this;

        // get data
        this.getConfigurationStorage(function __doneGet(config) {

            if (!(that.keyConfiguration in config) || config[that.keyConfiguration] === null) {
                config[that.keyConfiguration] = {};
                config[that.keyConfiguration][that.dataPlayer.id] = {"itens": []};
            }

            config[that.keyConfiguration][that.dataPlayer.id].itens = [];

            async.each(controllPowerOff,
                function(itemPowerOff, nextPowerOffSetting){
                    // push item
                    config[that.keyConfiguration][that.dataPlayer.id].itens.push(itemPowerOff);
                    nextPowerOffSetting();
                },
                function doneBrightnessConfig(){
                    // write data
                    that.setConfigurationStorage(config, function __doneApplyBrightness() {
                        // set configuration
                        that.init();
                    });
                }
            );
        });
    };

    /**
     * Clear configuration of local storage
     * @method clearConfiguration
     */
    ControllPowerOff.prototype.clearConfiguration = function __doneClearConfiguration ()
    {
        // overwrite
        var ab = {};
        ab[this.keyConfiguration] = null;
        this.setConfigurationStorage(ab);

        // clear timeouts
        this.clearTimeouts();
    };

    /**
     * Clear timeouts
     * @method clearTimeouts
     */
    ControllPowerOff.prototype.clearTimeouts = function __doneClearTimeouts (callback)
    {
        // remove all programation
        var power = new Power();

        function successCb(object, next) {
            if (typeof object === 'object' && object.timerList) {
                // each elements
                async.forEachOfSeries(object.timerList, function (item, key, next) {

                    console.log(item);

                    // clear Power
                    if ('inputSource' in item) {
                        power.deleteOnTimer(function() { next();}, function(a){console.warn("ERROR:", a); next();}, item);
                        return;
                    }

                    // clear Off
                    power.deleteOffTimer(function() { next();}, function(a){console.warn("ERROR:", a); next();}, item);
                }, function () {
                    // next
                    next();
                });
                return;
            }

            _log("ControllPoweroff.clearTimeouts Object", object);
            // next
            next();
        }
        function failureCb(cbObject, next) {
            var errorCode = cbObject.errorCode;
            var errorText = cbObject.errorText;

            console.warn ("Error Code [" + errorCode + "]: " + errorText);

            // next
            next();
        }

        async.parallel([
            function (next) {
                var options = {};
                options.allOnTimer = true;
                options.clearOnTimer = true;
                // list all times ON
                power.getOnTimerList(function (obj){successCb(obj, next); }, function (obj){failureCb(obj, next); }, options);
            },function (next) {
                // list all times OFF
                var options = {};
                options.allOffTimer = true;
                options.clearOnTimer = true;
                power.getOffTimerList(function (obj){successCb(obj, next);}, function (obj){failureCb(obj, next);}, options);
            },function (next) {

                // disable all itens
                var options = {};
                options.allOnTimer = true;
                options.clearOnTimer = true;
                power.enableAllOnTimer(function (obj){successCb(obj, next);}, function (obj){failureCb(obj, next);}, options);
            },function (next) {
                var options = {};
                options.allOffTimer = true;
                options.clearOffTimer = true;
                power.enableAllOffTimer(function (obj){successCb(obj, next);}, function (obj){failureCb(obj, next);}, options);
            },
        ], function () {
            // callback
            (callback? callback() : null);
        });
    };


    /**
     * Get data in local storage
     * @method getConfigurationStorage
     *
     * @param  {Function} callback
     */
    ControllPowerOff.prototype.getConfigurationStorage = function __doneGetConfigurationStorage (callback)
    {
        player4uc.storage.get([this.keyConfiguration], callback);
    };

    /**
     * Write data in local storage
     * @method setConfigurationStorage
     *
     * @param {Object} object
     * @param {Function} callback
     */
    ControllPowerOff.prototype.setConfigurationStorage = function __doneSetConfigurationStorage (object, callback)
    {
        player4uc.storage.set(object, callback);
    };

    _global.ControllPowerOff = ControllPowerOff;

}) (window);
