/* globals player4uc, FileSync, EventLayout, _, $, _log, _logError, async, ClassLayout, Player, resizeAll, HtmlPlayerArea, Rss, sync, FileSyncLayoutTemplate, console, PlayListManager, ManagerConnect, ConfigurationSocket, HttpFileSync, DownloadCenter, VideoPlayer, CarrosselVideoPlayer, ClassScreenShot */

'use strict';

window.APP_API_URL = 'http://chrome.4yousee.com/api';

/**
 * Caso o layout padrão seja alterado
 *
 * @property {boolean} layoutDefaultChange
 */
var layoutDefaultChange;

/**
 * Variável para identificar qual player está sendo tocado
 * @type {Object}
 */
window.typePlayer = window.typePlayer || {
    name: 'chrome'
};

/**
 * Versão do App
 * @type {String}
 */
var versionApp;


var verifyPlayerContent = {
    processAllFiles: false,
    donwloadContentVideo: false
};


var erroFileNewsLayout = true;

/**
 * Caso a janela principal esteja como fullscreen
 *
 * @type {Boolean}
 */
var windowFullScreen = true;


/**
 * Array com objetos do player. Formato: [{id: int, url: string}]
 *
 * @property {array} arrayObjctsPlayers
 */
var arrayObjctsPlayers;

/**
 * Variável com o nome da pasta do projeto
 *
 * @property {string} projectName
 */
var projectName;

/**
 * Variável com o hash para acessar a pasta do projeto
 * @property {string} appDir
 */
var appDir;

/**
 * Objeto da pasta do projeto
 * @property {Directory} rootDirEntry
 */
var rootDirEntry;

/**
 * Variável com o host do serviço http
 *
 * @property {String} hostPlayerHttp
 */
// var hostPlayerHttp = 'http://192.168.0.65/4yousee-chrome-player/storage/';
var hostPlayerHttp = 'http://localhost:9080/';

/**
 * Variável para controlar quando está sendo executado algum conteúdo.
 * GAMB: Não consigo no momento encontrar uma forma melhor de se fazer isso.
 * @type {Date}
 */
var controllPlayerContent;

/**
 * Variable with storage of brightness
 */
var brightness;

/**
 * Método para inicializar as configurações do projeto
 *
 * @method initConfiguration
 * @return {void}
 */
function initConfiguration (callback) {
    // Configurações que são padrão do projeto
    player4uc.storage.get(['settings.players', 'settings.projectName', 'settings.appDir', 'settings.fullscreen'], function (storageObj) {
        // lista de players
        window.arrayObjctsPlayers = storageObj['settings.players'];
        // pasta raiz do projeto
        window.projectName = FileSync.PROJECT_PREFIX + storageObj['settings.projectName'] || '';
        // pasta de escrita de aplicativos
        window.appDir = storageObj['settings.appDir'];
        // se o player deve estar em fullscree
        window.windowFullScreen = storageObj['settings.fullscreen'];
        // busca a pasta de arquivos do sistema
        window.rootDirEntry = 'file://internal/';
        // controll brightness
        window.brightness = storageObj['settings.brightness'] || [];

        // callback
        if (typeof callback == 'function') callback();
        // logs
        // console.info("storageObj", storageObj);
    });

    /**
     * Versão do App
     * @type {String}
     */
    window.versionApp = player4uc.runtime.xmlConfig.getValue("version");


    /**
     * Método para verificar o xml do servidor de atualizações
     */
    function updatePlayer() {
        var urlUpdate = player4uc.runtime.xmlConfig.getValue("urlupdate");

        // download do novo arquivo
        $.ajax({
            url: urlUpdate + 'config.xml',
            dataType: 'xml',
            success: function(responseXml) {
                var xml = responseXml.querySelector("widget");
                // verifica a versão do player
                if (xml.querySelector("version").textContent !== versionApp) {
                    var app = xml.querySelector("typeapp").textContent;
                    var ext = app.split(".").pop();

                    // verfica se o xml é de uma versão nova do player
                    verifyUpdate(xml, function __doneUpdate(status) {
                        if (!status) {
                            return;
                        }

                        // realizar o download do player
                        setServerPropertyLocal(urlUpdate + app, ext, true);
                    });
                }
            },
            error: function() {
                console.error("can't load the config.xml file!");
            }
        });
    }

    // verifica se existe atualização para Player
    updatePlayer();

    // evento para ser executa a cada x horas
    // ele irá fazer com que assim que o dia for alterado
    // caso o player fique ligado por  mais de 24 horas
    // troque a playlist
    window.dayOfWeek = new Date().getDay();

    //Agenda troca da playlist ao mudar o dia da semana
    setInterval(function _changeFileMedia() {
        var today = new Date().getDay();

        if(today != window.dayOfWeek){
            window.dayOfWeek = today;
            // evento para trocar o arquivo media do dia
            EventLayout.trigger('changeFileMedia', today);
            console.info('EVENTO PARA TROCAR A PLAYLIST DO DIA');
        }
    }, 1000 * 60 * 5);
}

/**
 * Função que começa todo o processo de player
 *
 * @method startupScreen
 * @return {void}
 */
function startupScreen() {
    // Cria a tela de configuração do player
    player4uc.window.create(
        'screens/start-config/startup-screen.html?' + new Date().getTime() , {
            bounds: {
                'width': window.screen.availWidth,
                'height': window.screen.availHeight
            },
            // frame: 'none', // disable window controlls (close, maximize, etc)
            id: 'configWindow',
            // state: 'fullscreen',
        },
        function windowCreated(window) {
            // window.fullscreen();
            _log('windowCreate - startup-screen');
            window.contentWindow.APP_API_URL = window.APP_API_URL;
            window.contentWindow.startMainWindow = startMainWindow;
        }
    );
}

/**
 * Método para iniciar a janela do player
 *
 * @method startMainWindow
 * @return {void}
 */
function startMainWindow(endCallback) {
    endCallback = (typeof endCallback === 'function'? endCallback : function() {});
    // var winwidth = parseInt(window.screen.availWidth/2);
    // var winheight = parseInt(window.screen.availHeight/2);

    // TODO:
    // ANTES QUE O SISTEMA INICIE TODAS AS JANELAS
    // VERIFICAR QUAIS SÃO OS PLAYERS E SABER SE TODOS OS ARQUIVOS ESTÃO LÁ
    // VERIFICAR SE PLAYER TEM CONDIÇÕES DE INICIAR

    var iterator = 1,
        windowId;

    // iniciando as configurações do sistema
    initConfiguration (function __doneInitConfiguration() {
        // pega informações sobre os monitores antes de abrir as janelas
        // será necessário pesquisar como utilizar o recursos para varios monitóres
        // player4uc.system.display.getInfo(configurationPlayers);
        configurationPlayers([
            {
                workArea: {
                    left: 0,
                    top: 0,
                    width: window.window.innerWidth,
                    height: window.window.innerHeight,
                }
            }
        ]);
    });

    /**
     * Método para realizar a configuração dos players
     * @param  {array} info
     * @return {void}
     */
    function configurationPlayers () {
        var players = window.arrayObjctsPlayers;
        var folder = window.projectName;
        var appDir = window.appDir;

        // largura/altura do monitor 0
        // var totalWidth = info[0].workArea.width;
        // var totalHeight = info[0].workArea.height;

        // verifica se tem o segundo monitor
        // var secondMonitor = null;
        // if (info.length > 1) {
        //     secondMonitor = info[1].workArea;
        // }

        // verifica quantas janelas por linha e qual a dimensão cada uma deve ter
        // var wins = players.length - (players.length % 2), // número de janelas no primeiro monitor
        //     perline = wins / 2, // total de janelas por linha
        //     winwidth = parseInt(totalWidth / (perline!=0?perline:1)), // largura de cada janela
        //     winheight = parseInt(totalHeight / 2), // altura de cada janela
        //     winleft =  info[0].workArea.left, // left inicial
        //     wintop = info[0].workArea.top; // top inicial

        // SAMSUNG ONLY
        // var winwidth =  info[0].workArea.width;
        // var winheight =  info[0].workArea.height;
        // var winleft = 0;
        // var wintop = 0;

        // O player 1 deve ser o padrão
        var listPlayersWindow = [players[0]];

        async.eachSeries(listPlayersWindow, function(playerData, cb){
            windowId = 'mainWindow-' + iterator;
            iterator++;

            // define qual é a orientação da janela
            window.OrientationPlayer = playerData.windowOrientationPlayer;

            /**
             * Abre as janelas dos players
             */
            player4uc.window.create(
                'screens/window.html', {
                    bounds: {
                        'width': window.screen.availWidth,
                        'height': window.screen.availHeight
                    },
                    id: windowId,
                },
                function (appWindow) {
                    // dados do player
                    appWindow.contentWindow.playerData = playerData;

                    // orientação do player
                    appWindow.contentWindow.OrientationPlayer = window.OrientationPlayer;

                    // Evento onload
                    appWindow.contentWindow.addEventListener('load', function() {
                        // starts the loop only after the html content was fully loaded
                        startSyncingFiles(appWindow, playerData, folder, appDir);

                        appWindow.contentWindow.startupScreen = startupScreen;

                        // Ao chegar aqui o chrome ainda não construiu a janela.
                        // classe de layout
                        appWindow.contentWindow.ClassLayout = new ClassLayout(
                            appWindow.contentWindow.document.getElementById('container') // <-- objeto container
                        );

                        // inicializando as configurações para ajuste de brilho
                        window.ControllBrightness = new ControllBrightness(100);

                        // inicializando as configurações para desligar e ligar a tv
                        window.ControllPowerOff = new ControllPowerOff();
                    });

                    // configura o appEnLight para o player
                    // AppEnlight.setRequestInfo({
                    //     "server": playerData.url,
                    //     "username": playerData.id,
                    //     'user_agent': navigator.userAgent
                    //   // request_id:"server_generated_uuid"
                    // });

                    // reajusta a orientação da tela
                    player4uc.window.orientation($(appWindow), playerData.windowOrientationPlayer);


                    // class áudio
                    // appWindow.contentWindow.ClassAudio = new ClassAudio(
                    //     appWindow.contentWindow.document.getElementById('container-audio'), // <-- objeto container
                    //     playerData
                    // );

                    // play no áudio
                    // applyAudioWindow(appWindow, playerData, true);

                    // Alterando a variável
                    appWindow.contentWindow.firstLayout = true;

                    // função de callback
                    cb();


                    //////////////////////
                    // TESTE DE MEMÓRIA //
                    //////////////////////
                    // textarea = appWindow.contentDocument.createElement("textarea");
                    // textarea.style.zIndex = 999999;
                    // textarea.style.position = 'absolute'
                    // textarea.style.backgroundColor = '#fff';
                    // textarea.style.width = '500px';
                    // textarea.style.height = '500px';
                    // textarea.style.bottom = 0;
                    // textarea.style.left = 0;

                    // document.body.appendChild(textarea);


                    // var memory = setInterval(function() {
                    // var deviceInfo = new DeviceInfo();
                    // deviceInfo.getSystemUsageInfo(function (o) {
                    //     var memo = ((1-o.memory.free/o.memory.total)*100).toFixed(1) + '%';
                    //     _log('memusage: ' + memo);
                    //     // textarea.textContent = memo;

                    //     }, function() {}, {memory : true});
                    // }, 20000);
                    //////////////////////
                    // TESTE DE MEMÓRIA //
                    //////////////////////
                }
            );

        }, function __doneEachSeriesPlayers(){
            // callback
            endCallback();
        });
    }

}

// class para tocar o conteudo rss
var classRssPlayerLayout = [];

/**
 * Método para montar o layout inicial
 *
 * @method applyLayoutWindow
 * @param  {Element} appWindow  Api da janela aberta
 * @param  {Play} playerData Classe Play
 * @return {void}
 */
function applyLayoutWindow (appWindow, playerData, idLayout, endCallback, forceChangeLayout) {

    // Busca json do layout
    var pathFile = window.projectName + '/' + FileSync.PLAYER_DIR_JSON + '/layout.' + playerData.id + '.json';

    // busca os arquivos na pasta de dados
    FileSync.readJSONFile(window.rootDirEntry, pathFile, createLayout);

    /**
     * Método para montar o layout na janela
     * @param  {Json} layoutJson Json dos layouts
     * @param  {string|null} error      Caso ocorra algum erro
     * @return {void}
     */
    function createLayout (layoutJson, error) {

        if (!_.isNull(error) || _.isNull(layoutJson)) {
            _logError(error);
            return endCallback();
        }

        var classLayout = appWindow.contentWindow.ClassLayout;

        // id do layout para aplicar
        var layoutId = classLayout.getIdLayout() || layoutJson.layouts.default;

        // caso seja informado um valor boleano verdadeiro será carregado o layout padrão
        // TODO: ISSO É UMA GAMB, PRECISA SER REVISTA E PENSAR EM UMA ALTERNATIVA
        if (typeof idLayout === 'boolean') {

            // caso a playlist principal tenha algum layout, o layout padrão não deve
            // ser carregado
            if (layoutDefaultChange === undefined) {
                layoutId = layoutJson.layouts.default;
            }

        } else if(_.isNumber(idLayout) || _.isString(idLayout)) {
            // caso seja informado algum layout
            layoutId = idLayout;
        }


        // caso o layout ainda não tenha sido aplicado
        if (classLayout.getIdLayout() !== layoutId+"" || forceChangeLayout) {

            // reseta o layout
            classLayout.resetLayout(!appWindow.contentWindow.firstLayout);

            // seta o arquivo de layout do cliente
            classLayout.setJsonAllLayout(layoutJson);

            // seleciona o layout informado ou o principal
            classLayout.setLayout(layoutId);

            // monta o layout
            classLayout.createLayout(undefined, !appWindow.contentWindow.firstLayout);

            // Os conteúdos estarem do tamanho da área do layout ao
            // executar o conteúdo
            var allArea = classLayout.getAreasAux() || [];
            allArea.push(classLayout.getAreaMain());
            resizeAll(
                allArea
            );

            // player nos templates
            playerContentAreas(classLayout);
        }

        // caso seja a primeira vez que o layout esteja sendo carregado
        if (appWindow.contentWindow.firstLayout) {
            // caso seja a primeira vez que o player está inicializando
            // será necessário que a playlist seja recarregada
            playlistManager.loadFiles();

            // seta a tag vídeo a classe que controla o vídeo
            // GAMB
            var tagContainer = classLayout.getAreaMain();
            var tagVideo = (tagContainer? tagContainer.getElementsByTagName("video")[0] : undefined);

            playlistVideo.setElementPlayer(tagVideo);
            playlistVideo.setElementContainer(tagContainer);
            tagVideo = null;
            tagContainer = null;

            // carrega a nova playlist
            playlistManager.resetPlaylistCurrent();

            // carrega a playlist
            playlistVideo.loadContent();

        }

        // variável para controlar a primeira troca de layout
        appWindow.contentWindow.firstLayout = false;

        // limite para função de callback
        // as outras funções são recursivas
        return endCallback();
    }

    /**
     * Método para rodar o conteudo nas áreas auxiliares
     * esse método faz com que somente conteudo HTML5 seja executado no conteudo
     * @return {void}
     */
    function playerContentAreas (classLayout) {
        // buscar todas as áreas auxiliares
        var arrayAreasAux = classLayout.getAreasAux();
        var i, time = 25;
        var path = window.projectName + '/layout/xml/';

        // pausa todos os conteúdos do layout
        _.each(classRssPlayerLayout, function(classRssPlayer) {
            classRssPlayer.clearTimeout();
        });

        // cria conteúdo para nova área
        for (i in arrayAreasAux) {

            // class para Player no conteúdo
            classRssPlayerLayout[i] = contructAreaLayout(appWindow, arrayAreasAux[i], time, path, classRssPlayerLayout[i]);

            // id do player
            classRssPlayerLayout[i].setIdPlayer(playerData.id);

            // iniciar o conteúdo
            (function (key) {
                setTimeout(function() {

                    // player no conteúdo
                    playerContentRssLayout(key, appWindow);
                }, 500);
            }) (i);
        }
    }
}

/**
 * Tocar a lista de notícias
 * @param  {integer} key Chave da posição da área
 * @return {void}
 */
function playerContentRssLayout(key, appWindow) {
    classRssPlayerLayout[key].play(function(msg, file){
        // TODO:
        //  - Avisar o socket
         // console.log("playerContentRssLayout: VOLTEI", new Date());

         // em caso de error
         if (_.isString(msg) && _.isUndefined(file)) {

            // logs
            // console.log("Notícia da barra do layout: não foi possível tocar.", msg);
            // espera um pouco para refaz todo o processo de construção do layout
            setTimeout(function() {

                // class layout
                var classLayout = appWindow.contentWindow.ClassLayout;

                // faz com o conteudo nas áreas sejam tocados
                playerContentAreas(classLayout);
            }, 5000);
            return;
         }
    }, key);
}

/**
 * Método para construir a classe responsável pela área do layout
 *
 * TODO: ESSE MÉTODO NÃO DEVE FICAR AQUI
 * @param  {Object} appWindow
 * @param  {ObjectHtml} arrayAreasAux
 * @param  {integer} time
 * @param  {string} path
 * @param  {undefined|HtmlPlayerArea} classRssPlayerLayout
 * @return {HtmlPlayerArea}
 */
function contructAreaLayout(appWindow, arrayAreasAux, time, path, classRssPlayerLayout) {
    // caso ainda não tenha sido definido
    if (_.isUndefined(classRssPlayerLayout)) {
        // class para Player no conteúdo
        classRssPlayerLayout = new HtmlPlayerArea(appWindow, new Rss(window.projectName + '/layout'), arrayAreasAux.getElementHtml(), time, path);
    }

    // seta o novo container
    classRssPlayerLayout.setContainer(arrayAreasAux.getElementHtml());

    // setando a fonte
    classRssPlayerLayout.setTemplate(arrayAreasAux.getTemplate());

    // caminho do arquivo
    var mediaFile = 'rss.' + arrayAreasAux.getFonte() + '.xml';

    // seta o arquivo de playlist
    classRssPlayerLayout.setPlayList(mediaFile);

    // gamb para esperar o chrome ler o arquivo de playlist
    // cancela a instancia anterior
    classRssPlayerLayout.clearTimeout();

    return classRssPlayerLayout;
}

/////////////////////////////////////////
// fica monitorando a troca de notícia //
/////////////////////////////////////////
addEventListener('message', function(e) {
    var obj = e.data;
    var keyArea = parseInt(obj.data.keyArea);
    var idWindow = obj.data.idWindow;
    var windowApp = player4uc.window.get(idWindow);

    // verifica se aconteceu algum problema com o template
    // LEGENDA:
    // 1 - Recarregar o layout
    if (obj.error == 1) {
        // logs
        _logError({
            'Error Layout ID': idWindow,
            'obj': obj,
        });

        // caso o problema tenha sido na área principal
        if (obj.data.main) {

            // em caso de erros o conteúdo deve ser trocado.
            controllPlayerContent = undefined;

            // próximo conteúdo
            nextContentMain();

            return;
        }

        // flag para evitar que todo o layout seja reiniciados
        erroFileNewsLayout = false;

        // processa o arquivo de layout novamente
        var filePlayer = FileSyncLayoutTemplate.getNameFile(windowApp.player.id);
        sync.processFileTemplate([filePlayer], function() {

            // reseta o somente á area
            EventLayout.trigger('loadTemplate', keyArea);
        });

        clearInterval(classRssPlayerLayout[keyArea].Timeout[keyArea]);
        classRssPlayerLayout[keyArea].Timeout[keyArea] = undefined;

        return;
    }

    // caso seja área principal
    if (obj.data.main) {

        if (obj.status) {
            // próximo conteúdo
            nextContentMain();
        }

        return;
    }

    // caso seja as áreas auxiliares do layout
    if (obj.status && classRssPlayerLayout[keyArea].Timeout[keyArea] !== undefined) {

        // para a função para monitorar a notícia
        clearInterval(classRssPlayerLayout[keyArea].Timeout[keyArea]);
        classRssPlayerLayout[keyArea].Timeout[keyArea] = undefined;
        // console.log("message: event", e);

        // callback
        // playerContentRssLayout(keyArea);
        classRssPlayerLayout[keyArea].changeNews(keyArea);
    }

    /**
     * Método para buscar o próximo conteúdo da área principal
     * @return {void}
     */
    function nextContentMain() {

        var contentWindowApp = windowApp.contentWindow;
        var nameFile = obj.data.file;
        var datePlay = new Date().toString();

        // caso essa função já tenha sido chamada
        // e nenhum conteúdo html está sendo tocado ele para a função
        if (contentWindowApp.HtmlTimeOut === undefined) {
            return;
        }

        // TODO: MUDAR ESSE BLOCO PARA UMA FUNÇÃO
        // conteúdo html
        clearInterval(contentWindowApp.HtmlTimeOut);
        contentWindowApp.HtmlTimeOut = undefined;

        // variável disponível no arquivo player.js
        classHtmlPlayer.getCallback()(null, nameFile);

        // logs
        return _log("----> HTML FIM", obj, datePlay);
    }
});

////////////////
// fim evento //
////////////////

/**
 * Método para poder executar o próximo conteúdo
 *
 * @method playNext
 */
var playNext, classPlayer;

/**
 * Busca o próximo conteúdo
 * @return {[type]} [description]
 */
function nextContent(preventUpdate) {
    var appWindow = player4uc.window.get('mainWindow-1');
    // busca o próximo item
    var next = playlistManager.getNext(preventUpdate);
    var item = playlistManager.getPlaylistItem(next);
    if(item) {
        appWindow.haveAItem_function(item);
    } else {
        setTimeout(function() { nextContent(preventUpdate); }, 5000);
    }
}
playNext = nextContent; // TODO: REFATORAR

/**
 * Método para fazer com que na área principal execute a playlist
 *
 * @method startMainLoop
 * @param  {Element} appWindow       [description]
 * @param  {PlaylisManager} playlistManager [description]
 * @param  {ManagerConnect} managerConnect  [description]
 * @param  {Object} playerData      [description]
 * @param  {string} folder          [description]
 * @param  {string} appDir          [description]
 * @return {void}                 [description]
 */
function startMainLoop(appWindow, playlistManager, managerConnect, playerData, folder, appDir, preventUpdate) {
    // get the playlist

    var next, verifyPlayerContentCount = 0;
    var item = null;

    if (classPlayer) {
        return;
    }


    /**
     * Função recursiva para rodar o conteudo
     * @return {void}
     */
    appWindow.haveAItem_function = function  (item){
        if(!item || (!('arquivo' in item) || item.arquivo === "") && item.tipo !== 'n') {
            // throw new Error(player4uc.i18n.getMessage("cantLoadPlaylist") + ' - File not found');
            return setTimeout(function() { nextContent(); }, 5000);
        }

        // plays the content
        var fileExt = item.arquivo.split('.').pop();
        var type = classPlayer.getTypeMapping(fileExt, item.tipo);

        // caso seja para trocar o layout
        if (item.tipo === 'l') {
            // troca layout
            _log('Trocando o layout', item.arquivo);

            // flag
            layoutDefaultChange = true;

            // troca o layout
            return EventLayout.trigger('loadLayout', [appWindow, item.arquivo, function __done () {
                // verifica se será preciso trocar a playlist de vídeo
                // TODO: ISSO NÃO DEVERIA FICAR AQUI
                EventPlayer.trigger('updatePlaylistVideo', [
                    undefined, //element html
                    function __callbackHtmlPlayer() { // callback
                        // callback
                        // O timeout é para que dê tempo da tv
                        // realizar o load do vídeo
                        nextContent();
                    }
                ]);
            }]);
        }

        // limpa todos os callbacks
        classPlayer.clearTimeOutPlayer();

        // play content
        classPlayer.play(type, item, function(error, arquivo){

            console.log('%c O conteúdo já foi tocado!', 'background: #222; color: #bada55', JSON.stringify(item));

            // caso não haja errors
            if(!error) {
                // tratando os dados para manager
                // caso o tipo for 'car' de carrocel ele deverá ser enviado para o manager como 'i' de institucional
                var tipo = item.tipo;
                if (tipo == 'car') {
                    tipo = 'i';
                }

                // TODO: rever isso aqui
                if (typeof arquivo === 'object') {
                    arquivo = arquivo.arquivo;
                }

                // envia os logs para o manager
                managerConnect.log.save('playlog', arquivo, tipo);
            }

            // proximo item
            nextContent();
        });
    };

    // caso ainda não exista layout para tocar o conteúdo
    if (appWindow.contentWindow.firstLayout) {
        // carrega layout
        return EventLayout.trigger('loadLayout', [appWindow, undefined, function __doneCallLoadLayout() {
            // id da janela
            appWindow.contentWindow.id = appWindow.id;

            // instanciando a classe player
            classPlayer = new Player(window.projectName, appWindow.contentWindow, playerData.url, playerData.id);

            // Próximo item
            nextContent(preventUpdate);
        }, true]);
    }
}



var screenShot, managerConnect, playlistVideo, playlistManager;

function startSyncingFiles(appWindow, playerData, folder, appDir) {
    var sync;

    // pasta do projeto
    folder = window.projectName;

    /**
     * Gerenciado de vídeos (especifico para LG)
     * @type {VideoPlayer}
     */
    playlistVideo = new VideoPlayer({
        "getPathContent": window.projectName + '/',
        "getPathContentPlay": window.projectName + '/' + FileSync.PLAYER_DIR_MEDIA + '/',
    },CarrosselVideoPlayer);
    appWindow.playlistVideo = playlistVideo;
    appWindow.contentWindow.playlistVideo = playlistVideo;


    /**
     * Gerenciador da playlist do player
     * @type {PlayListManager}
     */
    playlistManager = new PlayListManager(playerData.id, playerData.url);
    appWindow.playlistManager = playlistManager;
    appWindow.contentWindow.playlistManager = playlistManager;



    // classe para controlar o socket
    managerConnect = new ManagerConnect(playerData.id, playerData.url);
    managerConnect.listenUpdateCb = function(data){
        // console.log("listenUpdateCb: data", data);

        if(data.fileType == 'list') {
            _log('Sync Started by manager');
            window.sync.init(true, appWindow);
        }
    };

    // set a classe managerConnect
    ConfigurationSocket.setClassManangerConnect(managerConnect);

    // Registra a classe que cuida dos arquivos utilizados pelo player
    sync = new HttpFileSync(folder, playerData, playlistManager, managerConnect, false, appWindow);
    window.sync = sync;

    ////////////////
    // ScreenShot //
    ////////////////
    screenShot = new ClassScreenShot(managerConnect.socket);

    // iniciar a classe de download center
    window.downloadCenter = new DownloadCenter(playerData.url, folder, rootDirEntry, managerConnect);

    appWindow.contentWindow.$ = appWindow.contentWindow._$ = window._$ = $;
    appWindow.contentWindow._ = appWindow.contentWindow.__ = window.__ = _;
    appWindow.contentWindow.async = appWindow.contentWindow._async = window._async = async;

    setTimeout(function() {
        // começa a tocar o conteúdo
        startMainLoop(appWindow, playlistManager, managerConnect, playerData, folder, appDir, true);

        // inicia a download
        initDownloadPlayers();
    }, 1000);
}

/**
 * Método para baixar os arquivos do player
 * @param  {Object} playerData Objeto do player que deseja baixar os conteúdos
 * @return {void}
 */
function multiPlayersTvLg(playerData) {

    var _managerConnect = new window.parent.ManagerConnect(playerData.id, playerData.url);
    var _folder = window.projectName;

    // passa em player por player
    // checkProject
    new window.parent.HttpFileSync(_folder, playerData, undefined, _managerConnect, undefined, undefined, true);
}

/**
 * Método para iniciar o download dos arquivos de outros players
 * @param  {Object} players
 * @return {void}
 */
function downloadContentAnotherPlayers(players) {
    async.each(players, function(playerData, endCallBack){
        // baixa conteúdos
        multiPlayersTvLg(playerData);

        // próximo player
        endCallBack();
    });
}

/**
 * Método para alterar a playlist a ser utilizada pelo player
 * @param  {Object} playerData
 * @return {void}
 */
function changePlaylistPlayer(playerData) {

    var windowIframe = player4uc.window.current();
    var classPlayer = windowIframe.player;
    var classPlaylistManager = windowIframe.playlistManager;

    // classe que deverão ser trocadas

    // ManagerConnect
    managerConnect.setPlayerId(playerData.id);
    managerConnect.setEnv4YouSee(playerData.url);

    // HttpFileSync
    sync.setObjectPlayer(playerData);
    sync.setNameFiles(playerData);

    // Player
    classPlayer.setPlayerId(playerData.id);

    // FileSync
    FileSync.setPlayer(playerData);

    // PlayListManager
    classPlaylistManager.setPlayerId(playerData.id);
    classPlaylistManager.setPlayerUrl(playerData.url);

    EventLayout.trigger('changeFileMedia');

    // escode todos as janelas
    // TODO:  NÃO GOSTEI DISSO, DEVE SER REFATORADO
    windowIframe.contentWindow.openSelectsPlayers('hide');
    windowIframe.contentWindow.toggleMenu('hide');
}

/**
 * Método para iniciar download dos arquivos de outros players
 * @return {void}
 */
function initDownloadPlayers() {
    setTimeout(function() {
        // baixa conteúdo dos demais players
        var players = arrayObjctsPlayers.slice(1);
        downloadContentAnotherPlayers(players);
    }, 10000);
}
