// 'use strict';
/**
TODOS:
- Finish and test loading players and projects
*/
angular.module('4YouSeeChrome', ['chrome-i18n'])
.config(function($provide){

    $provide.decorator("$exceptionHandler", function($delegate, $injector){
        return function(exception, cause){
            _log(exception.toString());
        };
    });
})

.controller('StartupScreenCtrl', ['$scope', '$timeout', 'ConfigSvc', 'Project', 'Player', '$filter', StartupScreenCtrl]);

function StartupScreenCtrl($scope, $timeout, ConfigSvc, Project, Player, $filter) {

    // window.localStorage.clear();
    $scope.version = window.parent.versionApp;
	$scope.steps = [false, false, false];
	$scope.numberOfPlayers = '1';

	// holds the players and projects list
	$scope.players	= [
        // {
        //     "url": 'http://checklist.4yousee.com.br',
        //     "id": 10,
        //     "isvalid": true,
        //     "windowOrientationPlayer": 0
        // },
        // {
        //     "url": 'http://4usee.com/lg/7045BA/',
        //     "id": 4,
        //     "isvalid": true,
        //     "windowOrientationPlayer": 0
        // }
    ];
	$scope.formIsValid 	= false;
	$scope.projectList = [];


	$scope.showLoading = true;

	$scope.noDirSelectedShow = false;
    $scope.selectedDir = false;

    $scope.selectedPlayer = '';
    $scope.selectedProject = '';

    $scope.urlServer4UC = '';

    /**
     * Variável para controlar se a url do ambiente e todos os players são válidos.
     * @type {Boolean}
     */
    $scope.AllConfigurationValid = false;

	// show the step screen
	$scope.showStep = function(idx) {
		$scope.steps = [false, false, false];
		$scope.steps[(idx - 1)] = true;
		// hides the loading message
		$scope.showLoading = false;
	};

	$scope.step2 = function() {
		$scope.showStep(2);

        ConfigSvc.getDeviceToken(function(token) {
            ConfigSvc.getPlayers(checkPlayers);

            // check if the player was configured, shows the step 2 screen if not
            function checkPlayers(players) {
                _log("players", players);
                // inicia o player
                if (players) {
                    // console.log("player id: ", players);
                    return $scope.finish(true);
                }

                // console.log("passando por dentro");
                $scope.showLoading = true;

                // callback for when the projects are loaded
                $scope.selectedProject = 'default_project';
            }
        });
	};

	$scope.step3 = function() {
        document.body.focus();
        document.getElementById("startPlayer").focus();
	};

    /**
     * Copia da lista dos players
     * @type {Array}
     */
    $scope.playersToSave = [];



	/**
	 * Método para disponibilizar os canais que o player ira trocar.
	 * @return {void}
	 */
	$scope.savePlayers = function (){
		var numberOfPlayers = $scope.numberOfPlayers.split(',');

        // reseta lista
        $scope.playersToSave = [];

		for (var i = 0; i < numberOfPlayers.length; i++) {

            // atualiza o valor dos players
            var input_url = document.getElementById("urlServer4UC_" + i);
            var input_id = document.getElementById("selectedPlayer_" + i);

            $scope.players[i].url = input_url.value;
            $scope.players[i].id = input_id.value;

			// verifica se não existe player com id duplicado
			if (!validatePlayer($scope.players[i])) {
				// TODO: Id de players repetidos
				sweetAlert('Ops...', $filter("i18n")('PlayerIdsSame'), 'error');
				return false;
			}

            // verifica se arquivo list exist para o player
            (function (id) {
                var player;

                // copia do object
    			player = angular.copy($scope.players[id]);
    			delete player.isvalid;

                requestFileListPlayer(player, function (status) {

                    // em caso de erros
                    // COMENTADO PARA TESTES, DESCOMENTAR DEPOIS
                    if (!status) {
                        sweetAlert('Ops...', 'Player ' + player.id + ' inválido.', 'error');
                        return false;
                    }

                    // grava player
                    $scope.$apply(function() {
                        $scope.playersToSave.push(player);
                    })
                });
            }) (i);
		};
    }

    /**
     * Listener para verificar se os players são válido para serem persistidos
     * @param  {boolean} oldValue
     * @param  {boolean} newValue
     * @return {void}
     */
    $scope.$watchCollection('playersToSave', function (newValue, oldValue) {
        if (newValue.length && newValue.length === $scope.players.length) {

            // sava todos os players
            saveAllPlayers(newValue);

            // passa para a próxima tela
            $scope.showStep(3);
        }
    });

    /**
     * Método para savar o array com todos os players
     * @param  {array} playersToSave
     * @return {void}
     */
    function saveAllPlayers (playersToSave) {
        ConfigSvc.setPlayers(playersToSave);
        ConfigSvc.setProject($scope.selectedProject);
    }
    /**
     * Método para realizar uma requisição para o player e verificar se ele é válido
     * @param  {object}   player
     * @param  {Function} callback
     * @return {void}
     */
    function requestFileListPlayer (player, callback) {

        var url = player.url + "/common/json/list." + player.id + ".json";

        $.ajax({
            "url": url,
            "method": "GET",
            "cache": false,
            "success": function () {
                // callback sucesso
                callback(true);
            },
            "error": function() {
                // callback erro
                callback(false);

                // logs
                _logError({
                    "url": url,
                });
            }
        });
    }

	$scope.validatePlayer = validatePlayer;
	$scope.debounceValidatePlayer = _.debounce(validatePlayer, 500);

    /**
     * Método resetar todas as configurações dos players
     * @return {void}
     */
    function resetAllPlayers () {
        /**
         * Método para resetar as configurações do player
         * @return {void}
         */
        function resetConfig() {
            // reseta configurações
            window.localStorage.clear();
            player4uc.runtime.reload();
        }

        resetConfig();
    }


    /**
	 * Método para verficar se o player é válido
	 * @param  {object} objPlayer Objecto do player
	 * @return {boolean}
	 */
	function validatePlayer (objPlayer) {
        // caso esteja vazio
        if (_.isUndefined(objPlayer) || _.isEmpty(objPlayer)) {
            return false;
        }

		// habilita todos os players
		_.each($scope.players, function(objectPlayer) {
			objectPlayer.isvalid = true;
		});

		var idPlayerExist =  _.where($scope.players, {id: objPlayer.id});
		if (idPlayerExist.length > 1) {
			// TODO: não deveria fazer isso
			_.each(idPlayerExist, function (item) {
				item.isvalid = false;
			});

			return false
		}

		return true;
	}

	$scope.validateEnvironment = validateEnvironment;
	/**
	 * Método para verificar se URL do ambiente é válida ou não
	 * @param  {string} url Url do ambiente
	 * @return {boolean}
	 */
	function validateEnvironment(url) {

		// caso não seja preechindo
		if (_.isUndefined(url) || _.isEmpty(url)) {
			return false;
		}

        // TODO: TIRA ISSO DAQUI
		// atribue a url para todos os players
		for (var i = 0; i < $scope.players.length; i++) {
            // verifica se na url tem http://
            if (url.indexOf("http://") == -1) {
                url = 'http://' + url;
            }

            $scope.players[i].url = url
		};


		// ativa o botão confirmar
		// TODO: após verificar se a url e os ids dos players forem válidos
		$scope.formIsValid = true;

		return true;
	}

	// sends the flow control to the player screen
	$scope.finish = function(valided) {
        if (!valided) {
            return resetAllPlayers();
        }

		console.log("starting main window...");
		startMainWindow(function() {
            // fecha a janela de configuração
            player4uc.window.getAll()['configWindow'].close();
        });

        console.log('closing startup screen...');
	};

    /**
     * Método para iniciar a tela de configuração
     * @return {void}
     */
	function init() {

		ConfigSvc.is4UCDirSetted(function(items) {
            // por padrão o player inicia como fullscreen
            if (items['settings.fullscreen'] === undefined) {
                player4uc.storage.set({'settings.fullscreen': true});
            }

            // show the step2 screen
            $scope.step2();
		});
	}

    // iniciando o projeto
    init();
}


angular.module('4YouSeeChrome')
.factory('ConfigSvc', function() {
	return {
		// checks if the 4UC directory was selected by the user
		is4UCDirSetted: function(callback) {
			player4uc.storage.get(['settings.appDir', 'settings.fullscreen'], callback);
		},
		// checks wheter the device token was generated and generate when it was not
		getDeviceToken: function(callback) {
			player4uc.storage.get(['settings.deviceToken'], getDeviceTokenReturn);

			function getDeviceTokenReturn(storageObj) {

				// console.log('------ getDeviceToken', storageObj);
				var token;
				if (typeof storageObj['settings.deviceToken'] === 'undefined') {
					// TODO: generate a real uniqid() device token
					token = "34a5s6dfasdf";
					player4uc.storage.set({'settings.deviceToken': token});
				} else {
					token = storageObj['settings.deviceToken'];
				}
				callback(token);
			}
		},
		getPlayers: function(callback) {
			player4uc.storage.get(['settings.players'], getPlayersReturn);

			function getPlayersReturn(storageObj) {

				// console.log('------ getPlayers', storageObj);
				var players = false;
				if (typeof storageObj['settings.players'] !== 'undefined') {
					players = storageObj['settings.players'];
				}
				callback(players);
			}
		},
		setPlayers: function(playerId) {
			// console.log('------ setPlayers', playerId);

			player4uc.storage.set({'settings.players': playerId});
		},
		setProject: function(project) {

			// console.log('------ setProject', project);

			// player4uc.storage.set({'settings.projectId': project._id});
			player4uc.storage.set({'settings.projectName': project});
		},
		set4YouSeeEnv: function(env) {
			// console.log('urlServer4UC', env)
			player4uc.storage.set({'settings.env4YouSee': env});
		}
	};
})

// service that reflects the Project's REST API of 4yousee chrome
.factory('Project', ['$http', function($http) {
	// return $resource('/projects/:playerId', {'playerId': '@playerId'});
	return {
		get: function(playerId, user_token, callback) {

			callback([]);
			return;
		}
	};
}])
// service that reflects the Player's REST API of 4yousee chrome
.factory('Player', ['$http', function($http) {
	// return $resource('/players', {'project'});
	return {
		get: function(playerSelected, urlServer4UC, callback) {
			var url = urlServer4UC + '/common/json/list.' + playerSelected + '.json';
			// console.log('player ---', url)

			$http.get(url).success(function(data) {
				// console.log(data);
				callback(data);
			})
			.error(function(data) {
				callback(null, data);
			});
		}
	};
}])

.directive('ngSamsungRemoteControl',['$timeout', function($timeout){
    return function(scope, element, attrs) {
        $timeout(init, 1000); // Fix to init before the DOM ready

        function init () {

            // colocando o foco na tag body
            document.getElementById("urlServer4UC_label").focus();

            try{

                var keyboard = new Keyboard();
                var remoteControlNavigator = new RemoteControlNavigator();

                keyboard.init({
                    selectors: ['.tabindex']
                });

                remoteControlNavigator.init({
                    selectors : '.tabindex',
                    cssClass: 'inputHighlight',

                    onSetFocus: function(element){
                        // element.focus();
                    },
                    onSetInput: function (element) {
                        // element.focus();
                    }
                });

                // remoteControlNavigator.setActive('#urlServer4UC');
            } catch (error) {
                _log('error', error)
            }

            var keycode;

            document.addEventListener("keydown", function(inEvent){

                if(window.event) {
                    keycode = inEvent.keyCode;
                } else if(e.which) {
                    keycode = inEvent.which;
                }
                _log('keycode', keycode)

                // KEY Return
                if(keycode == 88) {
                    keyboard.close();
                    remoteControlNavigator.setBlur();
                }

                // KEY Control bottom && KEY Control right
                if(keycode == 29461 || keycode == 5 || keycode == 40) {
                    inEvent.stopPropagation();
                    inEvent.preventDefault();
                    remoteControlNavigator.nextElement();
                    return false;
                }

                // KEY Control Up && KEY Control left
                if(keycode == 29460 || keycode == 5 || keycode == 38) {
                    inEvent.stopPropagation();
                    inEvent.preventDefault();
                    remoteControlNavigator.prevElement();
                    return false;
                }

                // KEY Control Enter && KEY End
                if(keycode == 29443 || keycode == 13) {
                    remoteControlNavigator.setFocus();
                    remoteControlNavigator.onKeyEnter();
                }
            });
        }
    };
}]);
