(function() {

    /**
     * Classe para gerenciar as traduções do sistema
     *
     * @class GenericI18n
     * @constructor
     *
     * @param {string} language Linguagem a ser usada
     */
    function GenericI18n(language) {
        /**
         * @property defaultLanguage
         * @type {String}
         * @default "en"
         */
        this.defaultLanguage = 'en';

        if (!language) {
            language = this.defaultLanguage;
        }

        /**
         * @property language
         * @type {String}
         * @default "en"
         */
        this.language = language;

        /**
         * Armazena as strings de tradução do sistema
         * @property messages
         * @type {Object}
         */
        this.messages = {};

        this.getTranslationFile();
    }

    /**
     * Método para mudar a língua que está sendo utilizada
     *
     * @method setLanguage
     * @param {string} language a Língua
     */
    GenericI18n.prototype.setLanguage = function(language) {
        this.language = language;
    };

    /**
     * Método para buscar o arquivo com as traduções
     *
     * @method getTranslationFile
     * @param {Function} (optional)onfinish Callback a ser chamado quando concluir o carregamento das strings
     */
    GenericI18n.prototype.getTranslationFile = function(onfinish) {
        var that = this;

        $.ajax({
            url: MAIN_PATH + '_locales/' + this.language + '/messages.json',
            dataType: 'json',
            success: function(result) {
                that.messages = result;
                if (onfinish) {
                    onfinish();
                }
            },
            error: function() {
                console.error("The APP can't load the language file for " + this.language + " language");
                if (that.language != that.defaultLanguage) {
                    that.setLanguage(that.defaultLanguage);
                    that.getTranslationFile();
                } else {
                    throw new Error("The APP can't load the default language file!");
                }
            }
        });
    };

    /**
     * Busca uma mensagem no arquivo de traduções
     *
     * @method getMessage
     * @param {string} key A chave que é para ser buscada. Se não encontrar, retornará a própria chave
     * @return {string}
     */
    GenericI18n.prototype.getMessage = function(key) {
        if (_.has(this.messages, key)) {
            return this.messages[key].message;
        } else {
            return key;
        }
    };

    window.GenericI18n = GenericI18n;

})();
