(function() {

    /**
     * Classe para tratar o localStorage em dispositivos com suporte ao Storage do HTML5
     *
     * @class GenericStorage
     * @constructor
     * @param {player4uc.runtime} runtime
     */
    function GenericStorage(runtime) {

        if (typeof runtime == "undefined") {
            throw new Error("runtime argument for GenericStorage is empty");
        }

        this.storage = window.localStorage;
    }


    /**
     * Método responsável por gravar um item no localStorage
     *
     * @method set
     * @param {Object} items Objeto com chaves e valores a serem armazenados
     * @param {Function} (optional)callback Função opcional que será chamada se tudo der certo
     * @return {void}
     */
    GenericStorage.prototype.set = function(items, callback) {
        var that = this;

        _.each(items, function(value, key) {
            try {
                that.storage.setItem(key, JSON.stringify(value));
            } catch (e) {
                // TODO: gravar o erro em runtime.lastError
                throw e;
            }
        });

        if(callback)
            callback();
    };

    /**
     * Método responsável por retornar uma ou mais chaves armazenadas
     *
     * @method get
     * @param {string|array} (optional)keys As chaves que se deseja buscar. Se passar null serão retornados todos os valores do storage
     * @param {Function} callback A função que deverá ser chamada após recuperar todos os itens do storage.
     *         deve ser no formato function(items) {...}
     * @return {void}
     */
    GenericStorage.prototype.get = function(keys, callback) {
        var items = {};
        var that = this;
        if (typeof keys == "string") {
            keys = [keys];
        }

        _.each(keys, function(key) {
            var value = that.storage.getItem(key);

            try {
                value = JSON.parse(value);
            } catch (e) {
                value = value;
                // throw e;
            }

            if (value) {
                items[key] = value;
            } else {
                items[key] = null;
            }
        });

        if(callback)
            callback(items);
    };

    /**
     * Método responsável por remover os itens do storage
     *
     * @method remove
     * @param {string|array} keys Lista de chaves para apagar
     * @param {Function} (optional)callback Função a ser chamada após apagar todos os itens
     * @return {void}
     */
    GenericStorage.prototype.remove = function(keys, callback) {
        var that = this;
        if (typeof keys == "string") {
            keys = [keys];
        }

        _.each(keys, function(key) {
            try {
                that.storage.removeItem(key);
            } catch (e) {
                // TODO: gravar o erro em runtime.lastError
                throw e;
            }
        });

        if(callback)
            callback(items);
    };


    window.GenericStorage = GenericStorage;
})();
