(function(_global) {

    /**
     * Keyboard code
     * @type {Number}
     */
    var keycode;


    /**
     * LG Keyboard callback
     * @type {Number}
     */
    this.onKeyEnter;
    this.onSetInput;
    this.onSetFocus;

    /**
     * Class to implements remote control navigation
     *
     * @class RemoteControlNavigator
     * @constructor
     * @param {String | Array}
     */
    function RemoteControlNavigator(options) {
        this.tabindex = -1;
        this.elements = [];
        this.cssClass = '';

        if(options)
            this.init(options);
    }

    RemoteControlNavigator.prototype.init = function(options) {

        if(typeof options != 'object')
            return false;

        this.selectors = options.selectors || [];
        this.cssClass = options.cssClass || '';
        this.onKeyEnter = options.onKeyEnter || function(){};
        this.onSetFocus = options.onSetFocus || function(){};
        this.onSetInput = options.onSetInput || function(){};

        if(typeof this.selectors == 'string')
            this.selectors = [this.selectors];

        this.updateVisibleInputs();
    }

    RemoteControlNavigator.prototype.setActive = function(selector) {
        if(!selector)
            return false;

        var element;
        element = this.isElement(selector) ? selector : document.querySelector(selector);

        this.setInput(element);
        this.setFocus(element);
    }

    /**
     * Increment tabindex
     */
    RemoteControlNavigator.prototype.nextElement = function (){
        var updated = this.updateVisibleInputs();

        // blur in all inputs
        this.setBlur();
        console.log('nextElement updated', updated)
        console.log('before update', this.elements);

        this.tabindex ++;

        if(this.tabindex == this.elements.length || updated)
            this.tabindex = 0;

        this.setInput();
    }

    /**
     * Decrement tabindex
     */
    RemoteControlNavigator.prototype.prevElement = function (){
        var updated = this.updateVisibleInputs();

        // blur in all inputs
        this.setBlur();
        console.log('prevElement updated', updated)
        this.tabindex --;

        if(this.tabindex < 0 || updated) {
            this.tabindex = this.elements.length -1;
        }

        this.setInput();
    }

    RemoteControlNavigator.prototype.updateVisibleInputs = function () {
        var oldElements = this.elements;
        var similarElements;

        this.elements = [];
        for (var i = 0; i < this.selectors.length; i++) {
            if(this.isElement(this.selectors[i])) {
                if(this.isVisible(this.selectors[i]))
                    this.elements.push(this.selectors[i]);
            } else {
                similarElements = document.querySelectorAll(this.selectors[i]);

                for (var j = 0; j < similarElements.length; j++) {

                    if(this.isVisible(similarElements[j]))
                        this.elements.push(similarElements[j]);
                }
            }
        }

        for (var i = 0; i < oldElements.length; i++) {
            if(oldElements[i] != this.elements[i])
                return true;
        };

        return false;
    }

    /**
     * Highligt the current Element.
     * @param {Object} element [description]
     */
    RemoteControlNavigator.prototype.setInput = function (element) {

        for (var i = 0; i < this.elements.length; i++) {
            this.elements[i].classList.remove(this.cssClass);
        };

        if(element && this.isElement(element)) {
            this.onSetInput(element);
            return element.classList.add(this.cssClass);
        }

        this.onSetInput(this.elements[this.tabindex]);
        this.elements[this.tabindex].classList.add(this.cssClass);
    }

    /**
     * Select current element then active keyboard
     */
    RemoteControlNavigator.prototype.setFocus = function (element) {

        if(!this.isElement(element))
            element = this.elements[this.tabindex];

        if(element.tagName != 'INPUT' && element.tagName != 'TEXTAREA')
            element.click();
        else
            element.focus();

        this.onSetFocus(element);
    }

    /**
     * Method blur in all elements
     * @param {ElementHtml} element
     */
    RemoteControlNavigator.prototype.setBlur = function _doneSetBlur(element) {
        if (!element) {
            for (var i in this.elements) {
                this.elements[i].blur();
            }

            return;
        }

        element.blur();
    }

    /**
     * Verify if is dom element
     * @param  {Object}  el [description]
     * @return {Boolean}    [description]
     */
    RemoteControlNavigator.prototype.isElement = function (el) {
        var tagName = '';
        if(typeof el != 'object')
            return false;

        if(!('tagName' in el))
            return false;

        el.tagName = tagName;

        if(el.tagName != tagName)
            return true;

        return false;
    }

    RemoteControlNavigator.prototype.isVisible = function (el) {
        var style = window.getComputedStyle(el);

        if(style.opacity <= 0)
            return false;
        if(style.display == 'none')
            return false;
        if(!el.offsetParent)
            return false;
        if(!el.offsetTop)
            return false;
        if(!el.offsetLeft)
            return false;

        return true;
    }

    _global.RemoteControlNavigator = RemoteControlNavigator;
})(this);

// var remoteControlNavigator = new RemoteControlNavigator()
// remoteControlNavigator.init({
//     selectors : document.querySelectorAll('.tabindex'),
//     cssClass: 'inputHighlight',

//     onSetInput: function(element){
//         console.log(element);
//         // if(element.type == 'number')
//         //     _g_ime.keySet = '12key';
//         // else
//         //     _g_ime.keySet = 'qwerty';
//     }
// });
