(function( _global ) {
    /**
     * Classe com as configurações do arquivo XML da LG
     *
     * @class XMLConfig
     * @constructor
     */
    function XMLConfig(xml) {
        //
        this.xml = xml;
    }

    /**
     * Retorna o valor de um dos itens do xml
     *
     * @method getValue
     * @param {String} nome da tag para buscar
     * @return {mix} Valor da tag ou false, se não encontrar
     */
    XMLConfig.prototype.getValue = function(tag) {
        var item = this.xml.querySelector("widget").querySelector(tag);
        if (item) {
            return item.textContent;
        } else {
            return false;
        }
    };

    /**
     * Classe com informações sobre o sistema da LG
     *
     * @class LgRuntime
     * @constructor
     */
    function LgRuntime() {
        var that = this;

        $.ajax({
            url: MAIN_PATH + 'config.xml',
            dataType: 'xml',
            success: function(res) {
                that.xmlConfig = new XMLConfig(res); // <widget>
            },
            error: function() {
                console.error("can't load the config.xml file!");
            }
        });
    }

    /**
     * Método que retorna a versão do APP
     *
     * @method getVersion
     * @return {String} a versão do APP
     */
    LgRuntime.prototype.getVersion = function() {
        if (this.xmlConfig) {
            return this.xmlConfig.getValue('version');
        }
        return '?';
    };

    /**
     * Método para retornar os modelos suportados pela versão do player
     * @return {string}
     */
    LgRuntime.prototype.getModel = function() {
        if (this.xmlConfig) {
            return this.xmlConfig.getValue('model');
        }
        return '?';
    };

    /**
     * Método para retornar a versão do WebOS suportado pelo player
     * @return {string}
     */
    LgRuntime.prototype.getVersionWebOS = function() {
        if (this.xmlConfig) {
            return this.xmlConfig.getValue('webos');
        }
        return '?';
    };

    /**
     * Registra o último erro acontecido no APP
     * @method setLastError
     * @param {string} error
     */
    LgRuntime.prototype.setLastError = function(error) {
        this.beforeLastError = this.lastError;
        this.lastError = error;
    };

    _global.LgRuntime = LgRuntime;
})(this);
