
/**
 * Orientação do player: 1 - portrait | 0 - landscape
 * @type {Number}
 */
window.OrientationPlayer = 0;

/**
 * Resolução para que o vídeo utilize para realizar as proporções necessárias
 * Resolução padrão é Full HD 1920X1080
 * @type {Integer}
 */
// window.ResolutionPlayer = window.innerHeight/2;
window.ResolutionPlayer = 1080;

(function() {

    /**
     * Objeto que representa as bordas de uma janela
     * @class BoundsSpecification
     */
    var BoundsSpecification = {
        /**
         * Largura da janela
         * @property width
         * @type int
         */
         width: 0,
         /**
         * Altura da janela
         * @property height
         * @type int
         */
         height: 0,
         /**
         * Distância do topo da tela
         * @property top
         * @type int
         */
         top: 0,
         /**
         * Distância da esquerda da tela
         * @property left
         * @type int
         */
         left: 0
    };

    /**
     * Objeto que representa as opções de criação da janela
     * @class WindowCreateOptions
     */
    var WindowCreateOptions = {
        /**
         * ID da janela
         * @property id
         * @type string
         */
        id: '',
        /**
         * Representa as bordas da janela
         * @property bounds
         * @type BoundsSpecification
         */
        bounds: BoundsSpecification,
        /**
         * Representa as bordas da janela
         * @property innerBounds
         * @type BoundsSpecification
         */
        innerBounds: BoundsSpecification,
    };

    /**
     * Função que vai verificar se nas opções foi passado bounds ou innerBounds e copiar tudo para bounds
     * @function unifyBounds
     * @param {WindowCreateOptions} options
     * @return {WindowCreateOptions} Objeto modificado, com bounds preenchido
     */
    function unifyBounds(options) {
        if (_.isMatch(options.innerBounds, {width: 0, height: 0, left: 0, top: 0})) {
            options.innerBounds = options.bounds;
        } else {
            options.bounds = options.innerBounds;
        }
        return options;
    }

    /**
     * "Fecha" uma janela aberta com LgWindowManager.
     * Vai ser chamada no close() injetado no window do frame.
     *
     * @function windowClose
     */



    /**
     * Classe para gerenciar as janelas usando a mesma API do chrome, mas criando as janelas como
     * iframe no html
     *
     * @class LgWindowManager
     * @constructor
     */
    function LgWindowManager(runtime) {
        /**
         * @property runtime
         * @type {player4uc.runtime}
         */
        this.runtime = runtime;
    }

    /**
     * Lista de janelas criadas com este gerenciador
     * @property windows
     * @static
     * @type Object
     */
    LgWindowManager.windows = {};

    /**
     * Representa a janela principal, já que ela não é um iframe
     * @property mainWindow
     * @static
     * @type {Object}
     */
    LgWindowManager.mainWindow = {
        id: "mainWindow",
        contentWindow: window,
        contentDocument: window.document
    };

    /**
     * Método para criar um novo iframe carregando a URL indicada e com as configurações
     * definidas no argumento options.
     * Após criar a janela, vai chamar o ```calback``` passando o objeto window do iframe/
     *
     * @param {string} url
     * @param {WindowCreateOptions} (optional)options
     * @param {Function} (optional)callback
     */
    LgWindowManager.prototype.create = function(url, options, callback) {
        // verifica se passou segundo argumento como sendo o callback
        if (typeof options == 'function') {
            callback = options;
            options = {};
        } else if (typeof options == 'undefined') {
            options = {};
        }

        if (!_.has(options, 'id')) {
            options['id'] = '$window-' + _.random(0,100) + '-' + _.random(0,100);
        }
        var defaultOptions = _.clone(WindowCreateOptions);
        options = _.extend(defaultOptions, options);

        options = unifyBounds(options);


        var frame = document.createElement('iframe');
        frame.borders = 0;
        frame.frameborder = 0;
        frame.allowtransparency = "true";
        frame.id = options.id;
        $(frame).css({
            position: 'absolute',
            // left: options.bounds.left + 'px',
            // top: options.bounds.top + 'px',
            top: 0,
            left: 0,
            bottom: 0,
            right: 0,
            "maring": 0,
            "padding": 0,
            width: options.bounds.width + 'px',
            height: options.bounds.height + 'px',
            border: '0px'
        });

        LgWindowManager.windows[frame.id] = frame;
        frame.src = url + "?" + new Date().getTime();

        // ajusta a orientação da tela
        this.orientation(frame);

        document.body.appendChild(frame);

        this.createWindowInterface(frame);

        callback(frame);
    };

    /**
     * Método para rotacionar o layout
     * @param  {integer} option  0 - para Horizotal, e 1 para vertical
     * @return {void}
     */
    LgWindowManager.prototype.orientation200 = function __doneOrientasion(element, rotation) {
        var frame = $(element);
        var widthFrame = frame.css('width');

        var options = {
            portraitMode: Signage.OsdPortraitMode.OFF
        };

        var successCb = function (){
            console.debug("Portrait Mode successfully Set");
        };

        var failureCb = function(cbObject){
            var errorCode = cbObject.errorCode;
            var errorText = cbObject.errorText;
            console.debug( " Error Code [" + errorCode + "]: " + errorText);
        };

        var signage = new Signage();

        if (rotation) {
            options = {
                portraitMode: Signage.OsdPortraitMode.ON
            };
        }

        // troca a orientação
        signage.setPortraitMode(successCb, failureCb, options);

        // redimenciona o conteúdo
        if (frame[0].tagName == "IFRAME" && 'contentWindow' in frame[0] && frame[0].contentWindow !== null && frame[0].contentWindow.ClassLayout instanceof ClassLayout) {
            var __ClassLayout = frame[0].contentWindow.ClassLayout;
            var areaAux = __ClassLayout.getAreasAux() || [];
            var AllArea = areaAux.push(__ClassLayout.getAreaMain());
            resizeAll(AllArea);
        }
    };

    /**
     * Utilizando transforme para gira a tela
     * @param  {ElementHtml} element
     * @param  {boolean} rotation [description]
     */
    LgWindowManager.prototype.orientation = function __doneOrientasion(element, rotation) {
        var frame = $(element);
        var widthFrame = frame.css('width');

        var width  = window.innerWidth;
        var height = window.innerHeight;

        if (rotation) {
            height = window.innerWidth;
            width  = window.innerHeight;
        }

        // adicionar css
        frame.css('width', width + 'px');
        frame.css('height', height + 'px');
        frame.css('background-color', "transparent");

        // caso seja para rotacionar o conteúdo
        if (rotation) {
            frame.css("-webkit-transform", "rotate(270deg) translateX(-100%)");
            frame.css("-webkit-transform-origin", "top left");
            frame.css("transform-origin", "top left");
            frame.css("transform", "rotate(270deg) translateX(-100%)");

            // seta a nova altura da tela
            window.ResolutionPlayer = width;
        }
        else if (parseInt(widthFrame) !== window.innerWidth && frame.css("transform") != "") {
            frame.css("-webkit-transform", "rotate(0deg) translateX(0%)");
            frame.css("-webkit-transform-origin", "top left");
            frame.css("transform-origin", "top left");
            frame.css("transform", "rotate(0deg) translateX(0%)");

            // seta a nova altura da tela
            window.ResolutionPlayer = height;
        }


        // redimenciona o conteúdo
        if (frame[0].tagName == "IFRAME" && 'contentWindow' in frame[0] && frame[0].contentWindow !== null && frame[0].contentWindow.ClassLayout instanceof ClassLayout) {
            var __ClassLayout = frame[0].contentWindow.ClassLayout;

            if (__ClassLayout.getAreasAux()) {
                var AllArea = __ClassLayout.getAreasAux().push(__ClassLayout.getAreaMain());
                resizeAll(AllArea);
            }
        }
    };

    /**
     * Busca uma das janelas criadas
     * @method get
     * @param {string} windowId
     * @return {DOMElementIframe|Boolean}
     */
    LgWindowManager.prototype.get = function(windowId) {
        if (windowId == 'mainWindow') {
            return LgWindowManager.mainWindow;
        }
        if (_.has(LgWindowManager.windows, windowId)) {
            return LgWindowManager.windows[windowId];
        } else {
            runtime.setLastError("Can't find the window " + windowId);
            return false;
        }
    };

    /**
     * Busca todas as janelas criadas
     * @method getAll
     * @param {string}
     * @return {Array} Array of DOMElementIframe
     */
    LgWindowManager.prototype.getAll = function() {
        return LgWindowManager.windows;
    };

    /**
     * Pega a janela atual. Considera que a atual é a última aberta
     * @method current()
     * @return {DOMElementIframe}
     */
    LgWindowManager.prototype.current = function() {
        var idLast = _.last(_.keys(LgWindowManager.windows));
        if (!idLast) {
            return this.get('mainWindow');
        }
        return this.get(idLast);
    };

    /**
     * Cria as funções no iframe conforme as que já existiam no chrome, como close, etc
     * @method createWindowInterface
     * @param {DOMElementIFrame} frame
     * @private
     */
    LgWindowManager.prototype.createWindowInterface = function(frame) {
        // TODO: verificar se a forma como foi criado o método close() não vai gerar memory leaks
        frame.close = function() {

            var id = this.id;

            if (id !== "mainWindow") {

                this.contentWindow.parent.document.body.removeChild(this);
                this.remove();
                delete LgWindowManager.windows[id];
                frame = null;
            }
        };

        frame.contentWindow.player4uc = window.player4uc;
    };

    window.LgWindowManager = LgWindowManager;
})();
