(function(_global) {
    'use strict';

    /**
     * Classe para gerenciar Downloads
     *
     * @class Downloader
     * @constructor
     * @param {String} projPath Caminho do projeto
     */
    function Downloader(options, callback) {
        if(options)
            this.init(options);
    };

    Downloader.prototype.init = function(options, callback){

        if (!('downloaderPlugin' in options) || !('sefPlugin' in options)) {
            return;
        }

        this.DOWNLOADER_PLUGIN = options.downloaderPlugin;
        this.countPluginDownloader = this.DOWNLOADER_PLUGIN.length;

        this.SEF_PLUGIN = options.sefPlugin;
    };

    Downloader.prototype.start = function start(options, callback) {

        var TO_PATH,
            FROM_URL,
            filename,
            that = this ;

        var DOWNLOADER_PLUGIN_CURRENT = this.DOWNLOADER_PLUGIN.shift();

        TO_PATH = normalizePath(options.path);
        FROM_URL = options.url;

        filename = extractFileFromPath(TO_PATH);
        if(!filename) {
            filename = extractFileFromPath(FROM_URL);
            TO_PATH = TO_PATH + '/' + filename.split("?")[0];
        }

        this.onComplete = options.onComplete || function(){};

        // logs
        // _log({
        //     'Class': 'Downloader',
        //     'DOWNLOADER_PLUGIN_CURRENT': typeof DOWNLOADER_PLUGIN_CURRENT,
        //     "FROM_URL": FROM_URL,
        //     "TO_PATH": TO_PATH,
        //     'player4uc.download.DOWNLOADER_PLUGIN': player4uc.download.DOWNLOADER_PLUGIN.length,
        //     'player4uc.download.countPluginDownloader': player4uc.download.countPluginDownloader,
        // });

        // caso não tenha plugin
        if(!DOWNLOADER_PLUGIN_CURRENT)
           return callback({message: 'Download require sefPlugin'});

        if(!FROM_URL || !TO_PATH)
            return callback({message: 'invalid arguments. Required {url:"", path: ""}'});

        try {
            // registrando eventos
            DOWNLOADER_PLUGIN_CURRENT.OnComplete = function (param) {
                var strList = param.split('?');

                _log({
                    'Class': 'Downloader',
                    'strList': strList,
                    'filename': filename,
                });

                if (strList[0] == '1000' && strList[1] == '1') {
                    // subtrai os downloads
                    that.DOWNLOADER_PLUGIN.push(DOWNLOADER_PLUGIN_CURRENT);

                    callback({
                        error: null,
                        message: ''
                    }, true);
                }

                // em caso de erros
                if (strList[0] == '1000' && strList[1] != '1') {
                    // subtrai os downloads
                    that.DOWNLOADER_PLUGIN.push(DOWNLOADER_PLUGIN_CURRENT);

                    callback({
                        error: error,
                        message: 'Internal error'
                    }, false);
                }
            };

            var downloaded = DOWNLOADER_PLUGIN_CURRENT.StartDownFile(FROM_URL, TO_PATH);
        } catch(error) {
            return callback({
                error: error,
                message: 'Internal error'
            });
        }


         /**
         * Convert path string to correct format
         * @param  {String} pathString [description]
         * @example            normalizePath('/mtd_down')          /mtd_down
         * @example            normalizePath('/mtd_down/common')   /mtd_down/common
         * @example            normalizePath('/mtd_down/anyone')   /mtd_down/anyone
         * @example            normalizePath('/test')              /mtd_down/common/test
         * @return {String}
         */
        function normalizePath (pathString) {

            pathString = (pathString[0] == '/')    ?    pathString    :    '/' + pathString;
            pathString = (pathString[pathString.length -1] == '/')    ?    pathString.substring(0, pathString.length-1)    :    pathString;

            if(!pathString.match('/mtd_down'))
                pathString = pathString.match('/mtd_down/common') ? pathString : '/mtd_down/common' + pathString;


            return pathString;
        }
        /**
         * Extract file from path or URL
         * @return {String} file.txt
         */
        function extractFileFromPath(pathString) {
            var file;

            if(typeof pathString != 'string')
                return '';

            pathString = pathString.split('/');
            file = pathString[pathString.length-1];

            if(file.indexOf('.') > -1)
                return file;
            else
                return '';
        }
    }

    _global.Downloader = Downloader;
})(this);


/**
 *
 *    var downloader =  new Downloader({
 *        downloaderPlugin : document.querySelector('#DownloaderPlugin')
 *    });
 *
 *    downloader.start({
 *        url: 'http://example.com/file.zip',
 *        path: '/mtd_down/common/test',
 *    }, function(error, data){
 *        console.log('complete', error, data)
 *    });
 *
 **/
