(function(_global) {

    /**
     * Classe to manipulate filesystem
     *
     * @class Filesystem
     * @constructor
     * @param {String} projPath Caminho do projeto
     */
    function Filesystem(options, callback) {
        this.ROOT = 'file://internal/4yousee/';
        if(options)
            this.init(options, callback);
    }

    Filesystem.prototype.init = function init(options, callback) {

            if(callback)
                callback(null, {success: true});
    }

    /**
     * Creates a directory.
     *
     * @method mkdir
     * @param {string} path
     * @param {object} options || callback
     * @param {function} callback
     */
    Filesystem.prototype.mkdir = function (path, options, callback) {
        if(callback)
            callback(null, {success: true});
    }

    /**
     * Remove a directory.
     *
     * @method rmdir
     * @param {string} path
     * @param {object} options || callback
     * @param {function} callback
     */
    Filesystem.prototype.rmdir = function (path, options, callback) {
        if(callback)
            callback(null, {success: true});
    }

    /**
     * Remove a file.
     *
     * @method rm
     * @param {string} filePath
     * @param {object} options
     * @param {function} callback
     */
    Filesystem.prototype.rm = function (filePath, options, callback) {
        if(callback)
            callback(null, {success: true});
    }

    /**
     * Read a file like text.
     *
     * @method readFile
     * @param {string} filePath
     * @param {function} callback
     */
    Filesystem.prototype.readFile = function (filePath, callback) {
        if(callback)
            callback(null, {success: true});
    }

    /**
     * Write a file.
     *
     * @method writeFile
     * @param {string} path
     * @param {string} data
     * @param {string} fileType
     * @param {object} options
     * @param {function} callback
     */
    Filesystem.prototype.writeFile = function(filePath, data, fileType, options, callback){
        if(callback)
            callback(null, {success: true});
    }

   /**
     * Rename file.
     *
     * @method renameFile
     * @param {string} filePath
     * @param {string} newName
     * @param {function} callback
     */
    Filesystem.prototype.renameFile = function(filePath, newFilePath, callback) {
        if(callback)
            callback(null, {success: true});
    }

    /**
     * List all files that exists in the specified path.
     *
     * @param  {string}   path     path to list
     * @param  {Function} callback Callback function
     * @return {array}             Outputs an array of objects
     */
    Filesystem.prototype.ls = function ls(path, callback) {
        if(callback)
            callback(null, {success: true});
    }

    /**
     * Convert path string to correct format
     * @param  {String} pathString [description]
     * @example            normalizePath('/mtd_down')          /mtd_down
     * @example            normalizePath('/mtd_down/common')   /mtd_down/common
     * @example            normalizePath('/mtd_down/anyone')   /mtd_down/anyone
     * @example            normalizePath('/test')              /mtd_down/common/test
     * @return {String}
     */
    function normalizePath (pathString) {

        pathString = (pathString[0] == '/')    ?    pathString    :    '/' + pathString;
        pathString = (pathString[pathString.length -1] == '/')    ?    pathString.substring(0, pathString.length-1)    :    pathString;

        if(!pathString.match('/mtd_down'))
            pathString = pathString.match('/mtd_down/common') ? pathString : '/mtd_down/common' + pathString;


        return pathString;
    }

    /**
     * Extract file from path or URL
     * @return {String} file.txt
     */
    function extractFileFromPath(pathString) {
        var file;

        if(typeof pathString != 'string')
            return '';

        pathString = pathString.split('/');
        file = pathString[pathString.length-1];

        if(file.indexOf('.') > -1)
            return file;
        else
            return '';
    }

    _global.Filesystem = Filesystem;
})(this);


/**
 *
 *  var fs = new Filesystem();
 *      fs.init(function(){
 *          fs.ls('/', function(){});
 *       });
 *
 **/
