(function( _global ) {
    /**
     * Classe com as configurações do arquivo XML da Samsung
     *
     * @class XMLConfig
     * @constructor
     */
    function XMLConfig(xml) {
        //
        this.xml = xml;
    }

    /**
     * Retorna o valor de um dos itens do xml
     *
     * @method getValue
     * @param {String} nome da tag para buscar
     * @return {mix} Valor da tag ou false, se não encontrar
     */
    XMLConfig.prototype.getValue = function(tag) {
        var item = this.xml.getElementsByTagName(tag)[0];
        if (item) {
            return item.innerHTML;
        } else {
            return false;
        }
    };

    /**
     * Classe com informações sobre o sistema da samsung
     *
     * @class SamsungRuntime
     * @constructor
     */
    function SamsungRuntime() {
        var that = this;

        $.ajax({
            url: MAIN_PATH + 'config.xml',
            dataType: 'xml',
            success: function(res) {
                that.xmlConfig = new XMLConfig(res); // <widget>
            },
            error: function() {
                console.error("can't load the config.xml file!");
            }
        });
    }

    /**
     * Método que retorna a versão do APP
     *
     * @method getVersion
     * @return {String} a versão do APP
     */
    SamsungRuntime.prototype.getVersion = function() {
        if (this.xmlConfig) {
            return this.xmlConfig.getValue('ver');
        }
        return '?';
    };

    /**
     * Registra o último erro acontecido no APP
     * @method setLastError
     * @param {string} error
     */
    SamsungRuntime.prototype.setLastError = function(error) {
        this.beforeLastError = this.lastError;
        this.lastError = error;
    };

    _global.SamsungRuntime = SamsungRuntime;
})(this);
