'use strict';

(function(window) {
    var ctx, localMediaStream, canvas, video, timeout_snapshot;
    var local_storage_TAG = 'settings.idMediaScreen';
    /**
     * Classe para enviar o screenShot do player
     *
     * @class ScreenShot
     * @param {SocketIO} socket
     */
    function ScreenShot(socket) {
        // Adiciona o evento para ouvir o socket
        listenerScreenShot(socket);
    }

    /**
     * Método para adicionar o evento que escuta o pedido de screenshot
     *
     * @method listenerScreenShot
     * @param  {SocketIO} socket
     * @return {void}
     */
    function listenerScreenShot(socket) {
        // ENvia a imagem para socket
        socket.on('player_command', function player_command(data){

            if (!('type' in data) && !(data.type == 'SCREENSHOT')) {
                return;
            }

            _log('listenerScreenShot: player_command', data);

            // gerar a imagem
            snapshot(function __done(image) {
                var result = 'SUCCESS';

                if (_.isUndefined(image.ArrayBuffer)) {
                    result = 'FAIL';
                }

                // envia a resposta
                socket.emit('player_command_result', {
                    content: image.ArrayBuffer,
                    result: result,
                    type: 'SCREENSHOT'
                });

                _log('Imagem enviada', result);
            });

        });
    }

    /**
     * Método para capturar a imagem
     *
     * @method snapshot
     * @return {string} Imagem
     */
    function snapshot(callback) {
        var options = {};
        options.save = true;
        options.thumbnail = false;

        var successCB = function(cbObject){
            var size = cbObject.size;
            var encoding = cbObject.encoding;
            var data = cbObject.data;

            // callback
            callback({
                "base64": data,
                "ArrayBuffer": base64ToArrayBuffer(data),
            });

            // console.debug("Got Data size:" + size);
            // console.debug("Got Data encoding :" + encoding);
            // console.debug("Got Data :" + data);
        };

        var failureCB = function(cbObject){
            var errorCode = cbObject.errorCode;
            var errorText = cbObject.errorText;

            _logError("Error Code [" + errorCode + "]: " + errorText);
        }

        var signage = new Signage();
        signage.captureScreen(successCB, failureCB, options);

        /**
         * Método para converter da base64 para ArrayBuffer
         *
         * @method base64ToArrayBuffer
         * @param  {string} base64
         * @return {object}
         */
        function base64ToArrayBuffer(base64) {
            var binary_string =  window.atob(base64);
            var len = binary_string.length;
            var bytes = new Uint8Array( len );
            for (var i = 0; i < len; i++)        {
                bytes[i] = binary_string.charCodeAt(i);
            }
            return bytes.buffer;
        }
    }

    window.ClassScreenShot = ScreenShot;

})(window);
